/**
 * \file gre/dlgimagecorrelation.h
 * \brief GRE::UI::DLG_IMAGE_CORRELATION definitions
 *
 * \if NODOC
 * $Id: dlgimagecorrelation.h_v 1.2 2015/03/03 16:34:11 mju Exp $
 *
 * $Log: dlgimagecorrelation.h_v $
 * Revision 1.2  2015/03/03 16:34:11  mju
 * Keep track of regression separately for X vs Y dependent.
 *
 * Revision 1.1  2014/01/23 20:20:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_DLGIMAGECORRELATION_H
#define	INC_GRE_DLGIMAGECORRELATION_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_TOOLCANVAS_H
#include <mgui/toolcanvas.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_MENU_H
#include <mgui/menu.h>
#endif

#ifndef	INC_MGUI_STATUSBAR_H
#include <mgui/statusbar.h>
#endif

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MGUI_TOOLPOLYLINE_H
#include <mgui/toolpolyline.h>
#endif

#ifndef	INC_MGUI_TOOLSELECTPOINT_H
#include <mgui/toolselectpoint.h>
#endif

#ifndef	INC_MGUI_TOOLSEGMENT_H
#include <mgui/toolsegment.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif

#ifndef	INC_RVC_DBRAST_H
#include <rvc/dbrast.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
#include <mi32/contrastparm.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
#include <mi32/colorarray.h>
#endif

#ifndef	INC_MI32_STATMULTIVAR_H
#include <mi32/statmultivar.h>
#endif

#ifndef	INC_MI32_STATUNIVAR_H
#include <mi32/statunivar.h>
#endif

#ifndef	INC_Mi32_TEXTFORMAT_H
#include <mi32/textformat.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FILE_TEXT;
namespace GRE {
	class GROUP;
	class LAYER_IMAGE_SPATMOD;
	class MSG;
	class VIEW;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_AUTOPTR;
		}
	}
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

//! Dialog for image band correlation display.
class DLG_IMAGE_CORRELATION : public MGUI::DLGSHELL {
	public:

		enum DLGSTYLE {
			DLGSTYLE_Default =			0,
			DLGSTYLE_NoChgRasters =		0x01,		//!< Disallow changing selected rasters
			DLGSTYLE_NoChgMask =			0x02,		//!< Disallow changing mask
			};

		//! Default constructor.
		explicit DLG_IMAGE_CORRELATION (
			DLGSTYLE dlgstyle = DLGSTYLE_Default
			);

		//! Destructor.
		~DLG_IMAGE_CORRELATION ();

		//! Set group to use for pointer track highlighting.
		void SetGroupHighlight (
			GRE::GROUP *group
			);

		ERRVALUE SetObjItemList (
			const RVC::OBJITEMLIST& ObjItemList,
			const RVC::OBJITEM *pObjItemMask = 0
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Public so can define enum op bitwise.
		class CANVASAREA;
		class CANVASAREA_HISTOGRAM;
		class CANVASAREA_CORRELATION;

	private:
		typedef DLG_IMAGE_CORRELATION THISCLASS;

		static const UINT32 MAXTRACKRADIUS = 9;

		enum CMDID {
			CMDID_File_New =						1,
			CMDID_File_ClearHighlight =		5,
			CMDID_View_Full =						11,
			CMDID_View_MaxDetail =				12,
			CMDID_View_ZoomIn =					13,
			CMDID_View_ZoomOut =					14,
			CMDID_Option_ShowRegLine =			21,
			CMDID_Option_ShowHist =				22,
			CMDID_Option_ScaleHistLinear =	23,
			CMDID_Option_ScaleHistLog =		24,
			CMDID_Option_Colors =				25,
			CMDID_Option_Palette =				26,
			CMDID_Option_Font =					27,
			CMDID_Save_CorrelationAsRaster = 31,
			CMDID_Save_HighlightAsRaster = 	32,
			CMDID_Save_CorrelationAsText =	33,
			CMDID_Save_StatisticsAsText =		34,
			CMDID_Help
			};

		enum TOOL {
			TOOL_Drag,
			TOOL_Line,
			TOOL_HighlightRange,
			};

		enum SCALETYPE {
			SCALETYPE_Linear,
			SCALETYPE_Logarithmic
			};

		class COMPONENT {
			public:
				explicit COMPONENT (INT32 VarNum = 0);
				~COMPONENT ();

				INT32 CalcCorrBinFromGraphPos (
					INT32 GraphPos
					) const { return (FAST_FLOOR(m_CorrViewBinMin + GraphPos / m_CorrViewZoom)); }
				INT32 CalcGraphPosFromCorrBin (
					INT32 BinIdx
					) const { return ((BinIdx - m_CorrViewBinMin) * (m_GraphUsedMax - m_GraphUsedMin + 1) / GetCorrViewNumBins()); }
				double CalcGraphPosFromValue (
					double value
					) const;
				double CalcNewCorrViewZoom (
					double z,
					bool isabsolute
					) const;
				INT32 CalcStatBinFromGraphPos (
					INT32 GraphPos
					) const { return (FAST_FLOOR((m_CorrViewBinMin + GraphPos / m_CorrViewZoom) * m_CorrNumStatBinsCombined)); }
				ERRVALUE Create (
					const RVC::OBJITEM& ObjItem,
					RVC::RASTER::COMPONENT RasterComponent = RVC::RASTER::COMPONENT_Default
					);
				bool GetCanShowFieldValue (
					) const { return (m_CanShowFieldValue); }
				double GetCorrBinWidth (
					) const { return (m_CorrBinWidth); }
				INT32 GetCorrNumBins (
					) const { return (m_CorrNumBins); }
				INT32 GetCorrNumStatBinsCombined (
					) const { return (m_CorrNumStatBinsCombined); }
				INT32 GetCorrViewBinEnd (
					) const { return (m_CorrViewBinEnd); }
				INT32 GetCorrViewBinMin (
					) const { return (m_CorrViewBinMin); }
				INT32 GetCorrViewNumBins (
					) const { return (GetCorrViewBinEnd() - GetCorrViewBinMin()); }
				double GetCorrViewZoom (
					) const { return (m_CorrViewZoom); }
				double GetCorrViewZoomMax (
					) const { return (m_CorrViewZoomMax); }
				double GetCorrViewZoomMin (
					) const { return (m_CorrViewZoomMin); }
				MISTRING GetDisplayStrFromCorrBinIdx (
					INT32 CorrBinIdxMin,
					INT32 CorrBinIdxMax
					) const;
				MISTRING GetDisplayStrFromStatBinIdx (
					INT32 StatBinIdxMin,
					INT32 StatBinIdxMax
					) const;
				INT32 GetGraphSpace (
					) const { return (m_GraphSpace); }
				INT32 GetGraphUsedMax (
					) const { return (m_GraphUsedMax); }
				INT32 GetGraphUsedMin (
					) const { return (m_GraphUsedMin); }
				INT32 GetHistViewBinEnd (
					) const;
				INT32 GetHistViewBinMin (
					) const { return (static_cast<INT32>(m_CorrViewBinMin) * m_CorrNumStatBinsCombined); }
				INT32 GetHistViewNumBins (
					) const { return (GetHistViewBinEnd() - GetHistViewBinMin()); }
				MISTRING GetName (
					) const;
				INT32 GetNameLabelWidth (
					MGD::CONTEXT *gc
					) const;
				RVC::RASTER& GetObjRaster (
					) { return (m_ObjRaster); }
				const DOUBLE_RANGE& GetStatBinRange (
					) const { return (m_StatsFull.GetBinRange()); }
				MISTRING GetStatStr (
					) const;
				const STAT_UNIVARIATE& GetStatsFull (
					) const { return (m_StatsFull); }
				const STAT_UNIVARIATE& GetStatsUsed (
					) const { return (m_StatsUsed); }
				STAT_UNIVARIATE& GetStatsUsed (
					) { return (m_StatsUsed); }
				const DOUBLE_ARRAY<double>& GetTickValues (
					) const { return (m_TickValues); }
				double* GetTrackReadBuf (
					) { return (m_TrackReadBuf); }
				double GetValueFromCorrBinIdx (
					INT32 CorrBinIdx
					) const;
				double GetValueFromStatBinIdx (
					INT32 StatBinIdx
					) const { return (m_StatsFull.ComputeBinCenter(StatBinIdx)); }
				INT32 GetVarNum (
					) const { return (m_VarNum); }
				bool HasNull () const
					{ return (m_HasNull); }
				bool IsInteger () const
					{ return ((m_ObjRaster.GetCellType() & (RVC::IMAGE::CELLTYPE_Integer | RVC::IMAGE::CELLTYPE_Color)) != 0); }
				ERRVALUE Read (
					INT32 row,
					INT32 col,
					INT32 len,
					double *valuebuf,
					UINT8 *maskbuf
					);
				void SaveAsText (
					FILE_TEXT& file
					) const;
				void SetCorrViewBinCenter (
					INT32 CorrViewBinCenter,
					double z = 1.0,
					bool isabsolute = false
					);
				bool SetCorrViewZoom (
					double z,
					bool isabsolute
					);
				void SetCorrViewZoomMin (
					double CorrViewZoomMin
					);
				void SetGraphSpace (
					INT32 GraphSpace
					) { m_GraphSpace = GraphSpace; m_GraphUsedMin = 0; m_GraphUsedMax = GraphSpace - 1; }
				ERRVALUE SetMaxReadColumns (
					INT32 maxcols
					);
				void TrackPosAdd (
					double value
					);
				void TrackPosClear (
					) { m_TrackPos.Clear(); }
				INT32 TrackPosGetBinIdx (
					int index
					) const { return (m_TrackPos[index]); }
				INT32 TrackPosGetNumItems (
					) const { return (m_TrackPos.GetNumItems()); }
			private:
				INT32 m_VarNum;
				RVC::RASTER m_ObjRaster;
				RVC::NULLMASK m_ObjMask;
				RVC::RASTER::COMPONENT m_RasterComponent;
				STAT_UNIVARIATE m_StatsFull;
				STAT_UNIVARIATE m_StatsUsed;
				INT32 m_CorrNumBins;
				INT32 m_CorrNumStatBinsCombined;
				INT32 m_CorrViewBinMin;
				INT32 m_CorrViewBinEnd;
				double m_CorrViewZoom;				// Pixels per correlation bin
				double m_CorrViewZoomMin;
				double m_CorrViewZoomMax;
				double m_CorrBinWidth;
				INT32 m_GraphSpace;
				INT32 m_GraphUsedMin;
				INT32 m_GraphUsedMax;
				RVC::DBASE_RASTER m_ObjDb;
				RVC::DBTABLE m_ObjTable;
				mutable RVC::DBTABLE::RECORD m_Record;
				bool m_HasNull;
				bool m_CanShowFieldValue;
				bool m_ShowFieldValue;
				int m_FieldNum;
				MISTRING m_FieldName;
				int m_DisplaySize;					// Size of display area for component
				double m_DisplayMin;					// Component value corresponding to start of display area
				double m_DisplayScale;				// Scale from component value to display position
				SIMPLE_ARRAY<INT32> m_TrackPos;
				double m_TrackReadBuf[MAXTRACKRADIUS*2+1];
				mutable INT32 m_NameLabelWidth;
				DOUBLE_ARRAY<double> m_TickValues;

				void CalcTickValues ();
			};
		typedef MILIST<COMPONENT> COMPONENTLIST;

		class CORRELATION {
			public:
				CORRELATION (
					const COMPONENT& ComponentX,
					const COMPONENT& ComponentY,
					const STAT_MULTIVAR& StatMulti
					);

				~CORRELATION ();

				void Accumulate (
					UINT32 NumValues,
					const double *pValuesX,
					const double *pValuesY,
					const UINT8 *pValidity
					);

				void ClearCounts ();

				ERRVALUE Finalize ();

				COMPONENT& GetComponentX (
					) const { return (const_cast<COMPONENT&>(m_ComponentX)); }

				COMPONENT& GetComponentY (
					) const { return (const_cast<COMPONENT&>(m_ComponentY)); }

				const CONTRASTPARM& GetContrastParm (
					) const { return (m_ContrastParm); }

				const SIMPLE_ARRAY<UINT32>& GetCount () const
					{ return (m_Count); }

				MISTRING GetDisplayStrFromBinIdx (
					INT32 BinIdxMinX,
					INT32 BinIdxMinY,
					INT32 BinIdxMaxX,
					INT32 BinIdxMaxY
					) const;

				int GetIndex (
					double ValueX,
					double ValueY
					) const;

				void GetObjItemSaveAs (
					RVC::OBJITEM& ObjItem
					) const;

				bool GetRegLine (
					bool bXdep,
					POLYLINE& line
					) const;

				void GetStatStr (
					MISTRING& str,
					int indent,
					TEXTFORMAT_LINE2D::EQUATIONFORM LineEqForm
					) const;

				ERRVALUE SaveAsRaster (
					RVC::OBJITEM& ObjItem,
					const COLOR_ARRAY& ColorPalette
					) const;

				void SaveAsText (
					FILE_TEXT& file,
					TEXTFORMAT_LINE2D::EQUATIONFORM LineEqForm,
					bool IncludeCounts
					) const;

			private:
				const COMPONENT& m_ComponentX;
				const COMPONENT& m_ComponentY;
				const STAT_MULTIVAR& m_StatMulti;
				LPOINT2D m_BinIdxOfMode;
				LRECT2D m_BinRangeNonEmpty;
				UINT64 m_TotalCount;
				UINT32 m_MaxCount;
				SIMPLE_ARRAY<UINT32> m_Count;
				CONTRASTPARM m_ContrastParm;
				DPOINT2D m_PtRegLineXdep1;
				DPOINT2D m_PtRegLineXdep2;
				DPOINT2D m_PtRegLineYdep1;
				DPOINT2D m_PtRegLineYdep2;
				bool m_HaveRegLineXdep;
				bool m_HaveRegLineYdep;
			};
		typedef MILIST<CORRELATION> CORRELATIONLIST;

		typedef MILIST<CANVASAREA*> CANVASAREAPTRLIST;

		// Base for highlight tools.
		class TOOL_HIGHLIGHT : public MGUI::TOOL_BASE {
			public:
				// Constructor.
				TOOL_HIGHLIGHT (
					MGUI::TOOLCANVAS *canvas,
					CANVASAREAPTRLIST& CanvasAreaPtrList
					);

				// Destructor.
				virtual ~TOOL_HIGHLIGHT ();

				void SetNumComponents (
					int NumComponents
					) { m_NumComponents = NumComponents; v_SetNumComponents(NumComponents); }

			protected:
				CANVASAREAPTRLIST& m_CanvasAreaPtrList;
				int m_NumComponents;

			private:

				virtual void v_SetNumComponents (int NumComponents) = 0;
			};

		// Tool to highlight by range.
		class TOOL_HIGHLIGHT_RANGE : public TOOL_HIGHLIGHT {
			public:

				// Constructor.
				TOOL_HIGHLIGHT_RANGE (
					MGUI::TOOLCANVAS *canvas,
					CANVASAREAPTRLIST& CanvasAreaPtrList
					);

				// Destructor.
				virtual ~TOOL_HIGHLIGHT_RANGE ();

				const DOUBLE_RANGE* GetValueRangeArray (
					) const { return (m_ValueRangeArray); }

			private:
				// Enumeration constants
				enum ZONE {
					ZONE_NoFunction =	0,
					ZONE_CreateX =		0x01,		// Only combine with CreateY
					ZONE_InsideX =		0x02,		// Only combine with InsideY
					ZONE_NearMinX =	0x04,		// Only combine with NearMin/MaxY
					ZONE_NearMaxX =	0x08,		// Only combine with NearMin/MaxY
					ZONE_MaskX =		0x0F,
					ZONE_CreateY =		0x10,
					ZONE_InsideY =		0x20,
					ZONE_NearMinY =	0x40,
					ZONE_NearMaxY =	0x80,
					ZONE_MaskY =		0xF0,
					};

				DOUBLE_RANGE *m_ValueRangeArray;
				CANVASAREA *m_pCanvasArea;
				UINT16 m_PointerZone;					// Current cursor zone
				MGUI::POINT m_prevpt;					// Previous point during button-down

				void AdjustTool (const MGUI::POINT& ptCanvas, bool firsttime=false);
				void ComputePointerZone (const MGUI::POINT& ptCanvas);
				void UpdateCursor () const;

				// TOOL_BASE overrides.
				virtual void v_DoDraw ();
				virtual void v_OnClear ();
				virtual bool v_OnKey (MGUI::POINT point, MGUI::KEYCODE keycode, MGUI::KEYSTATE keystate);
				virtual bool v_OnPointerButtonDown (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
				virtual void v_OnPointerMove (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);

				// TOOL_HIGHLIGHT overrides.
				virtual void v_SetNumComponents (int NumComponents);
			};

		// Tool to highlight by range.
		class TOOL_LINE : public MGUI::TOOL_BASE {
			public:

				class SEGMENT {
					public:
						SEGMENT ();
						SEGMENT (const CANVASAREA_CORRELATION *pCanvasArea);
						~SEGMENT ();
						UINT32 ComputePointerZone (
							const MGUI::POINT& ptCanvas
							) const;
						void Draw (MGD::CONTEXT *gc) const;
						const CANVASAREA_CORRELATION* GetCanvasArea (
							) const { return (m_pCanvasArea); }
						const COMPONENT& GetComponentX (
							) const;
						const COMPONENT& GetComponentY (
							) const;
						const DPOINT2D& GetPointEnd (
							) const { return (m_PtEnd); }
						const DPOINT2D& GetPointStart (
							) const { return (m_PtStart); }
						bool IsActive (
							) const { return (m_IsActive); }
						void SetActive(
							bool IsActive
							) { m_IsActive = IsActive; }
						void SetPointEnd (
							const DPOINT2D& PtEnd
							) { m_PtEnd = PtEnd; }
						void SetPointStart (
							const DPOINT2D& PtStart
							) { m_PtStart = PtStart; }
						void Shift (
							double dx,
							double dy
							) { m_PtStart.Shift(dx,dy); m_PtEnd.Shift(dx,dy); }
					private:
						const CANVASAREA_CORRELATION *m_pCanvasArea;
						DPOINT2D m_PtStart;
						DPOINT2D m_PtEnd;
						bool m_IsActive;
					};
				typedef MILIST<SEGMENT> SEGMENTLIST;

				// Constructor.
				TOOL_LINE (
					MGUI::TOOLCANVAS *canvas,
					CANVASAREAPTRLIST& CanvasAreaPtrList
					);

				// Destructor.
				virtual ~TOOL_LINE ();

				const SEGMENTLIST& GetSegmentList (
					) const { return (m_SegmentList); }
				
			private:
				// Enumeration constants
				enum ZONE {
					ZONE_NoFunction =	0,
					ZONE_Create,
					ZONE_Start,
					ZONE_End,
					ZONE_Drag,
					};

				CANVASAREAPTRLIST& m_CanvasAreaPtrList;
				SEGMENTLIST m_SegmentList;
				const CANVASAREA_CORRELATION *m_pPointerCanvasArea;
				SEGMENT *m_pPointerSegment;
				UINT32 m_PointerZone;						// Current pointer zone
				MGUI::POINT m_prevpt;					// Previous point during button-down

				void AdjustTool (MGUI::POINT ptCanvas, bool firsttime=false);
				void ComputePointerZone (MGUI::POINT ptCanvas);
				void UpdateCursor () const;

				// TOOL_BASE overrides.
				virtual void v_DoDraw ();
				virtual void v_OnClear ();
				virtual bool v_OnKey (MGUI::POINT point, MGUI::KEYCODE keycode, MGUI::KEYSTATE keystate);
				virtual bool v_OnPointerButtonDown (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
				virtual void v_OnPointerMove (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
			};

		class CALCULATOR;

		MGUI::MENU m_MenuSave;
		MGUI::CTRL_TOOLBAR m_ToolBar;
		MGUI::FORM_BUTTONTEXT m_FormSelectMask;
		MGUI::TOOLCANVAS m_Canvas;
		MGUI::CTRL_TEXT m_TextStats;
		mutable MGUI::CTRL_STATUSBAR m_StatusBar;
		MGUI::TOOL_SELECTPOINT m_ToolDrag;
		TOOL_LINE m_ToolLine;
		TOOL_HIGHLIGHT_RANGE m_ToolHighlightRange;
		MGUI::TOOL_POLYLINE m_ToolHighlightPolygon;
		MGUI::FORM_COMBOBOX m_CbxSaveHighlightOutput;
		LPOINT2D m_CanvasSize;
		COLOR m_ColorBackground;
		COLOR m_ColorFrame;
		COLOR m_ColorTick;
		COLOR m_ColorText;
		COLOR m_ColorHighlight;
		MISTRING m_FontName;
		DLGSTYLE m_DlgStyle;
		bool m_NeedRestore;
		bool m_ShowHist;						// Whether to show histograms
		bool m_ShowRegLine;
		bool m_HaveTrackPos;
		bool m_SaveHighlightOutside;
		bool m_IsComposite;
		INT32 m_HistGraphHeight;						// "Height" of histogram displays
		SCALETYPE m_HistScaleType;
		TEXTFORMAT_LINE2D::EQUATIONFORM m_LineEqForm;
		INT16 m_TrackRadius;
		TRANS2D_MAPGEN m_TransImageToMap;
		RVC::OBJITEM m_ObjItemMask;
		RVC::OBJITEM m_ObjItemPalette;
		COLOR_ARRAY m_ColorPalette;
		COMPONENTLIST *m_pComponentList;
		CORRELATIONLIST m_CorrelationList;
		STAT_MULTIVAR m_StatMulti;
		LPOINT2D m_DragStartCenter;
		CANVASAREA *m_pCanvasAreaDrag;
		CANVASAREA *m_pCanvasAreaHighlightEllipse;
		CANVASAREA *m_pCanvasAreaHighlightPolygon;
		CANVASAREAPTRLIST m_CanvasAreaPtrList;
		GRE::GROUP *m_GroupHighlight;
		GRE::LAYER_IMAGE_SPATMOD *m_LayerHighlight;

		static void GRECB_GroupHighlight (const GRE::MSG *pMsg, void *vpDlg);
		static void GRECB_LayerHighlight (const GRE::MSG *pMsg, void *vpDlg);
		static void GRECB_View (const GRE::MSG *pMsg, void *vpDlg);

		void ClearAllTools ();
		void ClearMessage () const
			{ m_StatusBar.ClearText(); }
		ERRVALUE Compute ();
		void CreateDlgControlsSaveHighlight (MGUI::LAYOUT_PANE_BASE& pane);

		void DestroyLayerHighlight (bool RedrawGroupViews);

		void DoLayout (bool SetZoomFull = false);
		ERRVALUE DoNew ();
		ERRVALUE DoSaveAsRaster () const;
		ERRVALUE DoSaveAsText (bool IncludeCorrelation) const;
		ERRVALUE DoSaveHighlight ();
		void DoViewRecenter (MGUI::POINT ptCanvas);
		void DoViewZoom (const MGUI::POINT *pPtCanvas, double z, bool absolute=false);

		void DrawAll (bool CalcUsedRects = true);
		void DrawHistograms ();

		void EnableControls ();
		void FreeCanvasAreas ();
		CANVASAREA* GetCanvasArea (INT32 x, INT32 y);
		CANVASAREA* GetCanvasArea (const MGUI::POINT& pt)
			{ return (GetCanvasArea(pt.m_x,pt.m_y)); }
		MISTRING GetHighlightNameStr (
			) const;
		int IsMaskSelectable (const RVC::OBJITEM& ObjItem, MISTRING& msg, int) const;

		void OnCanvasDrawOverlay (MGD::CONTEXT *gc);
		bool OnCanvasKey (MGUI::POINT pt, MGUI::KEYCODE keycode, MGUI::KEYSTATE keystate);
		bool OnCanvasPointerButtonDown (MGUI::POINT pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		bool OnCanvasPointerButtonUp (MGUI::POINT pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void OnCanvasPointerLeave ();
		bool OnCanvasPointerMove (MGUI::POINT pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate, bool hasmore);
		void OnCanvasSize (int xsize, int ysize);

		void OnCbxSaveHighlightOutput ();

		void OnCmdFile (MGUI::ID cmdid, bool ToggleOn = false);
		void OnCmdMask ();
		void OnCmdOptions (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdLineEqForm (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdTool (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdTrackRadius (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdView (MGUI::ID cmdid, bool ToggleOn = false);

		void OnGroupHighlightMsg (const GRE::MSG& msg);
		void OnLayerHighlightMsg (const GRE::MSG& msg);
		void OnToolDragChange (MGUI::TOOL_BASE::CHANGESEQ ChgSeq, MGUI::TOOL_BASE::CHANGETYPE ChgType);
		MGUI::CURSORID OnToolDragGetCursor (const DPOINT2D& pt, bool ButtonDown);
		void OnToolHighlightRangeChange (MGUI::TOOL_BASE::CHANGESEQ ChgSeq, MGUI::TOOL_BASE::CHANGETYPE ChgType);
		bool OnToolHighlightRangeRightButtonDown (MGUI::POINT ptCanvas, MGUI::KEYSTATE keystate);
		void OnToolLineChange (MGUI::TOOL_BASE::CHANGESEQ ChgSeq, MGUI::TOOL_BASE::CHANGETYPE ChgType);
		ERRVALUE OnTrackReadSpan (INT32 col0, INT32 col1, INT32 row);
		void OnViewMsg (const GRE::MSG& msg);
		void OnViewPointerLeave (GRE::VIEW *view);
		void OnViewPointerMove (GRE::VIEW *view, const DPOINT2D& pt);

		ERRVALUE ReadPalette ();
		void ReadSettings ();
		void Restore (bool force);
		void SetMessage (const MISTRING& msg) const
			{ m_StatusBar.SetText(msg); }
		void SetupDftPalette ();
		int SetupSourceHighlight (SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage, bool UseOutside = false) const;
		void ShowStats (bool ToolOnly = false);
		void TrackPosClear ();
		void TrackPosSet (GRE::VIEW *view, const DPOINT2D& ptDisplay);
		ERRVALUE UpdateLayerHighlight ();
		void ViewAttach (GRE::VIEW *view);
		void ViewDetach (GRE::VIEW *view);

		// SHELL/DLGSHELL overrides.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnClose ();
		virtual void v_OnOpen ();

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_IMAGE_CORRELATION::DLGSTYLE)
#endif

//=====================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_DLGIMAGECORRELATION_H
