/**
 * \file gre/dlgmarkbyquery.h
 * \brief GRE::DLG_MARKBYQUERY definitions
 *
 * \if NODOC
 * $Id: dlgmarkbyquery.h_v 1.4 2013/10/03 21:12:06 mju Exp $
 *
 * $Log: dlgmarkbyquery.h_v $
 * Revision 1.4  2013/10/03 21:12:06  mju
 * Add buttons for apply to marked/unmarked.
 *
 * Revision 1.3  2011/11/28 17:08:02  mju
 * Fix dox dependency.
 *
 * Revision 1.2  2010/03/26 21:09:53  mju
 * Use form_tabbed.
 *
 * Revision 1.1  2010/01/25 20:57:23  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DLGMARKBYQUERY_H
#define	INC_GRE_DLGMARKBYQUERY_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_LAYOUT_H
#include <mgui/layout.h>
#endif

#ifndef	INC_MGUI_FORMTABBED_H
#include <mgui/formtabbed.h>
#endif

#ifndef	INC_MGUI_STATUSBAR_H
#include <mgui/statusbar.h>
#endif

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

namespace GRE {
//=====================================================================================================================

class DLG_MARKBYQUERY : public MGUI::DLGSHELL {
	public:

		// Constructor
		explicit DLG_MARKBYQUERY (
			GRE::LAYER::COMPINFO *pCompInfo
			);

		// Destructor.
		virtual ~DLG_MARKBYQUERY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_MARKBYQUERY THISCLASS;

		enum APPLYTO {
			APPLYTO_All,
			APPLYTO_Marked,
			APPLYTO_Unmarked
			};

		class PAGE_BUILDER;
		class PAGE_SCRIPT;

		GRE::LAYER::COMPINFO *m_pCompInfo;
		MGUI::FORM_TABBED m_Book;
		PAGE_BUILDER *m_pPageBuilder;
		PAGE_SCRIPT *m_pPageScript;
		MGUI::CTRL_PUSHBUTTON m_BtnApplyToAll;
		MGUI::CTRL_PUSHBUTTON m_BtnApplyToMarked;
		MGUI::CTRL_PUSHBUTTON m_BtnApplyToUnmarked;
		MGUI::CTRL_STATUSBAR m_StatusBar;
		SMLCONTEXT *m_ScriptContext;
		int m_ScriptSegment;
		bool m_bEnableApply;

		static void GRECB (const GRE::MSG *pMsg, void *vpDlg);

		void ClearMessage ()
			{ m_StatusBar.ClearText(); }

		ERRVALUE DoApply (APPLYTO ApplyTo);
		void EnableApply (bool enable);
		void EnableApplyToMarked ();

		INT64 MarkElementsByQuery (const MISTRING& scriptstr, APPLYTO ApplyTo);

		void OnBtnApplyToAll ();
		void OnBtnApplyToMarked ();
		void OnBtnApplyToUnmarked ();
		void OnGreMsg (const GRE::MSG& msg);

		void SetMessage (const MISTRING& msg)
			{ m_StatusBar.SetText(msg); }

		ERRVALUE v_CreateContent ();
		void v_OnClose ();
		void v_OnOpen ();

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_DLGMARKBYQUERY_H

