/**
 * \file gre/group.h
 * \brief GRE::GROUP class definitions
 *
 * \if NODOC
 * $Id: group.h_v 1.95 2015/03/31 13:08:59 mju Exp $
 *
 * $Log: group.h_v $
 * Revision 1.95  2015/03/31 13:08:59  mju
 * Remove hasLegend.
 *
 * Revision 1.94  2015/03/31 12:39:06  mju
 * Remove flag combineLayersInLegend.
 *
 * Revision 1.93  2015/02/19 21:01:03  mju
 * Add destroyLayersMatchingFlags.
 *
 * Revision 1.92  2014/12/09 16:30:27  mju
 * Add useRenderProjection to getSizeMeters.
 * Add useCenterIfNone to setupAutoProject.
 *
 * Revision 1.91  2014/06/24 22:19:30  bcolombe
 * Add method to test if group should be excluded from extents
 *
 * Revision 1.90  2014/05/13 15:54:09  mju
 * Add dlgGroupViewPoint3D.
 *
 * Revision 1.89  2013/10/23 21:37:24  mju
 * Add v_createRgnMgrNode.
 *
 * Revision 1.88  2013/10/22 21:44:58  mju
 * Remove rgnSourceClipped as unnecessary.
 *
 * Revision 1.87  2013/10/21 22:41:28  mju
 * Add regionsource members.
 *
 * Revision 1.86  2013/04/04 13:10:50  mju
 * Make sml groupx/yposn friends of posn.
 *
 * Revision 1.85  2013/04/03 21:22:42  mju
 * Add autoProject support.
 * Make posn members private with accessor methods.
 *
 * Revision 1.84  2013/04/02 13:38:41  mju
 * Add isScaleRelative.
 *
 * Revision 1.83  2013/03/29 21:02:11  mju
 * Make getRelativeScale non-inline.
 *
 * Revision 1.82  2013/03/28 21:44:13  mju
 * Add posn methods.
 *
 * Revision 1.81  2013/03/27 14:20:49  mju
 * Make attachment/attachref enum in group.
 *
 * Revision 1.80  2013/03/22 16:24:25  mju
 * Add dlgGroupSettings.
 *
 * Revision 1.79  2013/03/21 21:30:55  mju
 * Remove getFullName and setName.
 *
 * Revision 1.78  2013/03/20 19:52:34  mju
 * Remove dlgEditClipping.
 *
 * Revision 1.77  2013/03/04 15:53:59  mju
 * Remove getLayerZoom1X.
 *
 * Revision 1.76  2013/02/27 15:45:35  mju
 * Add updatedft to setDesignScale.
 *
 * Revision 1.75  2013/02/26 21:59:36  mju
 * Change getNext/Previous to use getNext/PreviousObjectOfType.
 *
 * Revision 1.74  2013/02/20 17:33:06  mju
 * Add destroyLayers.
 *
 * Revision 1.72  2013/02/14 22:42:41  mju
 * Use container base class.
 *
 * Revision 1.70  2013/02/07 18:00:38  mju
 * Add v_save and rename other virtual methods with v_ prefix.
 *
 * Revision 1.68  2012/07/19 14:16:46  mju
 * Add groupflag_NoAddLayers.
 *
 * Revision 1.65  2011/03/01 22:43:09  bcolombe
 * Add SavedLocations
 *
 * Revision 1.63  2008/12/15 15:43:03  mju
 * Add setAllLayersVisible methods.
 *
 * Revision 1.61  2008/10/13 19:48:15  mju
 * Add setPosn... methods.
 * Prefix member names with "m_" in preparation to making private.
 * Add setFlags variant with "notify" parm.
 *
 * Revision 1.60  2008/09/30 17:45:08  mju
 * Return bool from highlightSetDefault.
 *
 * Revision 1.59  2008/03/03 17:11:50  mju
 * Add setListPos with enum and reference gruop.
 *
 * Revision 1.56  2007/09/06 21:56:23  mju
 * Add support for layer common parms.
 *
 * Revision 1.48  2006/06/07 12:38:40  mju
 * Add createflag_sml and groupflags_CreatedBySML.
 *
 * Revision 1.45  2005/11/28 18:53:26  mju
 * Add v_getMax/MinScaleVisibleAny.
 *
 * Revision 1.37  2005/06/14 15:27:28  mju
 * Add parm to getMatchedGroup to determine if want first in link.
 *
 * Revision 1.17  2004/07/30 14:20:17  mju
 * Add groupflag_noLegacyRectExtents.
 *
 * Revision 1.16  2004/07/27 17:44:36  mju
 * Add groupflag to not clip to rectangle.
 *
 * Revision 1.1  2003/06/16 19:39:31  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_GROUP_H
#define	INC_GRE_GROUP_H

#ifndef	INC_GRE_VIEWABLE_H
	#include <gre/viewable.h>
#endif

#ifndef	INC_GRE_LAYER_H
	#include <gre/layer.h>
#endif

#ifndef	INC_MI32_TRANS2DAFFINE_H
	#include <mi32/trans2daffine.h>
#endif

namespace GRE {

//===================================================================================================================

#define	GROUPFLAG_None								0x00000000
#define	GROUPFLAG_Ignore							0x00000001	//!< Completely ignores group for positioning/display
#define	GROUPFLAG_NoForceAutoProjGeog			0x00000002	//!< Don't force auto-projection for geographic CRS
#define	GROUPFLAG_NoFixAspectLL					GROUPFLAG_NoForceAutoProjGeog
#define	GROUPFLAG_SingleLayer					0x00000004	//!< Group can only contain a single layer (other than background)
#define	GROUPFLAG_Clip								0x00000008	//!< Clip layers in group to specified region
#define	GROUPFLAG_OrientMatchRasterOnly		0x00000010	//!< Match first raster layer or nothing if no raster
#define	GROUPFLAG_OrientMatchRasterOther		0x00000020	//!< Match first raster layer or first layer if no raster
#define	GROUPFLAG_OrientMatchLayer				0x00000040	//!< Match first layer
#define	GROUPFLAG_NoSave							0x00000080	//!< Don't allow user to save group
#define	GROUPFLAG_NoUserSettings				0x00000100	//!< Don't allow user to change settings
#define	GROUPFLAG_3D								0x00000200	//!< 3-D group in a layout
#define	GROUPFLAG_NoDeleteGroup					0x00000400	//!< Don't allow user to delete group
#define	GROUPFLAG_NoDeleteLayers				0x00000800	//!< Don't allow user to delete (all) layers
#define	GROUPFLAG_ShowLatLonCoords				0x00001000	//!< Show group coordinates in lat-lon
#define	GROUPFLAG_NoAddLayers					0x00002000	//!< Don't allow user to add layers
#define	GROUPFLAG_NoLockZoom						0x00004000	//!< Don't lock group zoom factor
#define	GROUPFLAG_ShowLayerList					0x00008000	//!< Show list of layers for group
#define	GROUPFLAG_NoClipToAttachedGroup		0x00010000	//!< Don't clip to attached group if that group is clipped
// #define	OLDFLAG_CombineLayersInLegend		0x00020000	//!< Combine layers within group in LegendView by showing only legend for active layer
#define	GROUPFLAG_TemplateSaveLayersAsIs		0x00040000	//!< Save layers as-is to template
#define	GROUPFLAG_NoClipToRectOnly				0x00080000	//!< Don't force clipping to just be rectangle (for legacy layouts)
#define	GROUPFLAG_NoLegacyRectExtents			0x00100000	//!< Don't just use layer rectangle when compute extents (for legacy layouts)
#define	GROUPFLAG_CreatedBySML					0x00200000	//!< Group was created by SML

#define	GROUPFLAG_Mask_Orient					(GROUPFLAG_OrientMatchRasterOnly | GROUPFLAG_OrientMatchRasterOther | GROUPFLAG_OrientMatchLayer)
#define	GROUPFLAG_Mask_NeedRedraw				(GROUPFLAG_Ignore | GROUPFLAG_NoFixAspectLL | GROUPFLAG_Clip | GROUPFLAG_OrientMask | GROUPFLAG_3D | GROUPFLAG_NoClipToAttachedGroup | GROUPFLAG_NoClipToRectOnly)
#define	GROUPFLAG_Mask_NoNotify					GROUPFLAG_TemplateSaveLayersAsIs

#define	GROUPFLAG_OrientMask						GROUPFLAG_Mask_Orient


//! Container for group of layers all rendered in common CRS.
class GROUP : public GRE::VIEWABLE, public SERIALIZABLE {

	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class SML_GROUPXPOSN;
		class SML_GROUPYPOSN;
		#endif

		//! Flags for group creation.
		enum CREATEFLAGS {
			CREATEFLAG_None =					0x0000,
			CREATEFLAG_NoCallCreateCB =	0x0001,	//!< Don't call callback list when object created
			CREATEFLAG_Temporary =			0x0002,	//!< Object is temporary
			CREATEFLAG_3D =					0x0004,	//!< Group is 3D
			CREATEFLAG_NoAutoMatch =		0x0008,	//!< Don't default to automatch any layers
			CREATEFLAG_NoSetActive =		0x0010,	//!< Don't make newly-created group "active"
			CREATEFLAG_DeferCreate =		0x0020,	//!< Defer creation until Create() called (standalone group only)
			CREATEFLAG_SML =					0x0040,	//!< Being created by SML (standalone group only)
			CREATEFLAG_SingleLayer =		0x0080,	//!< Create as "single layer" group
			};

		enum ORIENTATION_BASE {
			ORIENTATION_BASE_Projection =		0,
			ORIENTATION_BASE_North =			1
			};

		enum LABEL_MANAGEMENT_MODE {
			LABEL_MANAGEMENT_MODE_RenderWithLayer = 0,
			LABEL_MANAGEMENT_MODE_RenderLast
			};

		//! Used with GetExtents().
		enum EXTENTCOORD {
			EXTENTCOORD_Group,
			EXTENTCOORD_Layout,
			EXTENTCOORD_View
			};

		//! Attachment location.
		enum ATTACHMENT {
			ATTACHMENT_None =				0,
			ATTACHMENT_Left =				1,
			ATTACHMENT_Bottom =			1,
			ATTACHMENT_Center =			2,
			ATTACHMENT_Right =			3,
			ATTACHMENT_Top =				3,
			ATTACHMENT_Geographic =		4
			};

		enum ATTACHREF {
			ATTACHREF_None =			0,
			ATTACHREF_Group =			1,
			ATTACHREF_Page =			2,
			ATTACHREF_Margin =		3
			};

		//! Layout positioning parameters.
		struct POSN {
			public:

				static const SERIALIZERITEM* SerialGetItemDef ();

				POSN ();
				POSN (const POSN& rhs);
				~POSN ();

				POSN& operator= (const POSN& rhs);

				bool operator== (const POSN& rhs) const
					{ return (IsEqual(rhs)); }

				bool operator!= (const POSN& rhs) const
					{ return (!IsEqual(rhs)); }

				void Clear ();
				ATTACHMENT GetAttachmentRef () const
					{ return (m_AttachmentRef); }
				ATTACHMENT GetAttachmentThis () const
					{ return (m_AttachmentThis); }
				ATTACHREF GetAttachRefType () const
					{ return (m_AttachRefType); }
				double GetOffset () const
					{ return (m_offset); }
				GROUP* GetRefGroup () const
					{ return ((m_AttachRefType == ATTACHREF_Group) ? m_refgroup : 0); }
				UINT8 GetState () const
					{ return (m_state); }
				bool IsAttachedGeographic () const
					{ return (m_AttachmentThis == ATTACHMENT_Geographic && GetRefGroup() != 0); }
				bool IsEqual (const POSN& rhs) const;
				void SetAttachedGeographic (GROUP *group);
				void SetAttachments (
					ATTACHMENT AttachmentThis,					//!< Attachment place for this group
					ATTACHMENT AttachmentRef					//!< Attachment place for reference group
					) { m_AttachmentThis = AttachmentThis; m_AttachmentRef = AttachmentRef; }
				void SetAttachRef (
					ATTACHREF AttachRefType,					//!< Attachment reference item type
					GRE::GROUP *RefGroup = 0					//!< Reference group for ATTACHREF_Group
					);
				void SetOffset (double offset)
					{ m_offset = offset; }
				void SetRefGroup (GROUP *group)
					{ m_refgroup = group; m_RefGroupName.Clear(); }
				void SetState (UINT8 state)
					{ m_state = state; }
				void SetValues (
					ATTACHREF AttachRefType,					//!< Attachment reference item type
					ATTACHMENT AttachmentThis,					//!< Attachment place for this group
					ATTACHMENT AttachmentRef,					//!< Attachment place for reference group
					double offset = 0,
					GRE::GROUP *RefGroup = 0					//!< Reference group for ATTACHREF_Group
					) { m_AttachRefType = AttachRefType; m_AttachmentThis = AttachmentThis; m_AttachmentRef = AttachmentRef; m_offset = offset; m_refgroup = RefGroup; }
				void UpdateRefGroupName () const;
				bool Validate (GROUP *group);
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef POSN THISCLASS;
				friend class GRE::GROUP::SML_GROUPXPOSN;
				friend class GRE::GROUP::SML_GROUPYPOSN;

				double m_offset;
				GROUP *m_refgroup;
				mutable MISTRING m_RefGroupName;
				ATTACHREF m_AttachRefType;
				ATTACHMENT m_AttachmentThis;
				ATTACHMENT m_AttachmentRef;
				UINT8 m_state;
				#endif
			};

	#ifdef X_NATIVE
		//! Get group from object attached to Widget userData
		static GROUP* GetFromWidget (
			Widget w
			);
	#endif

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct "stand-alone" group
		explicit GROUP (
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		//! Construct group within a layout
		explicit GROUP (
			GRE::LAYOUT *layout,
			CREATEFLAGS createflags = CREATEFLAG_None,
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,
			GROUP *refgroup = 0
			);

		//! Destructor.
		virtual ~GROUP ();

		//! Attach common parameter instance for layer type.
		//! Group will become owner of pointer and delete when necessary.
		void AttachLayerCommonParm (
			LAYER::TYPE LayerType,
			GRE::LAYER::COMMONPARM *pParmInstance
			);

		//! Determine if auto-projection setting can be changed.
		//! Auto-projection setting cannot be changed if group is auto-matched to a layer or another group,
		//! has a local CRS, or is 3D.
		bool CanChangeAutoProject () const;

		//! Check if layer of specified type can be added
		ERRVALUE CheckAddLayer (
			GRE::LAYER::TYPE layertype = GRE::LAYER::TYPE_None
			) const;

		//! Check if georeference for layers in group are compatible
		void CheckGeorefCompatible () const;

		void ClearPosns ()
			{ m_xposn.Clear(); m_yposn.Clear(); }

		//! Compute extents of group in layout
		void ComputeLayoutExtents ();

		//! Compute map coordinate extents for group
		ERRVALUE ComputeMapExtents (); 

		ERRVALUE ComputeRegionFromLayers (
			REGION2D& region,							//!< Region returned, will be in group's RenderCoordRefSys
			bool IncludeMargins,						//!< Include layer margins
			GRE::LAYER::FILTER *filter = 0			//!< Filter
			) const;

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Construct GROUPVIEW instance for this group and specified view.
		GRE::GROUPVIEW* ConstructGroupView (
			GRE::VIEW *view
			);
	#endif

		//! Create group if CREATEFLAG_DeferCreate was specified in constructor.
		void Create (
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		//! Destroy all layers in group.
		void DestroyAllLayers ();

		//! Destroy specified layer in this group
		void DestroyLayer (
			GRE::LAYER* layer
			);

		//! Destroy specified layers in this group.
		void DestroyLayers (
			const SIMPLE_ARRAY<LAYER*>& layers		//!< Layers to be destroyed.
			);

		//! Destroy all layers matching any specified creation flags.
		//! Typically used to destroy auto-created input and/or output layers.
		void DestroyLayersMatchingFlags (
			GRE::LAYER::CREATEFLAGS CreateFlags		//!< Any flags specified in layer creation
			);

		//! Draw group overlay containing 'active' element for each layer.
		ERRVALUE DrawActiveOverlay (
			GRE::DRAWFLAGS DrawFlags = GRE::DRAWFLAG_UpdateScreen,
			GRE::VIEW *view = 0							//!< View to draw overlay for, 0 for all, will be drawn first if drawing multiple
			);

		//! Get automatic projection used and whether setting can be changed.
		SPATREF::COORDOPDEF::AUTOPROJECT GetAutoProject (
			bool *pCanChange = 0							//!< Return whether setting can be changed
			) const;

		//! Get clipping region for group.
		const REGION2D& GetClipRegion () const;

		//! Get setting for "clip to matched layer" regardless of whether or not will be auto-matched.
		bool GetClipToMatchedLayer () const
			{ return (m_ClipToMatchedLayer); }

		//! Get extents of group in specified coordinates
		ERRVALUE GetExtents (
			DRECT2D& extents,								//!< Group extents returned
			EXTENTCOORD coords							//!< What to return extents in
			);

		//! Get first layer in group.
		GRE::LAYER* GetFirstLayer () const;

		//! Get current group flag settings
		UINT32 GetFlags () const
			{ return (m_groupflags); }

		const REGION2D& GetFullRegion () const
			{ return (m_FullRegion); }

		LABEL_MANAGEMENT_MODE GetLabelManagementMode () const
			{ return (m_LabelManagementMode); }

		//! Get last layer in group.
		GRE::LAYER* GetLastLayer () const;

		//! Get common parameter base class pointer for specific layer type.
		//! @return Pointer to parameter instance or NULL if none set for type.
		const GRE::LAYER::COMMONPARM* GetLayerCommonParm (
			LAYER::TYPE LayerType
			) const;

		//! Get layer set as active.
		GRE::LAYER* GetLayerActive () const;

		//! Get common parameter base class pointer for specific layer type.
		//! @return Pointer to parameter instance or NULL if none set for type.
		GRE::LAYER::COMMONPARM* GetLayerCommonParm (
			LAYER::TYPE LayerType
			);

		//! Get layout group is contained in.
		//! @return pointer to layout.
		GRE::LAYOUT * GetLayout () const;

		const DRECT2D& GetLayoutExtents () const
			{ return (m_LayoutExtents); }

		//! Return geographically-attached reference group if any.
		GROUP* GetMatchedGroup (
			bool firstonly = false				//! True to return immediate (first link) matched group
			) const;

		//! Get "set" for mutual exclusiveness between groups.
		UINT32 GetMutualExclusiveGroupSet () const
			{ return (m_MutualExclusiveGroupSet); }

		//! Determine if need updated extents.
		bool GetNeedUpdateExtents () const
			{ return (m_NeedUpdateExtents); }

		//! Get next group in layout.
		GROUP* GetNext () const
			{ return (static_cast<GROUP*>(GetNextObjectOfType(TYPE_Group))); }

		//! Get positioning parameters for X axis.
		const POSN& GetPosnX () const
			{ return (m_xposn); }

		//! Get positioning parameters for Y axis.
		const POSN& GetPosnY () const
			{ return (m_yposn); }

		//! Get previous group in layout.
		GROUP* GetPrevious () const
			{ return (static_cast<GROUP*>(GetPreviousObjectOfType(TYPE_Group))); }

		//! Get relative scale.
		double GetRelativeScale () const;

		//! Get rotation angle relative to base in radians.
		double GetRotationAngle () const
			{ return (m_RotAngle); }

		//! Get base for rotation.
		ORIENTATION_BASE GetRotationBase () const
			{ return (m_OrientBase); }

		//! Get Saved Locations string
		const MISTRING& GetSavedLocations () const
			{ return (m_SavedLocations); }

		//! Get serializer instance being used to load group.
		//! @return Pointer to SERIALIZER, NULL if not being loaded.
		SERIALIZER* GetSerializer () const
			{ return (m_pSerializer); }

		//! Get transformation from group to view.
		const TRANS2D_AFFINE& GetTransViewToGroup () const
			{ return (m_ViewToGroup); }

		//! Get CoordRefSys used for rendering.
		const SPATREF::COORDREFSYS& GetUsedCoordRefSys () const;

		GRE::VIEWPOINT3D* GetViewPoint3D () const
			{ return (m_ViewPoint3D); }

		//! Get view visibility.
		GRE::VISIBLEVIEW GetVisible () const
			{ return (m_VisibleViewLayout); }

		//! Get scale range over which group is visible
		void GetVisibleScaleRange (
			double& MinScaleVisible,				//!< Minimum scale visible returned, 0 for no minimum
			double& MaxScaleVisible					//!< Maximum scale visible returned, 0 for no maximum
			) const { MinScaleVisible = m_MinScaleVisible; MaxScaleVisible = m_MaxScaleVisible; return; }

		//! Determine if group has other groups attached to it
		bool HasAttachedGroups () const;

		//! Determine if any layers in group have "highlightable" elements.
		bool HasHighlightableLayer (
			GRE::VIEW *view = 0							//!< Reference view for layer visibility test
			) const;

		//! Determine if group has mutually-exclusive layers
		bool HasMutuallyExclusiveLayers () const
			{ return (m_HasMutuallyExclusiveLayers); }

		//! Setup for default highlighting of active layer elements.
		//! @return true if set highlighting, false if not.
		bool HighlightSetupDefault (
			GRE::VIEW *view = 0							//!< Optional view for determining layer visibility
			);

		//! Invalidate all view buffers for this group.
		void InvalidateAllViews (
			GRE::RENDERTARGET Target = GRE::RENDERTARGET_GroupAll | GRE::RENDERTARGET_LayerAll
			);

		//! Determine if group can be viewed in 3D
		bool Is3D () const
			{ return (!GetLayout() || (m_groupflags & GROUPFLAG_3D)); }

		bool IsBeingLoaded () const
			{ return (m_pSerializer != 0); }

		//! Determine if group is to be clipped.
		bool IsClipped () const;

		//! Determine if group is to be clipped to the auto-matched layer.
		bool IsClippedToMatchedLayer () const
			{ return (m_ClipToMatchedLayer && GetMatchedLayer() != 0); }

		//! Determine if group has been created.
		//! This is done automatically by constructor unless CREATEFLAG_DeferCreate was specified.
		bool IsCreated () const
			{ return (m_IsCreated); }

		//! Determine if all layers excluded from group extents
		bool IsExcludedFromGroupExtents () const;

		//! Determine if group should be ignored for extents determination
		bool IsIgnored () const;

		//! Determine if scale is computed relative to layout.
		bool IsScaleRelative () const
			{ return (m_RelativeScale > 0 && GetLayout() != 0); }

		//! Determine if group only allows a single layer
		bool IsSingleLayer () const
			{ return ((m_groupflags & GROUPFLAG_SingleLayer) != 0); }

		//! Determine if group visible at specified scale
		bool IsVisibleAtScale (
			double scale
			) const { return (!GetLayout() || (scale >= m_MinScaleVisible && (m_MaxScaleVisible <= 0.0 || scale <= m_MaxScaleVisible))); }

		//! Determine if group is visible in specified view.
		bool IsVisibleInView (
			const GRE::VIEW *view
			) const;

		//! Read from serialization
		virtual ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write values to serialization
		virtual ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			) const;

		//! Set group to 2D
		void Set2D ();

		//! Set active layer for group
		void SetActiveLayer (
			GRE::LAYER *layer								//!< Layer to make active
			);

		//! Set visibility of all layers in specified views.
		void SetAllLayersVisible (
			VISIBLEVIEW views,
			bool visible
			);

		//! Set visibility of all layers in specified view.
		void SetAllLayersVisible (
			VIEW *view,
			bool visible
			);

		//! Set clipping rectangle.
		void SetClipRect (
			const DRECT2D& ClipRect,					//!< Rectangle to clip to
			const SPATREF::COORDREFSYS& RectCRS,	//!< Rectangle CRS
			bool notify = true
			);

		//! Set clipping region.
//		void SetClipRegion (
//			const REGION2D& ClipRegion,				//!< Clip region
//			bool notify = true
//			);

		//! Set whether to clip to 'auto-matched' layer
		void SetClipToMatchedLayer (
			bool ClipToMatchedLayer,					//!< 'true' to clip, 'false' to not
			bool notify = true							//!< Send ACTION_ChangeBegin/End notification
			);

		//! Set specified group flags
		void SetFlags (
			UINT32 flags,									//!< GROUPFLAG_... flag values
			UINT32 mask										//!< GROUPFLAG_... mask for which flags to change
			);

		//! Set specified group flags
		void SetFlags (
			UINT32 flags,									//!< GROUPFLAG_... flag values
			UINT32 mask,									//!< GROUPFLAG_... mask for which flags to change
			bool notify
			);

		void SetLabelManagementMode (
			LABEL_MANAGEMENT_MODE mode
			) { m_LabelManagementMode = mode; }

		//! Set matte values.
		void SetMatte (
			const MATTE& matte,							//!< Matte values to set
			bool notify = true							//!< Set to invoke callbacks with "Changed" message
			);

		//! Set "group set" which group is mutually-exclusive with.
		void SetMutualExclusiveGroupSet (
			UINT32 GroupSet								//!< Group set bits
			);

		//! Set whether layers are mutually-exclusive.
		void SetMutuallyExclusiveLayers (
			bool LayersMutuallyExclusive = true
			);

		//! Set group to recompute extents.
		void SetNeedUpdateExtents (
			) { m_NeedUpdateExtents = true; }

		//! Set group rotation base and orientation.
		void SetOrientation (
			ORIENTATION_BASE OrientBase,				//!< Orientation base
			double NewRotAngle							//!< Rotation angle, radians clockwise relative to base
			) { m_OrientBase = OrientBase; m_RotAngle = NewRotAngle; }

		//! Set group projection to match specified layer
		void SetProjView (
			const GRE::LAYER *layer						//!< Layer to obtain projection from, NULL to match first raster/layer
			);

		//! Set positioning attachment as geographic.
		//! Does not send notification of changes.
		void SetPosnAttachGeographic (
			GRE::GROUP *RefGroup = 0					//!< Reference group, will use previous or default if NULL
			);

		//! Set positioning attachment for X.
		//! Does not send notification of changes.
		void SetPosnAttachmentX (
			ATTACHMENT AttachmentThis,					//!< Attachment place for this group
			ATTACHMENT AttachmentRef					//!< Attachment place for reference group
			);

		//! Set positioning attachment for Y.
		//! Does not send notification of changes.
		void SetPosnAttachmentY (
			ATTACHMENT AttachmentThis,					//!< Attachment place for this group
			ATTACHMENT AttachmentRef					//!< Attachment place for reference group
			);

		//! Set positioning reference item to attach to for X.
		//! Does not send notification of changes.
		void SetPosnAttachToX (
			ATTACHREF AttachRefType,					//!< Attachment reference item type
			GRE::GROUP *RefGroup							//!< Reference group for ATTACHREF_Group
			);

		//! Set positioning reference item to attach to for Y.
		//! Does not send notification of changes.
		void SetPosnAttachToY (
			ATTACHREF AttachRefType,					//!< Attachment reference item type
			GRE::GROUP *RefGroup							//!< Reference group for ATTACHREF_Group
			);

		//! Set positioning offset for X.
		//! Does not send notification of changes.
		void SetPosnOffsetX (
			double offset
			) { m_xposn.SetOffset(offset); }

		//! Set positioning offset for Y.
		//! Does not send notification of changes.
		void SetPosnOffsetY (
			double offset
			) { m_yposn.SetOffset(offset); }

		//! Set positioning state for X.
		void SetPosnStateX (
			UINT8 PosnState
			) { m_xposn.SetState(PosnState); }

		//! Set positioning state for Y.
		void SetPosnStateY (
			UINT8 PosnState
			) { m_yposn.SetState(PosnState); }

		//! Set all positioning values for X.
		void SetPosnValuesX (
			const POSN& posn
			) { m_xposn = posn; }

		//! Set all positioning values for X.
		void SetPosnValuesX (
			ATTACHREF AttachRefType,					//!< Attachment reference item type
			GRE::GROUP *RefGroup,						//!< Reference group for ATTACHREF_Group
			ATTACHMENT AttachmentThis,					//!< Attachment place for this group
			ATTACHMENT AttachmentRef,					//!< Attachment place for reference group
			double offset
			) { m_xposn.SetValues(AttachRefType,AttachmentThis,AttachmentRef,offset,RefGroup); }

		//! Set all positioning values for X.
		void SetPosnValuesY (
			const POSN& posn
			) { m_yposn = posn; }

		//! Set all positioning values for Y.
		void SetPosnValuesY (
			ATTACHREF AttachRefType,					//!< Attachment reference item type
			GRE::GROUP *RefGroup,						//!< Reference group for ATTACHREF_Group
			ATTACHMENT AttachmentThis,					//!< Attachment place for this group
			ATTACHMENT AttachmentRef,					//!< Attachment place for reference group
			double offset
			) { m_yposn.SetValues(AttachRefType,AttachmentThis,AttachmentRef,offset,RefGroup); }

		//! Set group scale relative to layout design scale.
		void SetRelativeScale (
			double RelativeScale,
			bool notify = true
			);

		//! Set coordinate reference system to render group in.
		ERRVALUE SetRenderCoordRefSys (
			const SPATREF::COORDREFSYS& RenderCRS
			);

		void SetSavedLocations (
			const MISTRING& string
			) { m_SavedLocations = string; }

		void SetSingleLayer (
			bool SingleLayer
			) { if (SingleLayer) m_groupflags |= GROUPFLAG_SingleLayer; else m_groupflags &= ~GROUPFLAG_SingleLayer; }

		//! Setup auto-projected CRS based on group center.
		ERRVALUE SetupAutoProject (
			SPATREF::COORDREFSYS& CoordRefSys,		//!< Coordinate Reference System definition returned
			bool bUseCenterIfNone = true				//!< Use center even if no auto-projection explicitly set
			) const;

		//! Set 3D viewpoint.
		void SetViewPoint3D (
			const GRE::VIEWPOINT3D& ViewPoint3D
			);

		//! Set group visibility in specified views
		void SetVisible (
			UINT32 flags,									//!< Bits indicating which views to modify visibility for
			bool set											//!< true if visible, false if not
			);

		//! Set scale range over which group is visible
		void SetVisibleScaleRange (
			double MinScaleVisible,
			double MaxScaleVisible
			) { m_MinScaleVisible = MinScaleVisible; m_MaxScaleVisible = MaxScaleVisible; }

		//! Set group visibility in specified view
		void SetVisibleView (
			GRE::VIEW *view,								//!< View to set visibility for
			bool visible									//!< true if visible, false if not
			);

		//! Write template to serialization
		ERRVALUE TemplateWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			);

		//! Validate attachment settings.
		void ValidateAttachments ();
		
	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
			static const SML::CLASSMEMBER* GetSMLClassMembers();
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef GROUP THISCLASS;

		SPATREF::COORDREFSYS m_RenderCRS;		// Coordinate reference system group rendered in if not auto-matched
		REGION2D m_ClipRegion;						// Clip region
		REGION2D m_FullRegion;						// Union of all layer regions in group's render CRS
		TRANS2D_AFFINE m_ViewToGroup;
		POSN m_xposn;
		POSN m_yposn;
		MISTRING m_SavedLocations;
		mutable MISTRING m_ActiveLayerName;
		DRECT2D m_LayoutExtents;					// Extents in layout coordinates
		double m_RotAngle;							// Rotation angle
		double m_MinScaleVisible;
		double m_MaxScaleVisible;
		double m_DesignScale;						// Group Design scale
		double m_RelativeScale;						// Scale relative to layout Design scale
		GRE::VIEWPOINT3D *m_ViewPoint3D;			// 3-D viewpoint for group when used in layout
		SERIALIZER *m_pSerializer;					// Set while group is being loaded
		LAYER::COMMONPARM **m_PtrsLayerCommonParm;
		UINT32 m_groupflags;
		UINT32 m_MutualExclusiveGroupSet;
		GRE::VISIBLEVIEW m_VisibleViewLayout;	// Bits for whether group is visible in layout views
		GRE::HIGHLIGHTMODE m_ElemHighlightMode;
		LABEL_MANAGEMENT_MODE m_LabelManagementMode;
		ORIENTATION_BASE m_OrientBase;
		SPATREF::COORDOPDEF::AUTOPROJECT m_AutoProject;
		bool m_IsCreated;
		bool m_NeedUpdateExtents;					// Extents need to be recomputed
		bool m_ClipToMatchedLayer;
		bool m_HasMutuallyExclusiveLayers;

		static void GRECB_Group (
			const GRE::MSG *msg,
			void *vgroup
			);

		static const SERIALIZERITEM* SerialGetItemDef (
			SERIALIZER& serializer
			);

		static ERRVALUE SerializerCB_OldClipProj (SERIALIZER&, const SERIALIZERITEM*, void *vgroup, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_OldClipRect (SERIALIZER&, const SERIALIZERITEM*, void *vgroup, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_OldGroupFlags (SERIALIZER&, const SERIALIZERITEM*, void *vgroup, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_ViewPoint3D (SERIALIZER&, const SERIALIZERITEM*, void *vgroup, SERIALIZERITEM::ACTION);

		static void smlGetNext (SML::CONTEXT*, SML::CLASSMEMBER*, void *vpGroup, SML::ARG*);
		static void smlGetPrev (SML::CONTEXT*, SML::CLASSMEMBER*, void *vpGroup, SML::ARG*);

		void Init ();

		//! Determine layer group projection will be auto-matched to
		GRE::LAYER *GetMatchedLayer () const;

		// GRE::OBJECT overrides.
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;
		virtual void v_SetModified (bool notify, bool ViewsNeedRedraw);

		// GRE::CONTAINER overrides.
		virtual TREEVIEWNODE* v_CreateRgnMgrNode (TREEVIEWNODE& nodeParent, TREEVIEWNODE *pNodeInsertBefore);

		// GRE::VIEWABLE overrides.
		virtual void v_Clear ();
		virtual ERRVALUE v_DrawAllViews (GRE::DRAWCONDITION condition);
		virtual GROUP* v_FindGroupGeoref (bool AllowLocalCRS);
		virtual double v_GetDesignScale () const;
		virtual const DRECT2D& v_GetExtents () const;
		virtual double v_GetMaxScaleVisibleAny () const;
		virtual double v_GetMinScaleVisibleAny () const;
		virtual ERRVALUE v_GetSizeMeters (DPOINT2D& sizemeters, bool bUseRenderProjection);
		virtual ERRVALUE v_Save (MDLGPARENT dlgparent, bool forceprompt);
		virtual void v_SetDesignScale (double DesignScale, bool notify, bool updateDft);
		virtual void v_UnhighlightAllElements (bool redraw);

		GROUP (const GROUP&);
		GROUP& operator= (const GROUP&);

		friend class GRE::LAYOUT;
		friend class GRE::LAYER;
		friend class SML_GROUPXPOSN;
		friend class SML_GROUPYPOSN;

		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GROUP::CREATEFLAGS)
#endif

//=====================================================================================================================

namespace UI {

//! Dialog to edit group properties/settings.
ERRVALUE DlgGroupSettings (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GROUP *group						//!< Group to edit settings for, used as initial group if in a layout
	);

void DlgGroupSettingsSetOffsetSize (
	LAYOUT *layout,
	double xoffset,
	double yoffset,
	double xsize,
	double ysize
	);

//! Open modeless dialog to set 3D viewpoint for group in layout.
ERRVALUE DlgGroupViewPoint3D (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	GROUP *group						//!< Group to set viewpoint for, must be a 3D group in a layout
	);

}	// End namespace UI

//=====================================================================================================================

}	// End namespace GRE

#endif	//!< INC_GRE_GROUP_H
