/**
 * \file gre/lrformula.h
 * \brief GRE_LAYER_FORMULA definitions
 *
 * \if NODOC
 * $Id: lrformula.h_v 1.14 2015/02/05 23:01:02 bcolombe Exp $
 *
 * $Log: lrformula.h_v $
 * Revision 1.14  2015/02/05 23:01:02  bcolombe
 * USE_MGUI
 *
 * Revision 1.13  2014/06/20 21:23:01  mju
 * Add v_hasProperty.
 *
 * Revision 1.12  2014/02/28 21:49:51  mju
 * Add dispparm methods to setContrastMethods and access color spread.
 *
 * Revision 1.11  2014/02/28 16:49:27  mju
 * Add contrast, colorspread, datatip support.
 *
 * Revision 1.10  2014/02/20 17:04:58  mju
 * Add private texture nested class.
 *
 * Revision 1.9  2014/02/19 18:30:44  mju
 * Update to use mgui.
 *
 * Revision 1.1  2005/01/20 16:56:25  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRFORMULA_H
#define	INC_GRE_LRFORMULA_H

#ifndef	INC_GRE_LAYER_H
	#include <gre/layer.h>
#endif

#ifndef	INC_GRE_DATATIP_H
	#include <gre/datatip.h>
#endif

#ifndef	INC_MI32_GEOFRMLA_H
	#include <mi32/geofrmla.h>
#endif

#ifndef	INC_MI32_COLORSPREAD_H
	#include <mi32/colorspread.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
	#include <mi32/contrastparm.h>
#endif

namespace GRE {

//===================================================================================================================

//! GeoFormula layer.
class LAYER_FORMULA : public GRE::LAYER {
	public:

		//! Display parameters for GeoFormula layer.
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				//! Assignment.
				DISPPARM& operator= (
					const DISPPARM& rhs
					);

				COLORSPREAD& AccessColorSpread ()
					{ return (m_ColorSpread); }

				GEOFORMULA& AccessFormula ()
					{ return (m_Formula); }

				const COLORSPREAD& GetColorSpread () const
					{ return (m_ColorSpread); }

				const CONTRASTPARM& GetContrastBlue () const
					{ return (m_ContrastB); }

				const CONTRASTPARM& GetContrastGreen () const
					{ return (m_ContrastG); }

				const CONTRASTPARM& GetContrastNumeric () const
					{ return (m_ContrastN); }

				const CONTRASTPARM& GetContrastRed () const
					{ return (m_ContrastR); }

				const DATATIPPARMS& GetDataTip () const
					{ return (m_DataTip); }

				const GEOFORMULA& GetFormula () const
					{ return (m_Formula); }

				void IniRead ();

				void SetContrastMethodN (
					CONTRASTPARM::METHOD method
					) { m_ContrastN.SetMethod(method); }

				void SetContrastMethodRGB (
					CONTRASTPARM::METHOD methodR,
					CONTRASTPARM::METHOD methodG,
					CONTRASTPARM::METHOD methodB
					) { m_ContrastR.SetMethod(methodR); m_ContrastG.SetMethod(methodG); m_ContrastB.SetMethod(methodB); }

				void SetDataTip (
					const DATATIPPARMS& datatip
					);

				void SetDataTipDefault ();

				ERRVALUE SetupContrast ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_FORMULA::DISPPARM THISCLASS;

				GEOFORMULA m_Formula;
				CONTRASTPARM m_ContrastN;
				CONTRASTPARM m_ContrastR;
				CONTRASTPARM m_ContrastG;
				CONTRASTPARM m_ContrastB;
				COLORSPREAD m_ColorSpread;
				DATATIPPARMS m_DataTip;

				static ERRVALUE SerializerCB_Formula (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);

				void Copy (const DISPPARM& rhs);

				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				#endif
			};	// End DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_FORMULA (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,		//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,			//!< Position in list
			GRE::LAYER *reflayer = 0,								//!< Reference layer for insertion
			GEOFORMULA::OUTPUTTYPE FormulaType = GEOFORMULA::OUTPUTTYPE_Color		//!< Initial formula type
			);

		virtual ~LAYER_FORMULA ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm () const
			{ return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER BASECLASS;

		class DRAWDATA;
		class TEXTURE;

		DISPPARM m_dispparm;								//!< Layer-specific display parameters
		DISPPARM::DLG *m_pDlgControlPanel;

		ERRVALUE Draw2D (GRE::LAYERDC& layerdc);
		ERRVALUE Draw3D (GRE::LAYERDC& layerdc);

		//! Overrides from GRE::LAYER.
		#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
		#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW* view);
		virtual bool v_DataTipHasAvailable () const;
		virtual bool v_DataTipHasEnabled (GRE::VIEW *view) const;
		virtual void v_DataTipSetDefault (int component, bool force);
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual bool v_HasProperty (PROPERTY property) const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_TemplateRead (SERIALIZER& serializer);
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_FORMULA (const LAYER_FORMULA&);
		LAYER_FORMULA& operator= (const LAYER_FORMULA&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_LRFORMULA_H
