/**
 * \file gre/lrosm.h
 * \brief GRE::LAYER_OSM definitions.
 *
 * \if NODOC
 * $Id: lrosm.h_v 1.9 2016/08/22 14:56:59 mju Exp $
 *
 * $Log: lrosm.h_v $
 * Revision 1.9  2016/08/22 14:56:59  mju
 * Remove unused member.
 *
 * Revision 1.8  2015/01/20 14:27:13  mju
 * Remove unused pointless function.
 *
 * Revision 1.7  2014/06/30 19:34:48  ldyas
 * added pointer to xml for visibility.
 *
 * Revision 1.6  2014/06/20 21:23:28  mju
 * Add v_hasProperty.
 *
 * Revision 1.5  2014/06/10 14:27:11  mju
 * Remove legendMsgHandler.
 *
 * Revision 1.3  2014/05/23 20:55:29  ldyas
 * adjusting handling language codes
 *
 * Revision 1.1  2014/05/21 17:20:50  ldyas
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LROSM_H
#define	INC_GRE_LROSM_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

#ifndef	INC_GRE_DATATIP_H
#include <gre/datatip.h>
#endif

#ifndef	INC_RVC_STYLEPATTERNBITMAP_H
#include <rvc/stylepatternbitmap.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEARRAY_H
#include <spatmod/imagestagearray.h>
#endif

#ifndef	INC_SPATMOD_IMAGETILESET_H
#include <spatmod/imagetileset.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef	INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif

#ifndef INC_MI32_DATETIME_H
#include <mi32/datetime.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class QUADTREE;
#endif


namespace GRE {

class LAYER_OSM : public GRE::LAYER {
	public:
		 
		// LAYER_OSM::DISPPARM
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					);

				const DATATIPPARMS& GetDataTipParms () const
					{ return (m_DataTip); }

				const TEXTSTYLE& GetLabelStyle (
					) const { return (m_LabelStyle); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				const FILEPATH& GetPath (
					) const { return (m_ObjItem.GetFilePath()); }

				double GetPointScale (
					) const { return m_PointScale; }

				const POINTSTYLE& GetPointStyle (
					) const { return (m_PointStyle); }

				const POLYSTYLE& GetPolyStyle (
					) const { return (m_PolyStyle); }

				UINT16 GetSelectMode () const
					{ return (m_SelectMode); }
				
				bool GetShowLabels (
					) const { return (m_showlabels); }

				const MISTRING& GetURL (
					) const { return (m_ObjItem.GetURL()); }

				bool HasURL () const
					{ return (m_ObjItem.GetType() == RVC::OBJITEM::TYPE_URL); }
 
				void SetDataTipParms (
					const DATATIPPARMS& DataTipParms
					) { m_DataTip = DataTipParms; }

				void SetLabelStyle (
					const TEXTSTYLE& style
					) { m_LabelStyle = style; }

				void SetObjItem (
					const RVC::OBJITEM& ObjItem
					) { m_ObjItem = ObjItem; }

				void SetPointScale (
					double scale
					) { m_PointScale = scale; }

				void SetPointStyle (
					const POINTSTYLE& style
					) { m_PointStyle = style; }

				void SetPolyStyle (
					const POLYSTYLE& style
					) { m_PolyStyle = style; }

				void SetShowLabels (
					bool label = true
					) { m_showlabels = label; }

				void GetLanguageCode (
					MISTRING& code
					){ code = m_LanguageCode; }

				const char* GetLanguageCode (
					){ 
					return (static_cast <const char *>(m_LanguageCodeFull.GetEncoded(CHAR_ENCODING_ASCII))); 
					}

				void SetURL (
					const MISTRING& url
					) { m_ObjItem.SetURL(url,RVC::OBJTYPE_RemoteKML); }

				void SetLanguageCode (
					const MISTRING& code
					){ m_LanguageCode = code; }

				void SetLanguageCode (
					const char* code 
					){ 
					m_LanguageCodeFull = MISTRING(code); 
					}

				bool GetHideCounties (
					){
					return m_hideCounties;
					}
				void SetHideCounties (
					bool hide
					){
					m_hideCounties = hide;
					}
				bool GetHideChurches (
					){
					return m_hideChurches;
					}
				void SetHideChurches (
					bool hide
					){
					m_hideChurches = hide;
					}

				bool GetHideParks (
					){
					return m_hideParks;
					}

				void SetHideParks (
					bool hide
					){
					m_hideParks = hide;
					}

				bool GetHideBikeParking (
					){
					return m_hideBikeParking;
					}

				void SetHideBikeParking (
					bool hide
					){
					m_hideBikeParking = hide;
					}
				
				 XMLDOC *m_LabelsDoc;
				 
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER::DISPPARM BASECLASS;
				typedef LAYER_OSM::DISPPARM THISCLASS;
		
				RVC::OBJITEM m_ObjItem;
				POLYSTYLE m_PolyStyle;
				LINESTYLE m_LineStyle;
				POINTSTYLE m_PointStyle;
				TEXTSTYLE m_LabelStyle;
				GRE::DATATIPPARMS m_DataTip;
				double m_PointScale;
				UINT16 m_SelectMode; 
				bool m_showlabels;
				bool m_hideChurches;
				bool m_hideCounties;
				bool m_hideParks;
				bool m_hideBikeParking;
		 
				MISTRING m_LanguageCode;
				MISTRING m_LanguageCodeFull;
	
				void Copy (const DISPPARM&);
				void Free ();

				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();
				
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_OSM::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_OSM (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_OSM ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		SPATMOD::IMAGE::TILESET m_tileset;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER BASECLASS;
		typedef LAYER_OSM THISCLASS;

		class DLG;

		class OSMELEMSTYLE {
			public:
				OSMELEMSTYLE ();
				~OSMELEMSTYLE ();

				POINTSTYLE m_PointStyle;
				POLYSTYLE m_PolyStyle;
				LINESTYLE m_LineStyle;
				TEXTSTYLE m_LabelStyle;
				
				INT32 m_IconIndex;
				MISTRING m_Label;
				MISTRING m_Description;
				double m_LabelScale;
				bool m_ShowLabel;
				XMLNODE *m_Node;
				
			};

	//	struct TILEROW {
		//	SIMPLE_ARRAY<bool> IsLoaded;
		//	}

		class ZOOMLEVEL {
			public:

				ZOOMLEVEL (GRE::LAYER_OSM *Layer, int level);

				~ZOOMLEVEL ();

				int DataTipGetString (ELEMTYPE ElemType, ELEMNUM ElemNum, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images);

				ERRVALUE Draw2D (DRECT2D& area, GRE::LAYERDC& layerdc, bool update=true);

				GRE::LAYER_OSM *m_Layer;
				INT32 m_Level;
				SIMPLE_ARRAY<XMLDOC*> m_Docs;
				XMLDOC m_XmlDoc;
				POLYLINE m_Locations;
				POLYLINELIST m_Paths;
				MILIST<REGION2D> m_Areas;
				MILIST<GRE::LAYER_OSM::OSMELEMSTYLE> m_PolyStyles;
				MILIST<GRE::LAYER_OSM::OSMELEMSTYLE> m_PointStyles;
				MILIST<LINESTYLE> m_LineStyles;
				REGION2D m_CurrentRegion; 
				MIMATRIX m_IsLoaded;
			//	SIMPLE_ARRAY<GRE::LAYER_OSM::TILEROW*> m_Tiles;
				SIMPLE_ARRAY<XMLNODE*> Relations;
				SIMPLE_ARRAY<XMLNODE*> Ways;
				SIMPLE_ARRAY<XMLNODE*> Nodes;
				SIMPLE_ARRAY<XMLNODE*> Tags;
				SIMPLE_ARRAY <XMLNODE*> Members;
				XMLNODE *m_RootNode;
				QUADTREE *m_SearchTree;

			private:

				ERRVALUE AddElements ( XMLNODE* RootNode, DRECT2D& extents );
			};

		DISPPARM m_dispparm;
		FILEPATH m_pathkmz;								// Path to KMZ file if used, may be a temporary file if source is remote
		DRECT2D m_LastRequestArea;  
		INT32 m_CurrentZoom;
		SIMPLE_ARRAY<ZOOMLEVEL*> m_Zooms;
		MILIST<RVC::STYLEPATTERN_BITMAP> m_IconBitmaps;
		FILEPATH m_Basepath;
		MISTRINGLIST m_IconUrls;
		INT32 m_LastNumRecords;
		INT32 m_SearchNumAdded;
	#ifdef USE_MGUI
		DLG *m_pDlgControls;
	#endif
		GRE::LEGEND_POLYGON m_Legend;
		double m_TimeLastUpdate;
		DATETIME DateTime;

		static void GRECB_Layer (const GRE::MSG *msg, void *cbdata);	
		
		ERRVALUE GetOSM ( const DRECT2D& view,const FILEPATH& file, XMLDOC* xmldoc, int level);
		ERRVALUE GetRealResourcePath (const MISTRING& src, MISTRING& actual) const;
	 
		ERRVALUE Update ();
		void UnLoad();

		// Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
	 	virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const GRE::DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (ELEMTYPE ElemType, ELEMNUM ElemNum, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images);
		virtual bool v_DataTipHasAvailable () const;
		virtual ERRVALUE v_DataTipSetParms (int component, const GRE::DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual bool v_HasProperty (PROPERTY property) const;
	#ifdef USE_MGUI
	 	virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		LAYER_OSM (const LAYER_OSM&);
		LAYER_OSM& operator= (const LAYER_OSM&);
		
		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace GRE

#endif	// INC_GRE_LROSM_H
