/**
 * \file gre/rendertoimage.h
 * \brief GRE::RENDERTOIMAGE definitions
 *
 * \if NODOC
 * $Id: rendertoimage.h_v 1.2 2014/03/18 15:37:29 mju Exp $
 *
 * $Log: rendertoimage.h_v $
 * Revision 1.2  2014/03/18 15:37:29  mju
 * Add setting to ignore scale ranges.
 *
 * Revision 1.1  2013/02/26 18:16:04  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_RENDERTOIMAGE_H
#define	INC_GRE_RENDERTOIMAGE_H

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class STEREORENDER;
namespace GRE {
	class VIEWABLE;
	}
namespace RVC {
	class OBJITEM;
	}
#endif


namespace GRE {

//=====================================================================================================================

//! Used for rendering to single image.
class RENDERTOIMAGE {
	public:

		//! Rendering parameters.
		class PARAMETERS {
			public:

				//! Construct with both image dimension and cell size.
				PARAMETERS (
					COLORSPACE ColorSpace,
					INT32 ImageColumns,
					INT32 ImageRows,
					double CellSize
					);

				//! Construct with image dimensions.
				PARAMETERS (
					COLORSPACE ColorSpace,
					INT32 ImageColumns,
					INT32 ImageRows
					);

				//! Construct with cell size.
				PARAMETERS (
					COLORSPACE ColorSpace,
					double CellSize
					);

				~PARAMETERS ();

				const COLOR& GetBackgroundColor () const
					{ return (m_BackgroundColor); }

				double GetCellSize () const
					{ return (m_CellSize); }

				const COLOR_ARRAY* GetColorPalette () const
					{ return (m_pPalette); }

				COLORSPACE GetColorSpace () const
					{ return (m_ColorSpace); }

				const DRECT2D& GetExtentsView () const
					{ return (m_ExtentsView); }

				MGD::PIXEL GetFallbackPixel () const
					{ return (m_FallbackPixel); }

				bool GetIgnoreScaleRanges () const
					{ return (m_bIgnoreScaleRanges); }

				INT32 GetImageColumns () const
					{ return (m_ImageColumns); }

				INT32 GetImageRows () const
					{ return (m_ImageRows); }

				bool GetSaveGeoreference () const
					{ return (m_SaveGeoreference); }

				const STEREORENDER* GetStereoValues () const
					{ return (m_pStereoValues); }

				bool GetUseClosestColorIfNotInPalette () const
					{ return (m_UseClosestIfNotInPalette); }

				int GetViewNum () const
					{ return (m_ViewNum); }

				bool HasColorPalette () const
					{ return (m_pPalette != 0); }

				bool IsDrawingStereo () const
					{ return (m_StereoDrawing && m_pStereoValues != 0); }

				void SetBackgroundColor (
					const COLOR& BackgroundColor
					) { m_BackgroundColor = BackgroundColor; }

				void SetExtents (
					const DRECT2D& ExtentsView
					) { m_ExtentsView = ExtentsView; }

				void SetIgnoreScaleRanges (
					bool bIgnoreScaleRanges = true
					) { m_bIgnoreScaleRanges = bIgnoreScaleRanges; }

				void SetPalette (
					const COLOR_ARRAY* pPalette,					//!< Pointer to palette, will be retained, NULL to dynamically create
					bool UseClosestIfNotInPalette = false,		//!< Use closest palette entry if not exact match
					MGD::PIXEL *pFallbackPixel = 0				//!< Pointer to fallback pixel, NULL to use 'transparent'
					);

				void SetSaveGeoreference (
					bool SaveGeoreference
					) { m_SaveGeoreference = SaveGeoreference; }

				void SetStereoDrawing (
					bool StereoDrawing = true
					) { m_StereoDrawing = StereoDrawing; }

				void SetStereoValues (
					const STEREORENDER& StereoValues
					);

				void SetViewNum (
					int ViewNum
					) { m_ViewNum = ViewNum; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COLORSPACE m_ColorSpace;
				INT32 m_ImageColumns;
				INT32 m_ImageRows;
				double m_CellSize;
				COLOR m_BackgroundColor;
				int m_ViewNum;
				const COLOR_ARRAY *m_pPalette;
				MGD::PIXEL m_FallbackPixel;
				DRECT2D m_ExtentsView;
				STEREORENDER *m_pStereoValues;
				bool m_SaveGeoreference;
				bool m_UseClosestIfNotInPalette;
				bool m_StereoDrawing;
				bool m_bIgnoreScaleRanges;

				void InitCommon ();
				#endif
			};

		//! Render "viewable" to RVC raster.
		static ERRVALUE RenderToRaster (
			VIEWABLE *pViewable,						//!< Viewable to render
			RVC::OBJITEM& objitem,					//!< Target raster location
			const PARAMETERS& parameters			//!< Rendering parameters
			);
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_RENDERTOIMAGE_H
