/**
 * \file gre/uiformgroupclipping.h
 * \brief GRE::UI::FORM_GROUP_CLIPPING definitions
 *
 * \if NODOC
 * $Id: uiformgroupclipping.h_v 1.2 2014/01/16 20:27:28 mju Exp $
 *
 * $Log: uiformgroupclipping.h_v $
 * Revision 1.2  2014/01/16 20:27:28  mju
 * Dox fix.
 *
 * Revision 1.1  2013/03/20 15:10:19  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIFORMGROUPCLIPPING_H
#define	INC_GRE_UIFORMGROUPCLIPPING_H

#ifndef	INC_MGUI_FORMEXTENTS_H
#include <mgui/formextents.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class GROUP;
	class MSG;
	}
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

//! Form to control group clipping.
class FORM_GROUP_CLIPPING : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Default =						0,
			FORMSTYLE_IncludeMatchAttached =		0x01,		//!< Include control to match geographically attached group
			FORMSTYLE_NoSetFromActiveLayer =		0x02,		//!< Don't include button to set from active layer
			};

		//! Constructor.
		FORM_GROUP_CLIPPING ();

		//! Destructor.
		virtual ~FORM_GROUP_CLIPPING ();

		//! Create form with optional initial group.
		//! If no group is set the form will be disabled.
		void Create (
			MGUI::LAYOUT_PANE_BASE& paneParent,			//!< Parent pane for form
			GRE::GROUP *group = 0,							//!< Optional group to initially set
			FORMSTYLE formstyle = FORMSTYLE_Default
			);

		//! Determine if all required values have been set.
		bool HasAllValues ();

		//! Determine if values have been changed since last updated to group.
		bool IsChanged () const
			{ return (m_bIsChanged); }

		//! Set delegate to notify when any change made.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChange = delegate; }

		//! Set whether form is enabled.
		virtual void SetEnabled (
			bool enabled = true
			);

		//! Set group, updating controls to show its current settings.
		//! Any previous settings that have not been updated to the group will be lost.
		void SetGroup (
			GRE::GROUP *group									//!< Group, will disable form if null
			);

		//! Update group from form.
		void UpdateGroup (
			bool notify = true								//!< Send change notification for group
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef FORM_COMPOSITE BASECLASS;
		typedef FORM_GROUP_CLIPPING THISCLASS;

		MGUI::CTRL_TOGGLEBUTTON m_TglClip;
		MGUI::CTRL_TOGGLEBUTTON m_TglMatchAttached;
		MGUI::FORM_EDIT_EXTENTS m_EditExtents;
		MGUI::CTRL_PUSHBUTTON m_BtnSetFromActiveLayer;
		DELEGATE_VOID_NOPARMS m_DelegateOnChange;
		GROUP *m_group;
		bool m_bIsChanged;

		static void GRECB_Group (const GRE::MSG *pMsg, void *vpForm);

		void OnAnyChange ();
		void OnBtnSetFromActiveLayer ();
		void OnGreMsgGroup (const GRE::MSG& msg);
		void OnTglClip ();

		#endif
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_GROUP_CLIPPING::FORMSTYLE)
#endif

//=====================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIFORMGROUPCLIPPING_H
