/**
 * \file uitin.h <gre/uitin.h>
 * \brief GRE User Interface definitions for TIN layer
 *
 * \if NODOC
 * $Id: uitin.h_v 1.6 2013/02/04 20:39:21 mju Exp $
 *
 * $Log: uitin.h_v $
 * Revision 1.6  2013/02/04 20:39:21  mju
 * Add dlg defn.
 *
 * Revision 1.1  2002/09/09 20:30:11  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UITIN_H
#define	INC_GRE_UITIN_H

#ifndef	INC_GRE_LRTIN_H
#include <gre/lrtin.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace GRE {

//===================================================================================================================

//! Dialog for modifying TIN layer display settings.
class LAYER_TIN::DISPPARM::DLG : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG (
			LAYER_TIN *layer,					//!< Associated layer
			bool Include3D						//!< Include 3D settings
			);

		//! Destructor.
		virtual ~DLG ();

		//! Get 'mask' for which LAYERDRAWFLAG values are settable on dialog.
		LAYERDRAWFLAGS GetLayerDrawFlagMask () const
			{ return (LAYERDRAWFLAG_None); }

		//! Get current LAYERDRAWFLAGS settings.
		LAYERDRAWFLAGS GetLayerDrawFlags () const;

		//! Get current DISPPARM values.
		const LAYER_TIN::DISPPARM& GetValues () const;

		//! Set DISPPARM and LAYERDRAWFLAGS values.
		ERRVALUE SetValues (
			const LAYER_TIN::DISPPARM& values,
			LAYERDRAWFLAGS LayerDrawFlags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		LAYER_TIN *m_layer;

		void ApplyChanges (bool ForceRedraw);

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual void v_OnOpen ();

		friend class GRE::LAYER_TIN::DISPPARM::DLG::PRIV;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_UITIN_H
