/**
 * \file gre/viewable.h
 * \brief GRE::VIEWABLE class definition
 *
 * \if NODOC
 * $Id: viewable.h_v 1.70 2014/12/09 16:30:14 mju Exp $
 *
 * $Log: viewable.h_v $
 * Revision 1.70  2014/12/09 16:30:14  mju
 * Add useRenderProjection to getSizeMeters.
 *
 * Revision 1.69  2014/12/08 14:58:28  mju
 * Add ifdef use_mgui.
 *
 * Revision 1.68  2013/03/21 21:32:31  mju
 * Remove getFullName.
 * Add ability to keep track of "settings" dialog.
 *
 * Revision 1.67  2013/02/27 17:37:06  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.66  2013/02/27 15:45:26  mju
 * Add updateDft parm to setDesignScale.
 *
 * Revision 1.65  2013/02/26 18:47:12  mju
 * Remove parm_rendertoraster and renderToRaster method.
 *
 * Revision 1.62  2013/02/14 22:44:09  mju
 * Use container base class.
 *
 * Revision 1.60  2013/02/07 18:00:13  mju
 * Add save method and corresponding v_save.
 * Rename all virtual methods to have v_ prefix.
 *
 * Revision 1.59  2012/03/29 21:46:13  mju
 * Use stereorender class.
 *
 * Revision 1.57  2011/02/01 15:59:24  bcolombe
 * Added getNoRenderLayerList
 *
 * Revision 1.56  2011/01/18 00:05:05  bcolombe
 * Add members to MAINSCRIPT for allowing encrypted scripts
 *
 * Revision 1.55  2010/01/27 18:17:40  bcolombe
 * Added DlgRenderToTileset
 *
 * Revision 1.50  2007/11/26 18:43:07  mju
 * Add getActiveLayer, unhighlightAllElements.
 *
 * Revision 1.48  2007/01/19 16:08:36  mju
 * Use color_array.
 *
 * Revision 1.46  2006/10/04 14:57:11  mju
 * Add dlgRenderToSVG.
 *
 * Revision 1.40  2006/05/05 17:46:54  mju
 * Add dlgRenderToKML.
 *
 * Revision 1.39  2006/02/16 15:02:03  mju
 * Add dlgEditScaleRanges.
 *
 * Revision 1.37  2005/11/28 18:54:00  mju
 * Add getMax/MinScaleVisibleAny.
 *
 * Revision 1.15  2004/01/12 21:20:52  mju
 * Add MAINSCRIPT.
 *
 * Revision 1.1  2003/06/16 19:40:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEWABLE_H
#define	INC_GRE_VIEWABLE_H

#ifndef	INC_GRE_CONTAINER_H
	#include <gre/container.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef	INC_MI32_MATTE_H
	#include <mi32/matte.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class STEREORENDER;
namespace MGUI {
	class DLGSHELL;
	}
#endif

namespace GRE {
//===================================================================================================================

//! Base class for objects that can have "views" (GRE::LAYOUT, GRE::GROUP).
class VIEWABLE : public GRE::CONTAINER {
	public:

	typedef fastdelegate::FastDelegate<ERRVALUE(SERIALIZER&,VIEWABLE*)> DELEGATE_ONSAVE;

	#ifdef USE_MGUI
		class MAINSCRIPT : public SERIALIZABLE {
			public:
				//! Externally callable functions.
				enum FUNC {
					FUNC_OnViewDataTipShowRequest,				//!< parm[GRE::VIEW,POINT2D,TOOLTIP] ret[-1=noshow,0=builtin,1=show]
					FUNC_OnViewDataTipShowText,					//!< parm[GRE::VIEW,POINT2D,TOOLTIP] ret[-1=noshow,0=builtin,1=show]
					FUNC_OnGroupDrawLabelsBegin,					//!< parm[GRE::GROUP,GRE::VIEW]
					FUNC_OnGroupDrawLabelsEnd,						//!< parm[GRE::GROUP,GRE::VIEW]
					FUNC_OnLayerDrawLabelsBegin,					//!< parm[GRE::LAYER,GRE::VIEW]
					FUNC_OnLayerDrawLabelsEnd,						//!< parm[GRE::LAYER,GRE::VIEW]
					FUNC_OnDrawLabel,									//!< parm[LABELPLACE, GRE::LAYERDC,COLOR,Num] ret[0=builtin,1=drawn by script]
					FUNC_COUNT
					};

				MAINSCRIPT (VIEWABLE *viewable);
				virtual ~MAINSCRIPT ();
				int CallFunction (
					FUNC func,
					SML::ARG *ArgIn,
					int NumArg,
					SML::ARG *ArgRet = 0
					);
				int CallFunction (
					FUNC func,
					int NumArg,
					void *ArgPtr1,
					void *ArgPtr2 = 0,
					void *ArgPtr3 = 0
					);
				void Destroy ();
				int DlgEdit (MDLGPARENT dlgparent);
				bool HasFunction (FUNC func) const;
				bool HasScript () const;
				ERRVALUE Initialize ();
				void Terminate ();
			private:
				class CONTEXT;
				VIEWABLE *m_viewable;
				CONTEXT *m_pContext;
				MISTRING m_String;
				MISTRING m_DisplayString;
				bool m_IsEncrypted;
				MISTRING m_ScriptPath;

				ERRVALUE CreateSMLContext (SML::CONTEXT *& smlcontext);
				static void GRECB_CtrlScript (const GRE::MSG *pMsg, void *vpCtrlScript);
				static const SERIALIZERITEM* SerialGetItemDef ();
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;
				friend class MAINSCRIPT::CONTEXT;
			};
	#endif

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT* context
			);

		//! Destructor.
		virtual ~VIEWABLE ();

		MATTE& AccessMatte ()
			{ return (m_Matte); }

		//! Attach "settings" dialog.
		//! Any previously attached dialog will be deleted.
		void AttachDlgSettings (
			MGUI::DLGSHELL *pDlg				//!< Dialog to attach, can be null
			);

		//! Clear contents of viewable, resetting to default state.
		void Clear ();

		//! Clear save location specification.
		void ClearSaveLocation ()
			{ m_SaveObjItem.Clear(); }

	#ifdef USE_MGUI
		// Open dialog to edit 'control script' if allowed.
		ERRVALUE ControlScriptEdit (
			MDLGPARENT dlgparent
			);
	#endif

		//! Destroy all views of this object.
		void DestroyAllViews ();

		//! Draw all views of this object.
		ERRVALUE DrawAllViews (
			GRE::DRAWCONDITION condition = GRE::DRAWCONDITION_Always
			) { return (v_DrawAllViews(condition)); }

		//! Find group with georeference.
		//! Will return first group with a non-local CRS.  If none such, and AllowLocalCRS is specified, will return first non-ignored group.
		GROUP* FindGroupGeoref (
			bool AllowLocalCRS
			) { return (v_FindGroupGeoref(AllowLocalCRS)); }

	#ifdef USE_MGUI
		//! Get reference to "display control script".
		MAINSCRIPT& GetControlScript ()
			{ return (m_CtrlScript); }
	#endif

		//! Get overall viewable extents.
		const DRECT2D& GetExtents () const
			{ return (v_GetExtents()); }

		//! Get 'design scale' for layout or group.
		double GetDesignScale () const
			{ return (v_GetDesignScale()); }

	#ifdef USE_MGUI
		//! Get pointer to "settings" dialog instance if any.
		MGUI::DLGSHELL* GetDlgSettings () const
			{ return (m_pDlgSettings); }
	#endif

		//! Get element highlighting mode.
		GRE::HIGHLIGHTMODE GetElemHighlightMode () const
			{ return (GetTopViewable()->m_ElemHighlightMode); }

		//! Get first view of this object.
		//! @return Pointer to view or NULL if no views of this object.
		//!
		//! To loop through all views for an object use this method to get the first view.
		//! Then use the GRE::VIEW::GetNext() to obtain the next view, checking for NULL as shown:
		//!
		//!		for (GRE::VIEW *view = object->GetFirstView(); (view != 0); view = view->GetNext()) {
		//!			...
		//!			}
		GRE::VIEW* GetFirstView () const;

		//! Get name of object to use in Ini file access.
		const char* GetIniName () const
			{ return (m_IniName); }

	#ifdef USE_MGUI
		//! Get MacroScript list.
		GRE::MACROSCRIPTLIST& GetMacroScriptList ()
			{ return (*m_pMacroScriptList); }
	#endif

		//! Get matte values.
		const MATTE& GetMatte () const
			{ return (m_Matte); }

		//! Get maximum visible scale for any item in viewable.
		//! @return 0 if at least one item has no maximum scale set.
		double GetMaxScaleVisibleAny () const
			{ return (v_GetMaxScaleVisibleAny()); }

		//! Get minimum visible scale for any item in viewable.
		double GetMinScaleVisibleAny () const
			{ return (v_GetMinScaleVisibleAny()); }

		//! Get array of layers that can't be rendered.
		void GetNoRenderLayerList (
			SIMPLE_ARRAY<LAYER*>& layers		//!< Array of layer pointers returned
			) const;

		//! Get path to file where viewable saved.
		const FILEPATH& GetSaveFilePath () const
			{ return (m_SaveObjItem.GetFilePath()); }

		//! Get object item reference to save location.
		const RVC::OBJITEM& GetSaveObjItem () const
			{ return (m_SaveObjItem); }

		//! Compute dimensions of object based on scale and georeference.
		ERRVALUE GetSizeMeters (
			DPOINT2D& sizemeters,						//!< Size returned in meters
			bool bUseRenderProjection = false		//!< Base on projection used for rendering instead of ground extents
			) { return (v_GetSizeMeters(sizemeters,bUseRenderProjection)); }

	#ifdef USE_MGUI
		//! Get ToolScript list.
		GRE::TOOLSCRIPTLIST& GetToolScriptList ()
			{ return (*m_pToolScriptList); }
	#endif

		//! Get topmost containing viewable (const).
		const VIEWABLE* GetTopViewable () const;

		//! Get topmost containing viewable (non-const).
		VIEWABLE* GetTopViewable ();

		//! Get view of this object given view number.
		//! @return Pointer to view or NULL if no matching view.
		GRE::VIEW* GetView (
			int ViewNum
			) const;

		//! Determine if save location has been specified.
		bool HasSaveLocation () const;

		//! Determine if changes have been made since last save.
		bool IsModified () const
			{ return (m_IsModified); }

		//! Determine if is topmost viewable.
		bool IsTopViewable () const
			{ return (this == GetTopViewable()); }

		//! Reset "is modified" status to "false".
		void ResetModified (
			) { m_IsModified = false; }

		//! Save to object, will prompt if requested or no location set.
		ERRVALUE Save (
			MDLGPARENT dlgparent,						//!< Parent for prompt dialog
			bool forceprompt = false					//!< Force to prompt (for save-as)
			) { return (v_Save(dlgparent,forceprompt)); }

		//! Set delegate to allow user to save viewable.
		void SetDelegateOnSave (
			DELEGATE_ONSAVE delegate
			) { m_DelegateOnSave = delegate; }

		//! Set 'design scale' for layout or group.
		void SetDesignScale (
			double DesignScale,							//!< Design scale to set
			bool notify = true,							//!< Send notification if changed
			bool updateDft = false						//!< Whether to update default, should only be true if was result of user entering value
			) { v_SetDesignScale(DesignScale,notify,updateDft); }

		//! Set element highlighting mode.
		void SetElemHighlightMode (
			GRE::HIGHLIGHTMODE HighlightMode
			);

		//! Set base name of object for use in Ini file acces.
		void SetIniBase (
			const char* name
			);

		//! Set that has been modified since last saved.
		void SetIsModified ()
			{ m_IsModified = true; }

	#ifdef USE_MGUI
		//! Set MacroScript list.
		void SetMacroScriptList (
			const GRE::MACROSCRIPTLIST& ScriptList
			);
	#endif

		//! Set path to file where viewable saved.
		//! Also resets object such that expects to save to non-RVC file.
		void SetSaveFilePath (
			const FILEPATH& SaveFilePath
			);

		//! Set object item reference to save location.
		void SetSaveObjItem (
			const RVC::OBJITEM& SaveObjItem
			);

	#ifdef USE_MGUI
		//! Set ToolScript list.
		void SetToolScriptList (
			const GRE::TOOLSCRIPTLIST& ScriptList
			);
	#endif

	#ifdef USE_MGUI
		//! Get all TreeViews which may contain this viewable.
		void TreeViewGetAll (
			SIMPLE_ARRAY<GRE::CTRL_TREEVIEW*>& TreeViewPtrs
			) const;
	#endif

	#ifdef USE_MGUI
		//! Set TreeView pointer.
		//! Used by GRE CTRL_TREEVIEW implementation, should not be called directly.
		void TreeViewSetPtr (
			GRE::CTRL_TREEVIEW *pTreeView
			) { m_pTreeView = pTreeView; }
	#endif

		//! Update user interface for changes to view scripts.
		void UpdateViewScriptUI ();

		//! Unhighlight all "marked" elements.
		void UnhighlightAllElements (
			bool redrawviews
			) { v_UnhighlightAllElements(redrawviews); }

	protected:

		MATTE m_Matte;

		//! Get serialization item definition array.
		static const SERIALIZERITEM* TemplateGetItemDef ();

		//! Constructor.
		VIEWABLE (
			OBJECT::TYPE type,									//!< Object type
			CONTAINER& Container,								//!< Container
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< List position
			GRE::OBJECT *pRefObject = 0						//!< Reference object for LISTPOS_Before/After
			);

		// Initialize 'control script' if exists and is allowed.
		ERRVALUE ControlScriptInitialize ();

		//! Get unique index for building Ini name.
		UINT8 GetIniIndex () const
			{ return (m_IniIndex); }

		ERRVALUE NotifyDelegateOnSave (
			SERIALIZER& serializer
			) { if (m_DelegateOnSave) return (m_DelegateOnSave(serializer,this)); else return (0); }

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef (
			SERIALIZER&
			) { return (TemplateGetItemDef()); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		char m_IniName[20];								// Name of object for use in Ini file access
		UINT8 m_IniIndex;									// Unique index of object for use in Ini file access
		UINT8 m_IniBaseLen;								// Length of base name not including appended index number
		bool m_IsModified;								// Changes have been made since last save
		RVC::OBJITEM m_SaveObjItem;					// Object item where saved
		GRE::HIGHLIGHTMODE m_ElemHighlightMode;
		DELEGATE_ONSAVE m_DelegateOnSave;
	#ifdef USE_MGUI
		MAINSCRIPT m_CtrlScript;
		GRE::CTRL_TREEVIEW *m_pTreeView;
		GRE::TOOLSCRIPTLIST *m_pToolScriptList;		// ToolScripts saved with this object
		GRE::MACROSCRIPTLIST *m_pMacroScriptList;		// MacroScripts saved with this object
		MGUI::DLGSHELL *m_pDlgSettings;
	#endif

	#ifdef USE_MGUI
		static ERRVALUE SerialCB_MacroScript (SERIALIZER&, const SERIALIZERITEM*, VIEWABLE *viewable, SERIALIZERITEM::ACTION);
		static ERRVALUE SerialCB_ToolScript (SERIALIZER&, const SERIALIZERITEM*, VIEWABLE *viewable, SERIALIZERITEM::ACTION);
	#endif

		// VIEWABLE overridables.
		virtual void v_Clear () = 0;
		virtual ERRVALUE v_DrawAllViews (GRE::DRAWCONDITION condition) = 0;
		virtual GROUP* v_FindGroupGeoref (bool AllowLocalCRS) = 0;
		virtual double v_GetDesignScale () const = 0;
		virtual const DRECT2D& v_GetExtents () const = 0;
		virtual double v_GetMaxScaleVisibleAny () const = 0;
		virtual double v_GetMinScaleVisibleAny () const = 0;
		virtual ERRVALUE v_GetSizeMeters (DPOINT2D& sizemeters, bool bUseRenderProjection) = 0;
		virtual ERRVALUE v_Save (MDLGPARENT dlgparent, bool forceprompt) = 0;
		virtual void v_SetDesignScale (double DesignScale, bool notify, bool updateDft) = 0;
		virtual void v_UnhighlightAllElements (bool redraw) = 0;

		VIEWABLE (const VIEWABLE&);
		VIEWABLE& operator= (const VIEWABLE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End VIEWABLE

//===================================================================================================================

//! Dialog to edit scale ranges for items in viewable.
//! @return TRUE if any changes, FALSE if not, < 0 if error or cancel
int DlgEditScaleRanges (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable
	);

//! Dialog to render viewable to raster.
ERRVALUE DlgRenderToRaster (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to KML for Google Earth.
ERRVALUE DlgRenderToKML (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to PDF for Acrobat.
ERRVALUE DlgRenderToPDF (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to SVG.
ERRVALUE DlgRenderToSVG (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to tileset.
ERRVALUE DlgRenderToTileset (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//===================================================================================================================

}	// End namespace GRE


#endif	// INC_GRE_VIEWABLE_H
