/**
 * \file mgui/dlgmisc.h
 * \brief Miscellaneous MGUI dialog functions
 *
 * \if NODOC
 * $Id: dlgmisc.h_v 1.16 2017/02/21 22:40:00 mju Exp $
 *
 * $Log: dlgmisc.h_v $
 * Revision 1.16  2017/02/21 22:40:00  mju
 * Add dlgCameraIntrinsics.
 *
 * Revision 1.15  2014/10/21 17:58:51  mju
 * Add dlgSystemLanguage.
 *
 * Revision 1.14  2014/01/23 16:51:48  mju
 * Add dlgShowHelp.
 *
 * Revision 1.13  2013/06/10 16:32:10  mju
 * Add dlgColor.
 *
 * Revision 1.12  2013/06/10 14:56:09  mju
 * Add dlgSystemPreferences.
 *
 * Revision 1.11  2013/01/22 22:30:26  mju
 * Add dlgEditTextFileObject.
 *
 * Revision 1.10  2012/08/29 17:42:55  mju
 * Need const in prototype.
 *
 * Revision 1.9  2012/08/29 17:29:19  mju
 * Add dlgRetryFileRawOpen.
 *
 * Revision 1.8  2012/04/26 14:22:32  mju
 * Add dlgViewHttpStatistics.
 *
 * Revision 1.7  2011/07/20 14:01:58  mju
 * Remove unused fn.
 *
 * Revision 1.6  2011/05/03 13:33:00  mju
 * Forward-declare mistring.
 *
 * Revision 1.5  2011/05/02 19:14:28  ldyas
 * use register instead of set for httpauthdlg
 *
 * Revision 1.4  2011/05/02 18:44:07  ldyas
 * move in to MGUI name space
 *
 * Revision 1.3  2011/05/02 15:34:12  ldyas
 * added set http authentication dialog.
 *
 * Revision 1.2  2011/04/15 16:13:10  mju
 * Add dlgGetSystemFonts.
 *
 * Revision 1.1  2010/05/27 14:16:07  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGMISC_H
#define	INC_MGUI_DLGMISC_H

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_MI32_FILEACCESS_H
#include <mi32/fileaccess.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FILEPATH;
class MISTRING;
class AUTORETRY_FILE_RAW;
class CAMERAINTRINSICS;
#endif

namespace MGUI {

//! Dialog to let user set camera intrinsic parameters.
//! @return TRUE if values changes, FALSE if not, < 0 if cancel.
int DlgCameraIntrinsics (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	CAMERAINTRINSICS &values		//!< Values
	);

//! Dialog to let user choose a color.
int DlgColor (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	const MISTRING& dlgtitle,		//!< Title for dialog
	COLOR& color,						//!< Color passed/returned
	COLOR::CAPABILITY colorcap = COLOR::CAPABILITY_NoTransparency
	);

ERRVALUE DlgEditTextFileObject (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	bool IsModal						//!< Whether dialog is modal
	);

//! Dialog to select system font list.
int DlgGetSystemFonts (
	MDLGPARENT dlgparent				//!< Parent for dialog
	);

//! Show dialog for user to set host/realm specific username and password.
int DlgRegisterHTTPAuthentication (	 //!< uses GetDlgParent() default for parent
	const MISTRING& host,
	const MISTRING& realm
	);

//! Dialog to retry FILE_RAW::Open operation.
ERRVALUE DlgRetryFileRawOpen (
	const AUTORETRY_FILE_RAW& AutoRetry,
	const FILEPATH& filepath,
	FILE_ACCESS access,
	FILE_SHARE share
	);

//! Show help dialog with specified help ID.
void DlgShowHelp (
	MDLGPARENT dlgparent,		//!< Parent for dialog
	HELPID helpid
	);

//! Dialog to let user set TNT interface language.
//! If specified dlgparent is NULL assumed to be at process startup and will force restart if language changed.
ERRVALUE DlgSystemLanguage (
	MDLGPARENT dlgparent,		//!< Parent for dialog
	bool bAllowRestart
	);

//! Get main TNT system preferences.
ERRVALUE DlgSystemPreferences (
	MDLGPARENT dlgparent			//!< Parent for dialog
	);

//! Show dialog for viewing HTTP statistics.
void DlgViewHttpStatistics ();

}	// End namespace MGUI

#endif	// INC_MGUI_DLGMISC_H
