/**
 * \file mgui/formcolorpicker.h
 * \brief MGUI::FORM_COLORPICKER definitions
 *
 * \if NODOC
 * $Id: formcolorpicker.h_v 1.2 2015/07/13 17:58:09 mju Exp $
 *
 * $Log: formcolorpicker.h_v $
 * Revision 1.2  2015/07/13 17:58:09  mju
 * Add sizealign for create.
 *
 * Revision 1.1  2015/07/13 14:55:56  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMCOLORPICKER_H
#define	INC_MGUI_FORMCOLORPICKER_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Form for picking color from simple palette grid.
class FORM_COLORPICKER : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Default =				0x00,
			FORMSTYLE_CenterPaletteH =		0x01,	//!< Center palette horizontally
			FORMSTYLE_NoSelectPalette =	0x02,	//!< Don't allow user to select palette via RMB
			};

		//! Constructor.
		FORM_COLORPICKER ();

		//! Destructor.
		virtual ~FORM_COLORPICKER ();

		//! Create form with current user default palette.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,
			FORMSTYLE formstyle = FORMSTYLE_Default,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create form with specified palette.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,
			const MISTRING &PaletteName,
			FORMSTYLE formstyle = FORMSTYLE_Default,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get current color.
		const COLOR& GetColor () const;

		//! Prompt user to select palette.
		//! @return true if palette changed, false if not.
		bool SelectPalette (
			MDLGPARENT dlgparent								//!< Parent for dialog
			);

		//! Set color.
		void SetColor (
			const COLOR &color,								//!< Color to set
			bool notify											//!< Whether to notify delegate of change
			);

		//! Set delegate to call when color changed.
		void SetDelegateOnChangeColor (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when palette changed.
		void SetDelegateOnChangePalette (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set palette by name.
		void SetPalette (
			const MISTRING &PaletteName					//!< Palette name
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_COLORPICKER (const FORM_COLORPICKER&);
		FORM_COLORPICKER& operator= (const FORM_COLORPICKER&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_COLORPICKER::FORMSTYLE)
#endif

//=====================================================================================================================

}		// End namespace MGUI

#endif	// INC_MGUI_FORMCOLORPICKER_H
