/**
 * \file mgui/formrasterfilterparms.h
 *
 * \if NODOC
 * $Id: formrasterfilterparms.h_v 1.11 2014/12/30 14:42:41 mju Exp $
 *
 * $Log: formrasterfilterparms.h_v $
 * Revision 1.11  2014/12/30 14:42:41  mju
 * Doxygen fix.
 *
 * Revision 1.10  2014/04/18 17:17:13  bcolombe
 * Add ChangeType delegate
 *
 * Revision 1.9  2014/03/06 23:11:30  bcolombe
 * Remove header
 *
 * Revision 1.8  2014/02/26 17:44:43  bcolombe
 * Add SetNumPlaces
 *
 * Revision 1.7  2014/02/21 18:14:28  bcolombe
 * Add EnableKernel delegate
 *
 * Revision 1.6  2014/02/17 18:37:19  bcolombe
 * Add CreateParameters
 *
 * Revision 1.5  2014/02/15 18:27:10  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMRASTERFILTERPARMS_H
#define	INC_MGUI_FORMRASTERFILTERPARMS_H

#ifndef INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef INC_MI32_RASTERFILTERPARMS_H
#include <mi32/rasterfilterparms.h>
#endif

namespace MGUI {

//! Form for spatial filter gui controls
class FORM_RASTER_FILTER_PARMS : public MGUI::FORM_COMPOSITE {
	public:
		//! Form style options.
		enum FORMSTYLE {
			FORMSTYLE_Default =				0x00,
			FORMSTYLE_IncludeApplyButton =	0x01		//!< Add 'apply spatial filter to layer' button
			};

		FORM_RASTER_FILTER_PARMS ();

		virtual ~FORM_RASTER_FILTER_PARMS ();

		//! Create Form
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			FORMSTYLE Style = FORMSTYLE_Default
			);

		//! Create Form with separate pane for kernel
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			MGUI::LAYOUT_PANE_BASE& KernelPane,
			FORMSTYLE Style = FORMSTYLE_Default
			);

		//! Create addtional parameters form (scale, boost, offset)
		void CreateParameters (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get settings as RASTER_FILTER_PARMS
		void GetValues (RASTER_FILTER_PARMS& values);

		//! Set delegate to call when any value changes.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when kernels are enabled/disabled
		void SetDelegateOnEnableKernel (
			DELEGATE_VOID_BOOL delegate
			);

		//! Set delegate to call when type changed
		void SetDelegateOnChangeType (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set number of decimal places in kernel
		void SetNumPlaces (INT32 NumPlaces);

		//! Set initial values from RASTER_FILTER_PARMS
		void SetValues (const RASTER_FILTER_PARMS& values); 

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_RASTER_FILTER_PARMS (const FORM_RASTER_FILTER_PARMS&);
		FORM_RASTER_FILTER_PARMS& operator= (const FORM_RASTER_FILTER_PARMS&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_RASTER_FILTER_PARMS::FORMSTYLE)
#endif

}	// End namespace MGUI

#endif	// INC_MGUI_FORMRASTERFILTERPARMS_H