/**
 * \file mgui/formregionmanager.h
 * \brief MGUI::FORM_REGIONMANAGER definition
 *
 * \if NODOC
 * $Id: formregionmanager.h_v 1.9 2014/12/16 15:40:10 mju Exp $
 *
 * $Log: formregionmanager.h_v $
 * Revision 1.9  2014/12/16 15:40:10  mju
 * Have separate handlers for item deselected vs selected.
 *
 * Revision 1.8  2014/09/10 21:31:12  mju
 * Add action menu.
 *
 * Revision 1.7  2014/03/06 20:42:03  mju
 * Add link method.
 *
 * Revision 1.6  2014/03/06 14:55:57  mju
 * Add formstyle_ButtonsTop2.
 *
 * Revision 1.5  2014/03/04 21:22:22  mju
 * Allow multiple delegates for selection change.
 *
 * Revision 1.4  2013/10/29 21:48:35  mju
 * Add combinemode enum.
 *
 * Revision 1.3  2013/10/23 13:55:29  mju
 * Add nested form_savecrs class.
 *
 * Revision 1.2  2013/10/21 19:26:57  mju
 * Use regionsource.
 *
 * Revision 1.1  2013/10/21 18:49:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMREGIONMANAGER_H
#define	INC_MGUI_FORMREGIONMANAGER_H

#ifndef	INC_MGUI_TREEREGIONMANAGER_H
#include <mgui/treeregionmanager.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Region "manager" form, allows user to select, add, remove, save, combine regions.
class FORM_REGIONMANAGER : public FORM_COMPOSITE {
	public:

		enum FORMSTYLE {
			FORMSTYLE_ButtonsLeft =		0x00,		//!< Place buttons on left in single column (default)
			FORMSTYLE_ButtonsLeft2 =	0x01,		//!< Place buttons on left in two columns
			FORMSTYLE_ButtonsTop =		0x02,		//!< Place buttons on top in single row
			FORMSTYLE_ButtonsTop2 =		0x04,		//!< Place buttons on top in two rows
			FORMSTYLE_NoCombine =		0x08,		//!< Don't include "combine" toolbar buttons
			};

		//! Constructor.
		FORM_REGIONMANAGER ();

		//! Destructor.
		virtual ~FORM_REGIONMANAGER ();

		//! Add delegate for selection change.
		void AddDelegateOnChangeActive (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Create form.
		void Create (
			LAYOUT_PANE_BASE& paneParent,
			int treerows = 8,
			int treewidth = 30,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand,
			FORMSTYLE formstyle = FORMSTYLE_ButtonsLeft
			);

		//! Get currently active source, if any.
		REGIONSOURCE* GetActiveSource () const
			{ return (m_TreeRgnMgr.GetActiveSource()); }

		//! Determine if has "active" source (indicated by inverted text).
		bool HasActiveSource () const
			{ return (m_TreeRgnMgr.GetActiveSource() != 0); }

		//! Link to another region manager form.
		//! When the user makes a selection in one form, all other linked forms will
		//! have their selection changed to match.
		void Link (
			FORM_REGIONMANAGER *pFormLinkTo		//!< Form to link to, NULL to unlink this form
			);

		//! Remove delegate for selection change.
		void RemoveDelegateOnChangeActive (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set active source.
		void SetActiveSource (
			const REGIONSOURCE *pSource
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_REGIONMANAGER THISCLASS;

		enum CMDID {
			CMDID_Region_Add =		100,
			CMDID_Region_Save,
			CMDID_Region_Delete,
			CMDID_Region_Combine,
			CMDID_Action_Rename,
			CMDID_Action_RemoveIslands,
			CMDID_Action_CalcConvexHull,
			};

		enum COMBINEMODE {
			COMBINEMODE_None,
			COMBINEMODE_Union,
			COMBINEMODE_Intersect,
			COMBINEMODE_ExUnion,
			COMBINEMODE_Subtract
			};

		class FORM_SAVECRS;

		CTRL_TREE_REGIONMANAGER m_TreeRgnMgr;
		CTRL_TOOLBAR m_Toolbar;
		CTRL_TOOLBAR m_Toolbar2;
		MENU m_MenuAction;
		CTRL_TOOLBAR *m_pToolbarCombine;
		FORM_REGIONMANAGER *m_pLinkPrev;
		FORM_REGIONMANAGER *m_pLinkNext;
		MILIST<DELEGATE_VOID_NOPARMS> m_DelegateListOnChangeActive;
		CTRL_TOOLBAR::ITEMHND m_hndAction;
		COMBINEMODE m_CombineMode;

		ERRVALUE DoCmdAction (MGUI::ID cmdid);
		ERRVALUE DoCmdAdd ();
		ERRVALUE DoCmdCombine ();
		ERRVALUE DoCmdSave ();
		void HandleSelectActiveSource (REGIONSOURCE *pActiveSource);
		bool IsUserSourceNameInUse (const MISTRING& name) const;
		void OnMenuAction (MGUI::ID cmdid, bool bToggleOn);
		void OnToolbarCombine (MGUI::ID cmdid, bool bToggleOn);
		void OnToolbarRegion (MGUI::ID cmdid, bool bToggleOn);
		void OnTreeChangeCheckedList ();
		void OnTreeItemDeselected (CTRL_TREE::ITEMHND hndItem);
		void OnTreeItemSelected (CTRL_TREE::ITEMHND hndItem);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_REGIONMANAGER::FORMSTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMREGIONMANAGER_H
