/**
 * \file mgui/formselectnumset.h
 * \brief MGUI::FORM_SELECT_NUMSET
 *
 * \if NODOC
 * $Id: formselectnumset.h_v 1.4 2014/12/30 14:43:23 mju Exp $
 *
 * $Log: formselectnumset.h_v $
 * Revision 1.4  2014/12/30 14:43:23  mju
 * Doxygen fix.
 *
 * Revision 1.3  2014/09/03 14:54:39  mju
 * Change to keep list of items instead of using delegate.
 *
 * Revision 1.2  2014/06/19 19:55:49  mju
 * Add sizealign parm to create.
 *
 * Revision 1.1  2014/06/18 21:15:30  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMSELECTNUMSET
#define	INC_MGUI_FORMSELECTNUMSET

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

// Forward declarations.
#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGUI {
	class FORM_COMBOBOX;
	}
#endif

namespace MGUI {

//=====================================================================================================================

//! Form to select predefined set of N numbers.
class FORM_SELECT_NUMSET : public FORM_COMPOSITE {
	public:

		enum FLAGS {
			FLAG_None =					0x00,
			FLAG_AllowNone =			0x01,		//!< Include "none", will return number as -1
			FLAG_AllowDuplicate =	0x02,		//!< Allow same number to be used more than once
			};

		//! Default constructor.
		FORM_SELECT_NUMSET ();

		//! Destructor.
		virtual ~FORM_SELECT_NUMSET ();

		//! Add item for selection.
		//! Should be done before Create.
		void AddItem (
			const MISTRING& label				//!< Item label to appear in comboboxes
			);

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& pane,
			int SetSize,							//!< Size of "set" (how many items to select numbers for)
			const TEXTID *SetNames,				//!< Name of each "set", cannot be NULL, must have at least "SetSize" entries
			const INT16 *DftSetNums,			//!< Default number for each set, if NULL will have no default, otherwise must have at least "SetSize" entries
			FLAGS flags = FLAG_None,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get selected numbers.
		const SIMPLE_ARRAY<INT16> GetSetNums () const
			{ return (m_SetNums); }

		//! Set delegate to call when user changes selection.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChange = delegate; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_NUMSET THISCLASS;

		MISTRINGLIST m_ItemList;
		SIMPLE_ARRAY<INT16> m_SetNums;
		DELEGATE_VOID_NOPARMS m_DelegateOnChange;
		FORM_COMBOBOX *m_CbxSetNums;
		FLAGS m_flags;

		void OnCbxSetNum ();

		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMSELECTNUMSET
