/**
 * \file mgui/formselecttextencoding.h
 * \brief MGUI::FORM_SELECT_TEXTENCODING definitions
 *
 * \if NODOC
 * $Id: formselecttextencoding.h_v 1.1 2015/08/03 19:52:21 mju Exp $
 *
 * $Log: formselecttextencoding.h_v $
 * Revision 1.1  2015/08/03 19:52:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMSELECTTEXTENCODING_H
#define	INC_MGUI_FORMSELECTTEXTENCODING_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_CHARENCODING_H
#include <mi32/charencoding.h>
#endif

namespace MGUI {

//=====================================================================================================================

class FORM_SELECT_TEXTENCODING : public MGUI::FORM_COMPOSITE {
	public:

		//! Default constructor.
		FORM_SELECT_TEXTENCODING ();

		//! Destructor.
		virtual ~FORM_SELECT_TEXTENCODING ();

		//! Create form with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,		//!< Parent pane
			ENCODELIST_FLAGS EncodeListFlags,		//!< Flags for which encodings to include
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create form with label.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,		//!< Parent pane
			const MISTRING &label,						//!< Label string
			ENCODELIST_FLAGS EncodeListFlags,		//!< Flags for which encodings to include
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get current encoding selection.
		CHAR_ENCODING GetEncoding () const
			{ return (m_encoding); }

		//! Get label control if any.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel ()
			{ return (m_label); }

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnSelection = delegate; }

		//! Set selected encoding
		void SetEncoding (
			CHAR_ENCODING encoding,						//!< Encoding to set
			bool notify = true							//!< Notify delegate if selection changed
			);

		//! Set selected item to item prior to current selection.
		//! Intended to be used in delegate to revert selection on failure.
		void SetSelectedPrevious (
			bool notify = false							//!< Notify delegate if selection changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_TEXTENCODING THISCLASS;

		MGUI::CTRL_LABEL m_label;
		MGUI::CTRL_COMBOBOX m_cbx;
		DELEGATE_VOID_NOPARMS m_DelegateOnSelection;
		CHAR_ENCODING m_encoding;

		void OnCbx ();

		FORM_SELECT_TEXTENCODING (const FORM_SELECT_TEXTENCODING&);
		FORM_SELECT_TEXTENCODING& operator= (const FORM_SELECT_TEXTENCODING&);
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMSELECTTEXTENCODING_H
