/**
 * \file <mgui/keystate.h>
 * \brief MGUI::KEYSTATE definitions
 *
 * \if NODOC
 * $Id: keystate.h_v 1.4 2014/10/30 13:09:14 mju Exp $
 *
 * $Log: keystate.h_v $
 * Revision 1.4  2014/10/30 13:09:14  mju
 * Rename lock as numlock.
 *
 * Revision 1.3  2014/10/28 19:40:36  mju
 * Add other x-equivalent keystate values Mod1-5 and Lock.
 *
 * Revision 1.2  2011/09/01 17:35:00  mju
 * Exclude macro from doxygen.
 *
 * Revision 1.1  2007/11/29 20:52:42  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_KEYSTATE_H
#define	INC_MGUI_KEYSTATE_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Enum used to specify which keys are pressed during a message handler.
//! NOTE:  These are exactly equivilant to some Windows constants.
//! The Alt KEYSTATE is not here because there is no mask for it in X
enum KEYSTATE {
	KEYSTATE_None		= 0x0000,
	KEYSTATE_NumLock	= 0x0001,
	KEYSTATE_Ctrl		= 0x0002,
	KEYSTATE_Shift		= 0x0004,
	KEYSTATE_Mod1		= 0x0008,
	KEYSTATE_Mod2		= 0x0010,
	KEYSTATE_Mod3		= 0x0020,
	KEYSTATE_Mod4		= 0x0040,
	KEYSTATE_Mod5		= 0x0080,
	KEYSTATE_Control	= KEYSTATE_Ctrl
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::KEYSTATE)
#endif

}	// End namespace MGUI

#endif	// INC_MGUI_KEYSTATE_H
