/**
 * \file mi32/coordopcoded.h
 * \brief COORDOP_CODED class definitions.
 *
 * \if NODOC
 * $Id: coordopcoded.h_v 1.1 2013/05/10 18:52:13 mju Exp $
 *
 * $Log: coordopcoded.h_v $
 * Revision 1.1  2013/05/10 18:52:13  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COORDOPCODED_H
#define	INC_MI32_COORDOPCODED_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

// Forward declarations.
#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
namespace SPATREF {
	class COORDREFSYS;
	}
#endif


namespace SPATREF {

//=====================================================================================================================

//! Service for converting between 'coded' and non-coded reference systems.
//! A "coded" CRS encodes location values as a single string.
class CLASSLIBEXPORT COORDOP_CODED {
	public:

		//! Constructor.
		COORDOP_CODED ();

		//! Copy constructor.
		COORDOP_CODED (
			const COORDOP_CODED& rhs
			);

		//! Destructor.
		~COORDOP_CODED ();

		//! Assignment.
		COORDOP_CODED& operator= (
			const COORDOP_CODED& rhs
			);

		//! Convert DPOINT2D to code string.
		ERRVALUE ConvertToCode2D (
			const COORDREFSYS& PointCRS,				//!< Coordinate reference system to convert from
			const DPOINT2D& ipoint,						//!< Point to convert
			MISTRING& codestr								//!< Code string returned
			) const;

		//! Convert code string to extents.
		ERRVALUE ConvertToExtents (
			const MISTRING& codestr,					//!< String to parse
			const COORDREFSYS& PointCRS,				//!< Coordinate reference system to convert to
			DRECT2D& extents								//!< Extents returned
			) const;

		//! Convert code string to single 2D point.
		ERRVALUE ConvertToPoint2D (
			const MISTRING& codestr,					//!< String to parse
			const COORDREFSYS& PointCRS,				//!< Coordinate reference system to convert to
			DPOINT2D& opoint								//!< Point returned
			) const;

		//! Set coded coordinate reference system.
		ERRVALUE SetCoordRefSys (
			const COORDREFSYS& CodeCRS					//!< Code coordinate reference system
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
	private:
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace SPATREF

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_COORDOPCODED_H
