/**
 * \file mi32/kmlhelper.h
 * \brief MIKML::HELPER definitions
 *
 * \if NODOC
 * $Id: kmlhelper.h_v 1.9 2015/07/07 20:49:08 ldyas Exp $
 *
 * $Log: kmlhelper.h_v $
 * Revision 1.9  2015/07/07 20:49:08  ldyas
 * aske stage pointer size of symbol so the bitmap can be presized
 *
 * Revision 1.8  2015/05/29 20:58:19  ldyas
 * adjusted to doing url setup and file download in setbaseurl
 *
 * Revision 1.7  2015/05/29 20:11:26  ldyas
 * mos.
 *
 * Revision 1.6  2015/05/29 20:04:04  ldyas
 * added some documentation.
 *
 * Revision 1.5  2015/05/26 15:35:54  mju
 * Don't have public variables in helper class, add accessor methods.
 *
 * Revision 1.4  2015/05/14 18:09:47  ldyas
 * added StyleAttribute
 *
 * Revision 1.3  2015/04/15 20:24:29  ldyas
 * adjustments to work better with grelrkml
 *
 * Revision 1.2  2015/04/10 19:38:17  ldyas
 * added new methods.
 *
 * Revision 1.1  2015/03/19 13:53:18  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_KMLHELPER_H
#define	INC_MI32_KMLHELPER_H

#ifndef	INC_RVC_STYLEPATTERNBITMAP_H
#include <rvc/stylepatternbitmap.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif
 
#ifndef	INC_MI32_ELEMSTYLE_H
#include <mi32/elemstyl.h>
#endif 

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLNODE;
class POLYLINE;
struct DRECT2D;
#endif

namespace MIKML {

//=====================================================================================================================

//! Stores Styles for a single KML element
class ELEMSTYLE {
	public:
		POINTSTYLE m_PointStyle;
		POLYSTYLE m_PolyStyle;
		LINESTYLE m_LineStyle;
		TEXTSTYLE m_LabelStyle;
		INT32 m_IconIndex; //!< index of Icon within the bitmap list if any
		MISTRING m_Label; //!< label read from name attribute
		MISTRING m_Description; //!< body of datatip
		double m_LabelScale; //!< sizing for label
		bool m_ShowLabel; //!< Display the label string or not
		MISTRING m_StyleAttribute;

		ELEMSTYLE ();
		~ELEMSTYLE ();
		};

//=====================================================================================================================

//! "Helper" for reading KML.
class HELPER {
	public:

		//! Get coordinates from XML.
		static ERRVALUE GetCoordinates (
			const XMLNODE *nodeCoordinates,		//!< XML "coordinates" node
			POLYLINE &polyline,						//!< Coordinates returned, will have colinear vertices removed
			DRECT2D *pExtents = 0					//!< Optional extents passed/returned (will be extended if specified)
			);

		HELPER ();
		
		~HELPER ();

		void ClearIconBitmapList ()
			{ m_IconBitmaps.Clear(); }

		const MISTRING& GetBaseUrl () const
			{ return (m_BaseUrl); }

		//!	Passes Icon bitmaps back to KML Layer or Import process
		const MILIST<RVC::STYLEPATTERN_BITMAP>& GetIconBitmapList () const
			{ return (m_IconBitmaps); }


		//!	Passes Icon urls back to KML Layer
		const MISTRINGLIST& GetIconUrls () const 
			{ return (m_IconUrls); }

		
		//! Get Style from XMLNode
		ERRVALUE ProcessStyle (
			XMLNODE *Node, //!< XML "coordinates" node
			bool ispoint,  //!< indicates if the Element is a Point avoides unnecessary processing 
			ELEMSTYLE &CurrentStyle //!< Style currently to be undated based on Node
			);

		ERRVALUE SetBaseUrl ( //!< Shouild be done before Process Style is called show where image paths are relative to
			const MISTRING &url
			); 

		ERRVALUE SetupSrcImageStage ( //!< Used for icon images Downloads Icons from RemoteSrc
  			const MISTRING &src, //! file or web path from in KML   
			SPATMOD::IMAGE::STAGE_AUTOPTR &PtrStage
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MISTRINGLIST m_IconUrls;  
		MILIST<RVC::STYLEPATTERN_BITMAP> m_IconBitmaps;
		MISTRING m_BaseUrl;
		FILEPATH m_filepath;
		FILEPATH m_pathkmz;
		DOUBLE m_PointScale;
		bool m_IsGoogle;
		SIMPLE_ARRAY<DOUBLE> m_IconScales;

		 //!< transforms a relative src using the kmls baseUrl/path in to one towards the actual image
		//!< if the src is already a full http path returns it unchanged.
		ERRVALUE GetRealResourcePath ( 
			const MISTRING &src, //! relative src
			MISTRING &ret //! adjusted result absolute path
			);

		ERRVALUE ReadSymbol ( 
 			RVC::STYLEPATTERN_BITMAP &Bitmap, //!< Symbol Returned.
			const COLOR &color,	//!< color to paint over the symbol like google earth does 
  			const MISTRING &src, //!< image src 
			const DPOINT2D &hotspot, //<!< location of hot spot in symbol
			const MISTRING &unitsx, //!sizing in X directtion -> "fraction"/"pixels"/"insetPixels"
			const MISTRING &unitsy	//!sizing information -> "fraction"/"pixels"/"insetPixels"
			);

		HELPER (const HELPER &);
		HELPER& operator= (const HELPER &);

		#endif
	};

//=====================================================================================================================

}	// End namespace MIKML

#endif	// INC_MI32_KMLHELPER_H
