/**
 * \file mi32/mg2cmsui.h
 * \brief User Interface for selecting ICM settings
 *
 * Include file for User Interface for ICM Color Management classes
 *
 * \if NODOC
 * $Log: mg2cmsui.h_v $
 * Revision 1.10  2013/09/13 12:48:35  mju
 * Rename method.
 *
 * Revision 1.4  2003/06/13 20:28:37  dwilliss
 * Added definition for printer ICM pane
 *
 * Revision 1.1  2003/06/04 17:23:31  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_MG2CMSUI_H
#define INC_MI32_MG2CMSUI_H

#ifndef INC_MI32_MG2CMD_H
#include <mi32/mg2cms.h>
#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifndef INC_MGUI_LAYOUT_H
#include <mgui/layout.h>
#endif



class ICMPANE {
	public:

		ICMPANE ();

		virtual ~ICMPANE ();

		ERRVALUE Init (
			MGUI::LAYOUT_PANE_BASE& pane
			) { return (v_Init(pane)); }

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual ERRVALUE v_Init (MGUI::LAYOUT_PANE_BASE& pane) = 0;
		#endif
	};


class RGBICMPANE : public ICMPANE {
	public:
		RGBICMPANE();
		virtual ~RGBICMPANE();

		ERRVALUE SaveSettings ();

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;

		PRIV *m_pPriv;

		virtual ERRVALUE v_Init(MGUI::LAYOUT_PANE_BASE& pane);
		#endif
	};


class PRINTERICMPANE : public ICMPANE {
	public:
		PRINTERICMPANE();
		virtual ~PRINTERICMPANE();

		void GetParms (
			MGD::PROFILEPARMS_PRINTER&
			) const;

		void SetParms (
			const MGD::PROFILEPARMS_PRINTER&
			);

		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;

		PRIV *m_pPriv;

		virtual ERRVALUE v_Init(MGUI::LAYOUT_PANE_BASE& pane);
		#endif
	};

#endif // INC_MI32_MG2CMSUI_H
