/*****************************************************************************
 * \file mi32/mg2svg.h
 * \brief SVG declarations for context ... device 
 *
 * $Log: mg2svg.h_v $
 * Revision 1.110  2016/08/18 22:29:19  mju
 * Clean up, remove unneeded stuff.
 *
 * Revision 1.109  2016/01/15 20:31:45  ldyas
 * add elements for storing looked up font.
 *
 * Revision 1.107  2015/10/12 14:48:26  ldyas
 * added suppresshidden
 *
 * Revision 1.105  2015/10/06 21:53:47  ldyas
 * added region drawing methods and made rectangle drawing based on them .
 *
 * Revision 1.102  2013/10/24 14:28:16  ldyas
 * link tables added.
 *
 * Revision 1.101  2013/10/11 21:39:44  ldyas
 * svg field dbase support
 *
 * Revision 1.100  2013/10/11 18:50:16  ldyas
 * added layersettings support
 *
 * Revision 1.89  2012/03/23 13:40:09  ldyas
 * Add setSymbolDirectory.
 *
 * Revision 1.87  2011/12/21 23:07:34  bcolombe
 * Add HasElements,SetHasElements
 *
 * Revision 1.84  2011/09/16 16:45:06  ldyas
 * added scale factors
 *
 * Revision 1.82  2011/09/09 17:40:07  ldyas
 * added clipping margin
 *
 * Revision 1.63  2005/03/04 20:44:37  ldyas
 * added way to differentiate builtin symbols and custom ones
 *
 * Revision 1.50  2004/04/22 22:52:30  ldyas
 * pt enlargement control added
 *
 * Revision 1.48  2004/04/14 20:02:44  ldyas
 * added enlarge point symbols highlighting.
 *
 * Revision 1.47  2004/03/26 00:30:53  ldyas
 * link to scripts added.
 *
 * Revision 1.46  2004/03/24 19:09:14  ldyas
 * various including layer ctrl panel added.
 *
 * Revision 1.41  2004/02/20 22:38:22  ldyas
 * implemented entities for tool tip data.
 *
 * Revision 1.1  2003/07/31 23:07:09  ldyas
 * Initial revision
 */

#ifndef INC_MI32_MG2SVG_H
#define INC_MI32_MG2SVG_H

#ifndef	INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif

#ifndef INC_MG2KML_H
#include <mi32/mg2kml.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef INC_MI32_FILERAW_H
#include <mi32/fileraw.h>
#endif
 
#ifndef INC_GRE_BASE_H
#include <gre/base.h>
#endif

#ifndef INC_MI32_TRANS2DAFFINE_H
#include <mi32/trans2daffine.h>
#endif

#ifndef INC_MI32_FILELEGACY_H
#include <mi32/filelegacy.h>
#endif
 
#ifndef INC_RVC_DBTABSTY_H
#include <rvc/dbtabsty.h>
#endif

#ifndef INC_RVC_STYLE_H
#include <rvc/style.h>
#endif

#ifndef	INC_RVC_STYLEPATTERNHATCH_H
#include <rvc/stylepatternhatch.h>
#endif

#ifndef INC_RVC_DBPOLY_H
#include <rvc/dbpoly.h>
#endif

#ifndef INC_RVC_DBPOINT_H
#include <rvc/dbpoint.h>
#endif

#ifndef INC_RVC_DBLINE_H
#include <rvc/dbline.h>
#endif
 
#include <contrib/zlib.h>		// <<< XXX Public include files not allowed to include from contrib!


#ifndef GENERATING_DOXYGEN_OUTPUT

namespace RVC {
	class STYLEPATTERN_BITMAP;
	}
class REGION2D;
class SUBREGION2D;
class SVG;
class JPEGEXT_FILE;
struct png_struct_def;
struct png_info_struct;

#endif // GENERATING_DOXYGEN_OUTPUT


namespace MGD {

#ifndef GENERATING_DOXYGEN_OUTPUT
class CONTEXT_SVG;
#endif // GENERATING_DOXYGEN_OUTPUT

class DEVICE_SVG : public MGD::DEVICE {

	public:

		DEVICE_SVG ();

		virtual ~DEVICE_SVG ();

		void Clear ();

		//! Close the SVG.
		//! Closing the SVG puts it into a finished state. but doesn't write it to file.
		ERRVALUE Close (); 

		ERRVALUE Create (
			const FILEPATH &filename, //!< Required for image names and css when images/css are not made inline.
			double height, //!< height in inches,
			double width, //!< width in inches
			const MIUNICODE *title, //!< title
			UINT32 resolution //!< optional
			);

		ERRVALUE ElementBegin (
			GRE::LAYER *lp,
			const RVC::ELEMIDX &elemidx
			);
		
		ERRVALUE ElementEnd ();

		ERRVALUE GetString ( 
			STRUTF8 &str
			);

		ERRVALUE GroupBegin (
			GRE::GROUP *gp = 0
			);
		
		ERRVALUE GroupEnd (
			GRE::GROUP *gp = 0
			);

		bool HasElements () const
			{ return m_bHasElements; }

		void InitializeDevice (
			UINT32 height,
			UINT32 width
			);

		ERRVALUE LayerBegin (
			GRE::LAYER *lp = 0
			);

		ERRVALUE LayerEnd ();
 
		void SetDataTipOpacity (
			DOUBLE opacity
			) { m_DataTipOpacity = opacity; }

		void SetDefaultStrokeWidth ( //some browsers have poor support //used for tilesets
			DOUBLE width
			) { m_DefaultStrokeWidth = width; } 

		void SetDisplayToView (
			const MAT3X3 matrix
			);

		void SetDoHighPrecisionLines (
			bool usedoubles
			) { m_bDoFloat = usedoubles; }

		void SetEmbedFonts (
			bool embed
			) { m_bEmbedFonts = embed; }

		//! controls inline styles or 
		void SetEmbedImages (
		   bool embed
		   ) { m_bEmbedImages = embed; }

		void SetEmbedWithNameSpacePrefix ( 
			const MISTRING &prefix
			) { 
			m_nsPrefix.Clear();
			if(!prefix.IsEmpty()){ m_nsPrefix<<prefix<<":"; }
			m_bHTMLwrap = true; 
			}

		void SetEventsForWebClient ( 
			bool webscript 
			) { m_bWebClient = webscript; }

		//!optimize for Illustrator no scaling.
		void SetForIllustrator ( 
			bool unscaled 
			) { m_bForIllustrator = unscaled;}

		void SetHighlightColor (
			COLOR color
			) { m_HighlightColor = color; }

		void SetIncludeLayerControls (  
		   bool include
		   ) { m_bIncludeLayerCtrlScript = include; }

		void SetIncludeMeasurementTools (
			bool include
			) { m_bIncludeMeasurementScript = include; }

		void SetJPEGQuality (
			int quality //!< 25 to 100 
			) { m_JPEGQuality = quality;}

		void SetLayerSettings (
			const MGD::DEVICE_KML::LAYERSETTINGS& settings
			) {
			m_Settings = settings;
			m_LayerSettingsList.PushBack(settings);
			}

		//! link in stylesheet
		void SetLinkStylesheet ( 
			bool link
			) { m_bExternalStyles = link; }

		void SetLinkToScripts (
			bool link
			) { m_bExternalScripts = link; }

		void SetMargin ( //in units of pixels - actually svg internal coordinates
			DOUBLE margin
			) { m_MasterMargin = margin;}

		void SetResolution ( 
			UINT32 resolution
			) { m_vres = m_hres = resolution; }

		void SetScale ( //in units of pixels - actually svg internal coordinates
			DOUBLE scale
			) { m_MasterScale = scale;}

		//! include coordinate script
		void SetShowCoordinates (  
		   bool show
		   ) { m_bIncludeShowCoordScript = show; }

		void SetShowDatatips (
			bool show, //!< if not true the following are ignored
			bool blink,
			bool invert
			) { 
			m_bIncludeDatatipScript = show; 
			m_bBlinkHighlight = blink;
			m_bUseInverseHighlights = invert;
			}

		void SetSuppressingHidden (
			bool suppress 
			) { m_bSuppressingHidden = suppress; }

		void SetSymbolDirectory (
			const FILEPATH& dir
			) { m_SymbolDirectory = dir; }

		void SetSymbolEnlargement (
			DOUBLE enlargement
			) { m_SymbolEnlargement = enlargement;}

		void SetUseAdobeMenu (
			bool usemenu
			) { m_bUseAdobeMenu = usemenu;}

		void SetUseEntitiesInDataTips (
			bool use
			) { m_bUseEntitiesInDataTips = use;}

		void SetUseFreehandMeasures (
			bool use
			) { m_bUseFreehandMeasures = use; }

		//! controls SVGZ output
		void SetUseGZIP (  
		   bool gzip
		   ) { m_bGzip = gzip; }

		//! controls inline styles 
		void SetUseInlineStyles (  
		   bool inlines
		   ) { m_bInlineStyles = inlines; }

		void SetUseLayerControlPanel (
			bool usepanel
			) { m_bIncludeLayerCtrlPanel = usepanel; }

		//! each layer this defines whether to used png or the jpeg default
		void SetUsePNG ( 
			bool usepngs 
			) { m_bUsingPNG = usepngs; }

		void SetUseSymbolDefinitions ( //some browsers have poor support //used for tilesets
			bool usesymbols
			) { m_bUseSymbolDefinitions = usesymbols; }

		ERRVALUE Validate ();

		//! overrides the svg content with the str allows gzipping of arbitrary string as a svgz.
		ERRVALUE WriteToFile ( 
			const FILEPATH &outpath,
			const STRUTF8 &str,
			bool useGzip
			);

		//! may be different than that used in creation (results in linked files with different prefixes.
		ERRVALUE WriteToFile ( 
			const FILEPATH &outpath
			);


	private:
#ifndef GENERATING_DOXYGEN_OUTPUT

		struct FONTPAIR {
			TEXTSTYLE orig;
			TEXTSTYLE result;
			};

		struct SVGLAYER {
			int id;
			MISTRING name;
			MISTRING description;
			bool isGroup;
			UINT32 group;
			DPOINT2D size;
			};

		struct SVGFILTER {
			int id;
			COLOR color;
			double opacity;
			double x;
			double y;
			double height;
			};

		struct SVGSTYLE {
			MISTRING className;
			MISTRING code;
			};

		struct SVGSYMBOL {
			int number;
			int id;
			int layer;
			double width;
			double height;
			bool builtin;
			};

		class SVGPATTERN {
			public:

				SVGPATTERN ();

				~SVGPATTERN ();

				void GetName (
					MISTRING& name,
					const COLOR *var = 0
					) const;

				char m_name[32];
				bool m_bDefined;
				bool m_bHasVarColor;
				bool m_bHasTransparent;
				int m_OtherColorsUsed;
				int m_PatternNum;				/* The pattern number within that layer */
				int m_Version;					/* # of other patterns with same name */
				int m_NumVarColorsUsed;
				UINT8 m_ColorUsage[32];
				UINT8 *m_pattbuf;				/* Free this */
				int m_size;
				COLOR *m_VarColor;	/* Array of variable colors for which pattern should be defined */
				SVGPATTERN *m_alias;		/* Points to identical pattern from previous layer in AllPatterns */
			};

		int m_len;			//Lenght of current line
		double m_hres; //horizontal resolution
		double m_vres; //vertical resolution
		double m_LineWidth;
		DOUBLE m_DataTipOpacity;
		MGD::LINECAP m_LineCapStyle;
		MGD::LINEJOIN m_LineJoinStyle;
		int m_LayerNum;			// Incremented every time we start a layer 
		INT32 m_ElemNum;
		MISTRING m_DisplayToView;
		MISTRING m_ViewToGroup;
		MISTRING m_EquiToLatLonString;
		LRECTXY m_ClipRect;
		MISTRINGLIST m_ClipPaths;
		MISTRING m_ClipPath;
		FILEPATH m_SymbolDirectory;
		int m_ClipIndex;
		SVGPATTERN *m_Patterns;	// Patterns this layer 
		int m_NumPatterns;
		SVGPATTERN *m_AllPatterns;	// All patterns we've seen so far 
		int m_TotalPatterns;
		RVC::ELEMIDX m_ActiveElement;
		MISTRINGLIST m_PatternNames;
		MILIST<SVGSTYLE> m_StyleSet;
		MISTRINGLIST m_Entities;
		MISTRINGLIST m_DataTipDefList; //like entitities but used for def blocks
		SVGLAYER m_SVGGroup; 

		bool m_bHasElements;
		bool m_bNeedsInit;
		bool m_bNoMessages;
		bool m_bForIllustrator;
		bool m_bExternalStyles;
		bool m_bExternalScripts;

		bool m_bStyleDefine;
		bool m_bScriptDefine;
		bool m_bClipDefine;

		bool m_bBeginningLayer;
		bool m_bBeginningGroup;
		bool m_bSingleLayerGroup;
		bool m_bInLayer;
		bool m_bInGroup;
		bool m_bInImage;
		bool m_bDisableStyles; //draw it sans style dont call activate GC etc.
		bool m_bRendering3DImage;
		bool m_bGroupIs3D;
		bool m_bProjNotFound;
		bool m_bLatLon;
		bool m_bStylesAreDefined;
		bool m_bEmbedFonts;
		bool m_bEmbedImages; //whether to embed images.
		bool m_bGzip; //whether to gzip
		bool m_bInlineStyles; //whether to inline the styles
		bool m_bStandardCompression;
		bool m_bIncludeLayerCtrlScript;
		bool m_bUseAdobeMenu;
		bool m_bIncludeLayerCtrlPanel;
		bool m_bIncludeShowCoordScript;
		bool m_bIncludeDatatipScript;
		bool m_bIncludeMeasurementScript;
		bool m_bUseFreehandMeasures;
		bool m_bUseEntitiesInDataTips;
		bool m_bUseInverseHighlights;
		bool m_bHighlightEnlargePointSymbol;
		bool m_bBlinkHighlight;
		bool m_bHTMLwrap;
		bool m_bClipBounds;
		bool m_bInTextBlock;
		bool m_bClippingOn;
		bool m_bCurLayerIsRaster;
		bool m_bColorIsVariable;
		bool m_bDoFill; // used by ActivateGC if false only set/create line style
		bool m_bDrawingFillPattern;
		bool m_bDrawingDataTip;
		bool m_bDoBorder;
		bool m_bUsingPNG;
		bool m_bForcePNG;
		bool m_bCoordsFound;
		bool m_bWebClient; //whether to include client server communication
		bool m_bDoFloat;
		bool m_bDoTitles;
		bool m_bUseStyledDataTips;
		bool m_bUseSymbolDefinitions;
		bool m_HyperLinkedPolys;
		bool m_bCurLayerIsLegend;
		bool m_bDrawingLinePattern; 
		COLOR m_DrawColor;
		COLOR m_FillColor;
		COLOR m_HighlightColor;
		double m_Opacity;
		bool m_bClosed;
		double m_LineThicknessScale;
		bool m_bSuppressingHidden;

		png_struct_def *png_ptr;
		png_info_struct *info_ptr;
		JPEGEXT_FILE* m_pJPEGFile;

		MISTRING m_TextStyle,m_FontName;
		double m_BaseAngle;
		DPOINT2D m_BasePoint;

		int m_JPEGQuality;

		int m_TextCurveIndex;	// used to create a uniqueid for each textPath
		int m_CurFillPattern;	// Pattern currently active (-1) if none
		double m_ImageX, m_ImageY;
		int m_ImageLins, m_ImageCols;	// For error checking on PutImageSpan 
		INT32 m_ImageSize; // for error checking on 3D image rendering
		int m_ImageLinsSeen;

		FILE_RAW m_PatternFile;
		FILE_RAW m_OutputFile;
		FILE_RAW m_StyleFile;
		FILE_RAW m_ScriptFile;
		FILE_RAW m_ActiveFile;
		FILE_RAW m_ZippedFile;

		int m_ImageFileIndex;
		FILE_LEGACY_STREAM m_ImageFile;
		MISTRING m_FilePrefix;

		FILEPATH m_OutputFilename;
		FILEPATH m_ImageFilename;

		INT32 m_StyleStartOffset;
		INT32 m_PatternStartOffset;

		int m_PngIndex;
		int m_CurrentStyle;
		int m_CurrentEntity;
		int m_CurrentGroupID;

		
		MILIST<SVGSYMBOL> m_Symbols;
		MILIST<SVGFILTER> m_Filters;
		MISTRINGLIST m_LayerNames;
		MILIST<SVGLAYER> m_SVGLayers;
		MISTRINGLIST m_Fonts;
		MILIST<FONTPAIR> m_FontPairList;
		TEXTSTYLE m_CurrentTextStyle;

		SIMPLE_ARRAY<UINT32> m_GroupChildren;
		SIMPLE_ARRAY<GRE::GROUP*> m_Groups;

		//--------------------------------
		SIMPLE_ARRAY<UINT8> m_3DimageBuffer;
		DRECT2D m_GroupExtents;
		SIMPLE_ARRAY<UINT8> m_GroupExtentsSet;
		//--------------------------------

		MISTRING m_nsPrefix; // used to allow prefix mixed namespace svg's
		MISTRING m_LayerName;
		MISTRING m_GroupName;
		MISTRING m_DataTip;
		MISTRING m_CurrentElementUrl;
		
		MISTRING m_LastFont;
		GRE::LAYER* m_Layer;
		GRE::GROUP* m_Group;
		SVGSYMBOL m_ActiveSymbol;
		SVGFILTER m_ActiveFilter;
		int m_SymbolNum,m_CurrentSymbol;
		INT32 m_CurrElem;
		INT32 m_CurrentDataTip;
		int m_DefaultPrecision;

		bool m_HasEquiToLatLon;
		TRANS2D_AFFINE m_EquiToLatLon;
		MISTRING m_GeoTransformStr;

		bool m_bDrawingSymbol;
		bool m_bInDefinition;
		bool m_bGzipping; //present state i am gzipping
		bool m_ElementHasData;

		double vScale, hScale, vOffset, hOffset;

		MILIST<MGD::DEVICE_KML::LAYERSETTINGS> m_LayerSettingsList;
		MGD::DEVICE_KML::LAYERSETTINGS m_Settings; //settings for current layer;

			//CurrentLayers Databases
		RVC::DBASE_POLYGON m_PolygonDBaseObj;
		RVC::DBASE_LINE m_LineDBaseObj;
		RVC::DBASE_POINT m_PointDBaseObj;
		RVC::DBASE_CAD m_CADDBaseObj;

		//CurrentLayers Tables

		RVC::DBTABLE m_PointNameTable;  
		RVC::DBTABLE m_PointDescTable;
		RVC::DBTABLE m_PointSnippetTable;
		RVC::DBTABLE m_PointDataTable;

		RVC::DBTABLE m_PointLinkTable;
		RVC::DBTABLE m_PointIDTable;
		
		RVC::DBTABLE m_LineNameTable;
		RVC::DBTABLE m_LineDescTable;
		RVC::DBTABLE m_LineSnippetTable; 
		RVC::DBTABLE m_LineDataTable;

		RVC::DBTABLE m_LineLinkTable;
		RVC::DBTABLE m_LineIDTable;

		RVC::DBTABLE m_PolyNameTable;
		RVC::DBTABLE m_PolyDescTable;
		RVC::DBTABLE m_PolySnippetTable;
		RVC::DBTABLE m_PolyDataTable;

		RVC::DBTABLE m_PolyLinkTable;
		RVC::DBTABLE m_PolyIDTable;
		
		RVC::DBTABLE::FIELDINFOARRAY PolyDataFinfo;
		RVC::DBTABLE::FIELDINFOARRAY LineDataFinfo;
		RVC::DBTABLE::FIELDINFOARRAY PointDataFinfo;


		RVC::GEOMETRIC* m_GeoObj;

		//--------------------------------
 
		//These stored as strings so that it can be passed
		//back as a string or validated using xml library
		//xml parse is too slow however. Using the library 
		// more extensively so it isnt parsed after the fact.

		MISTRING m_Projname;
		MISTRING m_SVG; // svg code before the entity description eventually stores all.
		MISTRING m_EntitiesString; //entities stored in output form.
		MISTRING m_Head; //code between entities and stylesheet
		MISTRING m_StyleSheet; //css styles compiled
		MISTRING m_Body; //code after styles  
		MISTRING m_Script; // stores script chunks
		DOUBLE panelLeft; //used for offsetting main map and control panel
		DOUBLE m_SymbolEnlargement;
		DOUBLE m_DefaultStrokeWidth;

		MISTRING m_ElementClassName; //stores stylename appropriate for currently rendering element

		RVC::DBASE_POLYGON m_DbObjPolygon;
		RVC::STYLE m_StyleObjPolygon;	
		RVC::DBTABLE_STYLE m_TableStylePolygon;

		RVC::DBASE_LINE m_DbObjLine;
		RVC::STYLE m_StyleObjLine;	
		RVC::DBTABLE_STYLE m_TableStyleLine;

		RVC::DBASE_POINT m_DbObjPoint;
		RVC::STYLE m_StyleObjPoint;	
		RVC::DBTABLE_STYLE m_TableStylePoint; 

		RVC::DBTABLE::RECORD_STYLE_ELEMENT m_RecordStyle;

		DOUBLE m_MasterMargin;
		DOUBLE m_MasterScale;
		MISTRING m_ElemTitle;
		
		#ifndef NOLIBZ
		 z_stream m_zstream;
		#endif
	
		UINT32 m_crc;
		UINT8 m_zbuf[32768];
		UINT32 m_filter;


		// Unimplemented.
		DEVICE_SVG (const DEVICE_SVG &);
		DEVICE_SVG& operator= (const DEVICE_SVG &);
 
		ERRVALUE ActivateGC (
			MGD::CONTEXT_SVG *gc
			);

		MISTRING ActivateTC (
			MGD::CONTEXT_SVG *gc
			);

		int AddClippingPath (
			MGD::CONTEXT_SVG* gc,
			const MISTRING& region
			) ;

		// Searches the list of fonts and writes the font 
		// to the style file only if it isnt already in the list
		ERRVALUE AddFontToList (
			const MISTRING& fontfam,
			const MISTRING& fontname
			);

		ERRVALUE AppendFile (
			const FILEPATH &filename,
			bool b_encode64
			);

		int CompressBegin ();

		int CompressEnd ();

		ERRVALUE Draw3DPNG ();

		int DumpFillPattern (
			SVGPATTERN *pat,
			const RVC::STYLEPATTERN_BITMAP& BitmapPatt,
			COLOR *var,  // Variable color 
			MISTRING& name
			);

		int DumpFillPatternLayer (
			const RVC::STYLEPATTERN_BITMAP& BitmapPatt,
			int c,				// Color number 
			const COLOR& color		// RGB value of color
			);
		
		MISTRING Encode64 (
			const UINT8 *buff,
			int len
			) const;

		//checks to see if the shadow filter is in the list,(else outputs it and)
		//also assigns the defined filter to be the active one.
		//should be done before an activateTC but not inside an object
		INT32 FilterInList (
			double x,
			double y,
			const COLOR& color,
			double opacity,
			double height
			);

		int FindCurrentStyle (
			const MISTRING& code,
			MISTRING& className,
			bool noelement=false
			);

		int FindDataTipDef (
			CONTEXT_SVG *gc,
			const MISTRING &def
			);

		int FindEntity ( //!< allowing strings to be referred to using xml entities
			const MISTRING& code
			);

		void FreePrivateData ();

		MISTRING GenerateOnClick (bool bUseCurrentElementURL = false) const;
			 
		bool GetClassNameFromStyAsTable (		// Returns TRUE if have className, FALSE if don't
			const ELEMTYPE type,
			RVC::DBTABLE_STYLE& TableStyle,
			INT32 ElemNum,
			const RVC::STYLE& StyleObj,
			MISTRING& className
			);

		ERRVALUE InsertDataTip (
			CONTEXT_SVG *gc
			);

		ERRVALUE InsertFile (
			FILE_RAW& file1,
			FILE_RAW& file2,
			UINT32 offset
			);

		int PortWrite (
			const char *buf,
			int len
			) { return (PortWrite(reinterpret_cast<const UINT8*>(buf), len)); }
		
		int PortWrite (
			const UINT8 *buf,
			int len
			);

		void SendDouble (double num);

		int SendEOL ();

		int SendGroupHeader ( 
			MGD::CONTEXT_SVG* gc,
			bool isGroup,
			const MISTRING& name
			);

		void SendLong (INT32 num);
		void SendString (const char *str);
		void SendString (const MISTRING &str);

		void SendXYCmdDouble (
			double x,
			double y,
			const char *cmd = 0
			);

		void SendXYCmdUnscaledLong (
			INT32 x,
			INT32 y,
			const char *cmd = 0
			);

		ERRVALUE SetFillPattern (
			MGD::CONTEXT_SVG* gc
			);
		
		ERRVALUE StartClipping (
			MGD::CONTEXT_SVG* gc
			);

		//tests to see of the symbol is already defined 
		//and if it is returns its index else -1
		INT32 SymbolInList (
			INT32 symnum,
			bool builtin
			);

		//mostly superfulous with an svg integer coordinate system.
 		void TransCoord ( 
			INT32 x, 
			INT32 y, 
			double *dx, 
			double *dy
			) const {
			*dx = (double)x; 
			*dy = (double)y;
			} 

		int WriteString (
			const MISTRING& str
			);

		int z_PutUINT16 (UINT16 x);
		int z_PutUINT32 (UINT32 x);

		virtual bool v_AvoidFillSpans () const;
		virtual ERRVALUE v_Clear (const COLOR &color);
		virtual MGD::CONTEXT* v_CreateContext ();
		virtual ERRVALUE v_FillSpan (MGD::CONTEXT *gc, INT32 x1, INT32 x2, INT32 y);
		virtual MGD::CAPABILITY v_GetCapabilities () const;
		virtual MGD::PIXEL v_GetColorPixel (const COLOR& color);
		virtual COLORSPACE v_GetColorSpace () const;
		virtual bool v_GetNeedFullImageLine () const;
		virtual void v_GetPixelColor (MGD::PIXEL pixel, COLOR &color) const;
		virtual bool v_UseAntiAliasing () const;

		friend class MGD::CONTEXT_SVG;
#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGD

#endif	// ifndef INC_MI32_MG2SVG_H
