/**
 * \file mi32/mgdmask.h
 * \brief MGD::DEVICE_MASK definitions.
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef INC_MI32_MGDMASK_H
#define INC_MI32_MGDMASK_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGD {

//=====================================================================================================================

//! Base mask device.
class DEVICE_MASK : public DEVICE {
	public:

		//! Construct with reference to target device.
		DEVICE_MASK (
			DEVICE& DevTgt,		//!< Reference to target device, must exist as long as DEVICE_MASK instance
			UINT16 MaskSizeX,		//!< X size of mask in pixels, must be positive
			UINT16 MaskSizeY,		//!< Y size of mask in pixels, must be positive
			const UINT8 *Mask		//!< Mask data
			);

		//! Destructor
		virtual ~DEVICE_MASK ();

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		class CONTEXT;

		DEVICE& m_DevTgt;
		SIMPLE_ARRAY<UINT8> m_Mask;
		UINT16 m_MaskSizeX;
		UINT16 m_MaskSizeY;

		DEVICE_MASK (const DEVICE_MASK&);
		DEVICE_MASK& operator= (const DEVICE_MASK&);

		virtual ERRVALUE v_Clear (const COLOR& color);
		virtual MGD::CONTEXT* v_CreateContext ();
		virtual ERRVALUE v_FillSpan (MGD::CONTEXT *gc, INT32 x1, INT32 x2, INT32 y);
		virtual CAPABILITY v_GetCapabilities () const;
		virtual const COLOR_ARRAY* v_GetColorMap () const;
		virtual PIXEL v_GetColorPixel (const COLOR& color);
		virtual COLORSPACE v_GetColorSpace () const;
		virtual bool v_GetNeedFullImageLine () const;
		virtual UINT16 v_GetNumFrameBuffers () const;
		virtual void v_GetPixelColor (PIXEL pixel, COLOR& color) const;
		virtual bool v_IsX11 () const;
		virtual bool v_SupportsOpacityBlendMode (MGD::OPACITY_MODE) const;
 		virtual bool v_UseAntiAliasing () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

} // End of namespace MGD


#endif	// INC_MI32_MGDMASK_H

