/**
 * \file mi32/printsettings.h
 * \brief PRINTSETTINGS definitions
 *
 * \if NODOC
 * $Id: printsettings.h_v 1.9 2015/05/19 19:07:08 bcolombe Exp $
 *
 * $Log: printsettings.h_v $
 * Revision 1.9  2015/05/19 19:07:08  bcolombe
 * Add margin for sml
 *
 * Revision 1.8  2013/09/20 15:00:20  mju
 * Add setAnaglyph...
 *
 * Revision 1.7  2013/09/19 21:46:24  mju
 * Add members for stereo.
 *
 * Revision 1.6  2013/09/16 20:58:58  mju
 * Add isRasterDithered/Monochrome.
 *
 * Revision 1.5  2013/09/12 22:20:15  mju
 * Add driver parm when validate.
 *
 * Revision 1.4  2013/09/10 21:18:39  mju
 * Change getSizeUnits to getPageSizeUnits so not ambiguous.
 *
 * Revision 1.1  2013/08/30 21:31:59  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_PRINTSETTINGS_H
#define	INC_MI32_PRINTSETTINGS_H

#ifndef INC_MI32_PRINTGAMMA_H
	#include <mi32/printgamma.h>		
#endif

#ifndef INC_MI32_COLORBAL_H
	#include <mi32/colorbal.h>		// for defn of CBPARMS
#endif

#ifndef INC_MI32_MG2CMS_H
	#include <mi32/mg2cms.h>		// for defn of PROFILE
#endif

#ifndef INC_MI32_PRINTMEDIA_H
	#include <mi32/printmedia.h>
#endif

#ifndef INC_MI32_XMLDATA_H
	#include <mi32/xmldata.h>
#endif

#ifndef INC_MI32_FIXEDSTR_H
	#include <mi32/fixedstr.h>
#endif

#ifndef INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef INC_MI32_STEREODEVICE_H
	#include <mi32/stereodevice.h>
#endif

#ifndef INC_MI32_STEREORENDER_H
	#include <mi32/stereorender.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declaration
class PRINTDRIVER;
class PRINTENGINE;
namespace GRE {
	class LAYOUT;
	}
namespace SML {
	union ARG;
	class CLASSMEMBER;
	class CONTEXT;
	}
#endif

//=====================================================================================================================

//! Container for print setup.
class PRINTSETTINGS {
	public:

		enum DESTTYPE {
			DESTTYPE_Device = 0,
			DESTTYPE_File =	1,
			DESTTYPE_Raster = 2
			};

		static const SERIALIZERITEM* SerialGetItemDef ();

		PRINTSETTINGS ();

		PRINTSETTINGS (const PRINTSETTINGS&);

		~PRINTSETTINGS ();

		PRINTSETTINGS& operator= (const PRINTSETTINGS&);

		CBPARMS& AccessColorBalance ()
			{ return (m_ColorBalance); }

		XMLDATA& AccessDriverSpecific ()
			{ return (m_DriverSpecific); }

		PRINTGAMMA* AccessGamma ()
			{ return (m_gamma); }

		MGD::PROFILEPARMS_PRINTER& AccessProfileParms ()
			{ return (m_ProfileParms); }

		STEREORENDER& AccessStereoRender ()
			{ return (m_StereoRender); }

		//! Display "Print Setup" or "Print" dialog.
		//! @return TRUE if settings changed, FALSE if not, < 0 if error/cancel
		int DlgSetup (
			MDLGPARENT dlgparent,			//!< Parent for dialog
			PRINTENGINE *pPrintEngine		//!< Print engine, if printing, 0 if Setup
			);

		STEREODEVICE::ANAGLYPHMODE GetAnaglyphColors () const
			{ return (m_AnaglyphColors); }

		double GetBrightness () const
			{ return (m_Brightness); }

		const CBPARMS& GetColorBalance () const
			{ return (m_ColorBalance); }

		double GetContrast () const
			{ return (m_Contrast); }

		double GetDesignScale () const
			{ return (m_DesignScale); }

		DESTTYPE GetDestType () const
			{ return (m_DestType); }

		const char* GetDirectDeviceName () const
			{ return (m_DirectDeviceName); }

		const MISTRING& GetDirectPortName () const
			{ return (m_DirectPortName); };

		UINT16 GetDirectPortType () const
			{ return (m_DirectPortType); };

		const char* GetDitherPatternRaster () const
			{ return (m_DitherPatternRaster); }

		const char* GetDitherPatternVector () const
			{ return (m_DitherPatternVector); }

		const PRINTMEDIA& GetDocMedia () const
			{ return (m_DocMedia); }

		const XMLDATA& GetDriverSpecific () const
			{ return (m_DriverSpecific); }

		const PRINTGAMMA* GetGamma () const
			{ return (m_gamma); }

		const PRINTGAMMA& GetGamma (int idx) const
			{ return (m_gamma[idx]); }

		const PRINTMEDIA& GetPageMedia () const
			{ return (m_PageMedia); }

		const MGD::PROFILEPARMS_PRINTER& GetProfileParms () const
			{ return (m_ProfileParms); }

		const DPOINT2D& GetResolution () const
			{ return (m_PageMedia.GetResolution()); }

		double GetResolutionDPI (
			) const { return (m_PageMedia.GetResolution(SCREENUNIT_Inches)); }

		const MISTRING& GetSystemDeviceName () const
			{ return (m_SystemDeviceName); }

		SCREENUNIT GetPageSizeUnits () const
			{ return (m_PageMedia.GetSize().GetUnits()); }

		const STEREORENDER& GetStereoRender () const
			{ return (m_StereoRender); }

		double GetZoom () const
			{ return (m_Zoom); }

		bool HasDirectPort () const
			{ return (m_DirectPortType != 0 && !m_DirectPortName.IsEmpty()); }

		bool HasDitherPatternRaster () const
			{ return (m_DitherPatternRaster[0] != 0); }

		bool HasDitherPatternVector () const
			{ return (m_DitherPatternVector[0] != 0); }

		bool HasDriverSpecific () const
			{ return (m_DriverSpecific.HasAnyValues()); }

		bool HasUserColorBalance () const
			{ return ((m_flags & FLAG_UserColorBalance) != 0); }

		bool HasUserGamma () const
			{ return ((m_flags & FLAG_UserGamma) != 0); }

		bool IsAnaglyph () const
			{ return ((m_flags & FLAG_Anaglyph) != 0); }

		bool IsDeviceDirect () const
			{ return ((m_flags & FLAG_DeviceDirect) != 0); }

		bool IsDeviceSystem () const
			{ return ((m_flags & FLAG_DeviceSystem) != 0); }

		bool IsDocMultiPage () const;

		bool IsFitToPage () const
			{ return ((m_flags & FLAG_FitToPage) != 0); }

		//! Determine if page is "continuous" (has 0 margins in downward-print direction).
		bool IsPageContinuous () const;

		bool IsPageLandscape () const
			{ return (m_PageMedia.GetSize().IsLandscape()); }

		bool IsRasterDithered () const
			{ return ((m_flags & FLAG_RasterDithered) != 0); }

		bool IsRasterMonochrome () const
			{ return ((m_flags & FLAG_RasterMonochrome) != 0); }

		//! Read defaults from Ini file.
		ERRVALUE ReadDefault (const char *IniGroup);

		void SetAnaglyph (bool bAnaglyph);

		void SetAnaglyphColors (STEREODEVICE::ANAGLYPHMODE AnaglyphColors)
			{ m_AnaglyphColors = AnaglyphColors; }

		void SetBrightness (INT16 brightness)
			{ m_Brightness = brightness; }

		void SetColorBalance (
			const CBPARMS& ColorBalance
			);

		void SetContrast (INT16 contrast)
			{ m_Contrast = contrast; }

		void SetDesignScale (double scale);

		void SetDestType (DESTTYPE DestType)
			{ m_DestType = DestType; }

		//! Set output device name.
		void SetDeviceDirect (
			const char *devicename
			);

		//! Set to "system" device.
		void SetDeviceSystem (
			const MISTRING& DeviceName
			);

		//! Set port type/name when output is to device.
		void SetDirectPort (UINT16 PortType, const MISTRING& PortName);

		void SetDitherPatternRaster (const char *DitherPattern);

		void SetDitherPatternVector (const char *DitherPattern);

		//! Set document bottom margin, keeping document total height and top margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetDocMarginBottom (double margin)
			{ return (m_DocMedia.SetMarginBottom(margin)); }

		//! Set document left margin, keeping document total width and right margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetDocMarginLeft (double margin)
			{ return (m_DocMedia.SetMarginLeft(margin)); }

		//! Set document right margin, keeping document total width and left margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetDocMarginRight (double margin)
			{ return (m_DocMedia.SetMarginRight(margin)); }

		//! Set document top margin, keeping document total height and bottom margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetDocMarginTop (double margin)
			{ return (m_DocMedia.SetMarginTop(margin)); }

		void SetDocDimensions (const PRINTMEDIA& dims);

		//! Set document total height, keeping top and bottom margins unchanged if possible.
		void SetDocTotalHeight (double height);

		//! Set document total width, keeping left and right margins unchanged if possible.
		void SetDocTotalWidth (double width);

		void SetFitToPage (bool bFitToPage);

		void SetGamma (int idx, const PRINTGAMMA& gamma);

		void SetPageLandscape (bool bLandscape);

		//! Set page bottom margin, keeping page total height and top margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetPageMarginBottom (double margin)
			{ return (m_PageMedia.SetMarginBottom(margin)); }

		//! Set page left margin, keeping page total width and right margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetPageMarginLeft (double margin)
			{ return (m_PageMedia.SetMarginLeft(margin)); }

		//! Set page right margin, keeping page total width and left margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetPageMarginRight (double margin)
			{ return (m_PageMedia.SetMarginRight(margin)); }

		//! Set page top margin, keeping page total height and bottom margin unchanged if possible.
		//! @return true if other values adjusted, false if not.
		bool SetPageMarginTop (double margin)
			{ return (m_PageMedia.SetMarginTop(margin)); }

		void SetPageMedia (const PRINTMEDIA& media);

		//! Set page total height, keeping top and bottom margins unchanged if possible.
		void SetPageTotalHeight (double height);

		//! Set page total width, keeping left and right margins unchanged if possible.
		void SetPageTotalWidth (double width);

		void SetRasterDithered (bool bRasterDithered);

		void SetRasterMonochrome (bool bRasterMonochrome);

		void SetResolution (double resolution);

		void SetResolution (const DPOINT2D& resolution);

		void SetZoom (double zoom);

		//! Validate settings, adjusting if necessary.
		//! @return true if settings changed, false if not
		bool Validate (
			const PRINTDRIVER *pDriver = 0				//!< Driver to use for validation
			);

		//! Write defaults to Ini file.
		ERRVALUE WriteDefault (const char *IniGroup);

		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum FLAGS {
			FLAG_None =						0,
			FLAG_UserColorBalance =		0x00000008,
			FLAG_UserGamma =				0x00000010,
			FLAG_FitToPage =				0x00000400,
			FLAG_Anaglyph =				0x00001000,
			FLAG_RasterMonochrome =		0x00004000,		// Print raster is monochrome (b/w or grayscale)
			FLAG_RasterDithered =		0x00008000,		// Print raster is "dithered" (1 bit/component)
			FLAG_DeviceDirect =			0x00010000,		// Known to use direct device
			FLAG_DeviceSystem =			0x00020000,		// Known to use system device
			};

	protected:

		static const SML::CLASSMEMBER* SML_GetClassMembers ();

	private:
		typedef PRINTSETTINGS THISCLASS;

		class DLG;

		CBPARMS m_ColorBalance;							// Color balance parameters
		PRINTGAMMA m_gamma[3];
		MGD::PROFILEPARMS_PRINTER m_ProfileParms;
		XMLDATA m_DriverSpecific;
		PRINTMEDIA m_PageMedia;
		PRINTMEDIA m_DocMedia;
		STEREORENDER m_StereoRender;
		double m_DesignScale;						// Design scale
		double m_Zoom;									// Zoom factor (normally 1.0)
		double m_Contrast;
		double m_Brightness;
		double m_DotGain[4];							// Percent (0.0 to 1.0)
		double m_ScreenAngle[4];					// in degrees
		double m_ScreenFreq;							// in LPI. All componants must be same frequency
		FLAGS m_flags;
		STEREODEVICE::ANAGLYPHMODE m_AnaglyphColors;
		char m_DirectDeviceName[64];				// Name of driver.  Not localizable.
		char m_DitherPatternRaster[32];
		char m_DitherPatternVector[32];
		DESTTYPE m_DestType;
		UINT16 m_DirectPortType;					// PORTTYPE_ value
		MISTRING m_DirectPortName;
		MISTRING m_SystemDeviceName;

		static ERRVALUE SerializerCB (SERIALIZER&, const SERIALIZERITEM *pItem, void *vpInst, SERIALIZERITEM::ACTION action);

		static void SML_GetDeviceName (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetBottomMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetTopMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetLeftMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetRightMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetDocFullHeight (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetDocFullWidth (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetDocPrintHeight (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetDocPrintWidth (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetFullPage (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);
		static void SML_GetResolution (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pRet);

		static void SML_SetDeviceName (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetBottomMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetTopMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetLeftMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetRightMargin (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetDocFullHeight (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetDocFullWidth (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetFullPage (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);
		static void SML_SetResolution (SML::CONTEXT *pContext, SML::CLASSMEMBER*, void *vpInst, SML::ARG *pVal);

		ERRVALUE SerialReadWrite (SERIALIZER& serializer, const SERIALIZERITEM& item, SERIALIZERITEM::ACTION action);

		void UpdateDocResolution ();
		bool ValidatePaperSize (const PRINTDRIVER& driver);
		bool ValidateResolution (const PRINTDRIVER& driver);

		#endif
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(PRINTSETTINGS::FLAGS)
#endif

//=====================================================================================================================

#endif	// INC_MI32_PRINTSETTINGS_H
