/**
 * \file mi32/regionmanager.h
 * \brief REGIONMANAGER definitions
 *
 * \if NODOC
 * $Id: regionmanager.h_v 1.5 2013/10/23 21:38:27 mju Exp $
 *
 * $Log: regionmanager.h_v $
 * Revision 1.5  2013/10/23 21:38:27  mju
 * Eliminate treeviewfolder.
 *
 * Revision 1.4  2013/10/21 19:57:25  mju
 * Use treeviewnode/folder/object.
 *
 * Revision 1.3  2013/10/21 19:26:09  mju
 * Don't nest source, name as regionsource and put in own header.
 *
 * Revision 1.2  2013/10/21 17:51:22  mju
 * Add getFolderUserAdded.
 *
 * Revision 1.1  2013/10/18 21:27:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_REGIONMANAGER_H
#define	INC_MI32_REGIONMANAGER_H

#ifndef	INC_MI32_TREEVIEW_H
#include <mi32/treeview.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
class REGION2D;
class REGIONSOURCE;
namespace RVC {
	class OBJITEM;
	}
#endif

//=====================================================================================================================

//! Region manager, provides management for list of source regions and observers of the list.
class REGIONMANAGER : public TREEVIEW {
	public:

		//! Get shared region manager instance.
		static REGIONMANAGER& GetShared ();

		//! Determine if file is selectable for adding as region.
		static int IsFileSelectableAdd (const FILEPATH& filepath, MISTRING& msg, int);

		//! Add region to "user-added" folder.
		//! @return Pointer to added source.
		REGIONSOURCE* AddRegion (
			const REGION2D& region,					//!< Region to add
			const MISTRING& name						//!< Region name
			);

		//! Add "user" region(s) from specified location.
		ERRVALUE AddRegion (
			const RVC::OBJITEM& objitem,			//!< Location of region to add
			REGIONSOURCE **ppLastSourceAdded = 0		//!< Returned pointer to last source added, 0 if not needed
			);

		//! Add "user" region(s) from XML file.
		ERRVALUE AddFromXML (
			const FILEPATH& filepath,				//!< Path to XML file containing region(s)
			REGIONSOURCE **ppLastSourceAdded = 0		//!< Returned pointer to last source added, 0 if not needed
			);

		//! Find source from RVC::OBJITEM.
		//! @return Pointer to REGIONSOURCE, NULL if does not exist.
		const REGIONSOURCE* FindSource (
			const RVC::OBJITEM& objitem
			) const;

		//! Get folder for user-added regions.
		const TREEVIEWNODE& GetFolderUserAdded () const
			{ return (m_FolderUserAdded); }

		//! Get last user-added source, if any.
		//! @return Pointer to REGIONSOURCE, NULL if no user-added sources.
		const REGIONSOURCE* GetLastSourceUserAdded () const;

		//! Determine if source is "user-added".
		bool IsSourceUserAdded (
			const TREEVIEWNODE *pNode
			) const;

		//! Remove (user-added) region.
		void RemoveRegion (
			const REGIONSOURCE *pSource
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class SOURCE_USER;
		typedef MILIST<SOURCE_USER*> USERSOURCELIST;

		TREEVIEWNODE m_FolderUserAdded;
		USERSOURCELIST m_UserRegionList;

		REGIONMANAGER ();
		~REGIONMANAGER ();

		REGIONMANAGER (const REGIONMANAGER&);
		REGIONMANAGER& operator= (const REGIONMANAGER&);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_REGIONMANAGER_H
