/**
 * \file mi32/windowfunction.h
 * \brief WINDOWFUNCTION class definitions
 *
 * \if NODOC
 * $Id: windowfunction.h_v 1.1 2015/09/14 20:08:12 mju Exp $
 *
 * $Log: windowfunction.h_v $
 * Revision 1.1  2015/09/14 20:08:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_WINDOWFUNCTION_H
#define	INC_MI32_WINDOWFUNCTION_H

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

//=====================================================================================================================

//! Compute "window function" array.
class WINDOWFUNCTION {
	public:

		//! General window types.
		enum TYPE {
			TYPE_Rectangular,
			TYPE_TriangularNminus1,
			TYPE_TriangularN,
			TYPE_TriangularNplus1,
			TYPE_Hann,
			};

		//! Subtypes for alignment.
		enum SUBTYPE {
			SUBTYPE_Symmetric,
			SUBTYPE_Periodic,
			SUBTYPE_DFT_Even
			};

		//! Compute function array.
		static ERRVALUE Compute (
			DOUBLE_ARRAY<double> &array,
			TYPE type,				//!< Type
			SUBTYPE subtype,		//!< Subtype
			INT32 size				//!< Array / Window size
			);

		//! Constructor.
		WINDOWFUNCTION ();

		//! Destructor.
		~WINDOWFUNCTION ();

		//! Compute function array.
		ERRVALUE Compute (
			TYPE type,				//!< Type
			SUBTYPE subtype,		//!< Subtype
			INT32 size				//!< Array / Window size
			) { return (Compute(m_Values,type,subtype,size)); }

		//! Get array values.
		const DOUBLE_ARRAY<double>& GetValues () const
			{ return (m_Values); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<double> m_Values;
		#endif

	};

//=====================================================================================================================

#endif	// INC_MI32_WINDOWFUNCTION_H
