/**
 * \file rvc/dbfieldflags.h
 * \brief RVC::DBFIELDFLAGS enumeration
 *
 * \if NODOC
 * $Id: dbfieldflags.h_v 1.1 2015/01/08 22:27:54 mju Exp $
 *
 * $Log: dbfieldflags.h_v $
 * Revision 1.1  2015/01/08 22:27:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_DBFIELDFLAGS_H
#define	INC_RVC_DBFIELDFLAGS_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Database field flags.
enum DBFIELDFLAGS {
	DBFIELDFLAG_None =				0x00000000,
	DBFIELDFLAG_PrimaryKey =		0x00000001,	//!< Field is a primary key (force index)
	DBFIELDFLAG_ForeignKey =		0x00000002,	//!< Field is a foreign key related to field in another table
	DBFIELDFLAG_Indexed = 			0x00000004,	//!< Field has index 
	DBFIELDFLAG_ComputeOnRead = 	0x00000008,	//!< Field is computed 
	DBFIELDFLAG_NotNull = 			0x00000010,	//!< Value cannot be null/empty
	DBFIELDFLAG_Unique = 			0x00000020,	//!< value must be unique 
	DBFIELDFLAG_ReadOnly = 			0x00000040,	//!< User can't change value 
	DBFIELDFLAG_SecondaryKey =		0x00000080,	//!< Secondary (or higher) key 
	DBFIELDFLAG_Invisible = 		0x00000100,	//!< Field is hidden
	DBFIELDFLAG_Closest = 			0x00000200,	//!< Related to "closest" value
	DBFIELDFLAG_RangeAbove = 		0x00000400,	//!< Related to value above or equal
	DBFIELDFLAG_RangeBelow = 		0x00000800,	//!< Related to value below or equal
	DBFIELDFLAG_ComputeOnWrite = 	0x00001000,	//!< Compute field on write 
	DBFIELDFLAG_LocalTime = 		0x00002000,	//!< TYPE_DateTime field is stored in local time
	DBFIELDFLAG_AutoIncrement =	0x00004000,	//!< Default field value to next "AutoIncrement" value and increment that
	DBFIELDFLAG_ExtIndex =			0x00010000,	//!< Field has index (in ODBC), but we can't change it
	DBFIELDFLAG_MASK_Computed = 	(DBFIELDFLAG_ComputeOnRead | DBFIELDFLAG_ComputeOnWrite),
	DBFIELDFLAG_MASK_KeyFlags =	(DBFIELDFLAG_PrimaryKey | DBFIELDFLAG_ForeignKey | DBFIELDFLAG_Indexed),
//	DBFIELDFLAG_Searchable =		(DBFIELDFLAG_Indexed | DBFIELDFLAG_SSIndex)
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBFIELDFLAGS)
#endif

//=====================================================================================================================

//! Database key flags for MakeKey / MakeIndexed etc.
enum DBKEYFLAGS {
	DBKEYFLAG_None =					0x0000,
	DBKEYFLAG_PrimaryKey =			DBFIELDFLAG_PrimaryKey,		//!< Field is a primary key
	DBKEYFLAG_ForeignKey =			DBFIELDFLAG_ForeignKey,		//!< Field is a foreign key
	DBKEYFLAG_Indexed = 				DBFIELDFLAG_Indexed,			//!< Field is indexed
	DBKEYFLAG_RebuildIndex = 		0x2000,							//!< Rebuild the index if already there
	DBKEYFLAG_Sorted = 				0x4000,							//!< Hint that table is already sorted
	DBKEYFLAG_MASK_FieldFlags = 	DBFIELDFLAG_MASK_KeyFlags,	//!< Values that can be translated to DBFIELDFLAGS
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBKEYFLAGS)
#endif

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_DBFIELDFLAGS_H
