/**
 * \file rvc/dbtable.h
 * \brief interface for the RVC::DBTABLE class.
 *
 * \if NODOC
 * $Id: dbtable.h_v 1.294 2015/12/08 19:52:12 mju Exp $
 *
 * $Log: dbtable.h_v $
 * Revision 1.294  2015/12/08 19:52:12  mju
 * Add getFieldTypeString.
 *
 * Revision 1.293  2015/06/29 14:48:38  mju
 * Add makeparms::validate.
 *
 * Revision 1.292  2015/06/19 16:26:10  mju
 * Add openChildContainer.
 *
 * Revision 1.291  2015/05/28 14:46:24  mju
 * Add fieldinfo::isDateTime.
 *
 * Revision 1.290  2015/05/15 15:31:33  mju
 * Add pvt method.
 *
 * Revision 1.289  2015/05/13 21:57:26  mju
 * Remove bitset parm from appendTo.
 *
 * Revision 1.288  2015/05/12 21:43:43  mju
 * Use dbtabletrans etc.
 *
 * Revision 1.287  2015/04/29 22:10:54  mju
 * Use elemidx.
 *
 * Revision 1.286  2015/04/28 23:18:21  bcolombe
 * Add flags to write
 *
 * Revision 1.285  2015/04/21 15:31:09  mju
 * Add record::create using fieldinfo array.
 *
 * Revision 1.284  2015/04/09 21:25:44  mju
 * Adjust for dbioflags enum.
 *
 * Revision 1.283  2015/04/03 15:54:19  mju
 * Add getNumComputedFields.
 *
 * Revision 1.282  2015/04/03 15:25:19  mju
 * Add hasComputedField.
 *
 * Revision 1.281  2015/04/03 13:37:39  mju
 * Use utf8 encoding by default.
 *
 * Revision 1.280  2015/03/02 21:42:50  mju
 * Remove gremode hdr.
 *
 * Revision 1.279  2015/02/27 17:03:07  mju
 * Remove obsolete makeparm_style classes and record_style_cad.
 *
 * Revision 1.278  2015/02/27 16:24:37  mju
 * Remove usage_CADStyle.
 *
 * Revision 1.277  2015/02/25 18:27:59  mju
 * Adjust class descriptive comments.
 *
 * Revision 1.276  2015/01/08 22:28:07  mju
 * Use dbfieldflags/dbkeyflags.
 *
 * Revision 1.275  2014/12/24 15:48:46  mju
 * Remove legacy copyTo method.
 *
 * Revision 1.274  2014/12/24 15:40:20  mju
 * Remove legacyinfo fieldinfo::type enum.
 *
 * Revision 1.273  2014/12/23 16:07:43  mju
 * Remove legacy fieldinfo::usage enum.
 *
 * Revision 1.272  2014/11/17 16:19:33  mju
 * Use dbfieldusage enum to avoid nesting.
 *
 * Revision 1.271  2014/11/12 21:57:19  mju
 * Add fieldinfo::usage_Segment values.
 *
 * Revision 1.270  2014/10/09 14:45:50  mju
 * Add getNumNonDeletedRecords.
 *
 * Revision 1.269  2014/10/02 20:42:25  mju
 * Add fieldinfo::isFloat.
 *
 * Revision 1.268  2014/10/02 13:54:11  mju
 * Remove unused legacy fieldinfo::type values.
 *
 * Revision 1.267  2014/10/01 21:17:11  mju
 * Use rvc::dbfieldtype.
 * Use rvc::dbfieldcontol to replace constraint nested class.
 *
 * Revision 1.266  2014/09/22 20:04:35  mju
 * Add fieldinfo methods to help with usage/format.
 *
 * Revision 1.265  2014/09/19 14:29:53  mju
 * Add fieldinfo::accessFieldControl.
 *
 * Revision 1.264  2014/09/17 20:30:53  mju
 * Add fieldinfo::getUsageTextID.
 *
 * Revision 1.263  2014/09/15 20:42:51  mju
 * Add copyTo with attachment.
 *
 * Revision 1.262  2014/09/15 14:53:07  mju
 * Add fieldflag_Strip0.
 *
 * Revision 1.261  2014/07/09 17:01:50  mju
 * Add fieldvalue::getAsString.
 *
 * Revision 1.260  2014/01/03 16:19:43  mju
 * Remove obsolete deprecated methods.
 *
 * Revision 1.259  2013/11/21 22:40:05  mju
 * Use elemnum type for getAttachedRecords.
 *
 * Revision 1.258  2013/11/21 17:49:48  mju
 * Add fieldinfo::getTypeDisplay.
 *
 * Revision 1.257  2013/11/21 14:05:45  mju
 * Add getAttachedRecords using rvc::element.
 *
 * Revision 1.256  2013/11/20 21:39:47  mju
 * Change dbtable::record::value to dbtable::fieldvalue.
 * Add getFieldValueType.
 *
 * Revision 1.255  2013/11/20 18:45:05  mju
 * Add record::getValueType.
 *
 * Revision 1.254  2013/11/20 17:22:17  mju
 * Fix value::op= to return.
 *
 * Revision 1.253  2013/10/30 15:18:59  bcolombe
 * Add schema usage
 *
 * Revision 1.252  2013/05/09 21:07:06  mju
 * Remove unused getobjfilter defns.
 *
 * Revision 1.251  2013/05/09 20:40:03  mju
 * Add op_increment for dbtable::usage.
 *
 * Revision 1.250  2013/02/01 18:57:24  mju
 * Add setHidden.
 *
 * Revision 1.248  2012/06/22 17:26:44  mju
 * Add usage_FieldOfView.
 *
 * Revision 1.247  2012/05/15 18:52:48  mju
 * Add fieldinfo::isString.
 *
 * Revision 1.246  2011/12/08 15:43:50  mju
 * Remove record defns leftover from tntsim3d.
 * Just use single linkparms class with public ctors.
 *
 * Revision 1.243  2010/09/13 19:24:38  mju
 * Add makeparms::useFieldNamesAsHeading.
 *
 * Revision 1.242  2010/08/16 17:03:12  mju
 * Add usage_Properties.
 *
 * Revision 1.240  2010/02/17 14:16:16  mju
 * Add open using mistring.
 *
 * Revision 1.237  2010/02/15 17:28:41  mju
 * Add record::value class.
 *
 * Revision 1.236  2009/10/20 13:32:53  mju
 * Add clarified field types.
 *
 * Revision 1.234  2009/09/02 22:11:18  mju
 * Add m_bufSize so don't have to get from table every time.
 *
 * Revision 1.232  2009/04/01 14:56:12  mju
 * Allow increment op for fieldinfo::type enum.
 *
 * Revision 1.230  2009/03/27 13:45:14  mju
 * Add usage_picklist.
 *
 * Revision 1.229  2009/02/12 20:35:14  mju
 * Add makeparm_style_element::setupStaticFields.
 *
 * Revision 1.228  2008/12/04 22:07:48  bcolombe
 * Add bGetUnitsShown to GetUnits to convert from shown to stored units
 *
 * Revision 1.227  2008/10/01 22:34:38  scowan
 * Added makeparms for oracle.  Used to create a table in oracle database.
 *
 * Revision 1.226  2008/09/29 19:47:22  mju
 * Use dbfieldconstraint enum values for enum values in dbtable:;constraint.
 *
 * Revision 1.224  2008/09/25 22:26:09  scowan
 * Added copyMatchingFieldsTo method.
 *
 * Revision 1.222  2008/09/05 17:47:01  scowan
 * Added USAGE Shape and isUserDefined method.
 *
 * Revision 1.220  2008/09/03 22:48:32  scowan
 * Added getFieldInfo for record class.
 *
 * Revision 1.219  2008/08/28 20:57:32  mju
 * Add recordreftype enum.
 *
 * Revision 1.218  2008/06/16 20:10:15  scowan
 * Added shape and table object link to las files.
 *
 * Revision 1.217  2008/05/07 17:58:22  vdronov
 * substituted Unknkwn attachment by NoRestrictions
 *
 * Revision 1.213  2007/11/01 18:00:51  scowan
 * Added table encoding field to record.
 *
 * Revision 1.206  2007/06/25 20:07:03  dwilliss
 * added table usage for enumeration tables
 *
 * Revision 1.202  2007/05/08 21:41:08  dwilliss
 * Added SetPromptOnAddElement method to DBTABLE
 *
 * Revision 1.196  2007/03/19 20:36:48  dwilliss
 * TestIsJoinable need CopyFlags parameter
 *
 * Revision 1.184  2006/10/05 14:59:54  bcolombe
 * Added optional threshold value to CompareField and CompareFieldOrdered
 * Added CompareRecords function which works same as operator== function for comparing records but takes threshold
 *
 * Revision 1.178  2006/09/06 22:17:16  dwilliss
 * Changed the FIELDINFO::SetName() method that took a TEXTID to take an MISTRING
 * instead.  Since MISTRING has implicit ctor from TEXTID, it still works either way.
 *
 * Revision 1.156  2006/06/09 15:52:39  dwilliss
 * Added MakeDistinctTableFromField
 *
 * Revision 1.147  2006/04/19 20:50:58  dwilliss
 * defined FORMAT enum and some methods to get/set it
 *
 * Revision 1.140  2006/01/25 15:40:33  dwilliss
 * Added ComputeMinimumAttachmentType
 *
 * Revision 1.138  2006/01/09 20:25:14  dwilliss
 * Added optional encoding parameters to a couple methods
 *
 * Revision 1.132  2005/11/18 23:11:49  scowan
 * Added findFieldByIdentifier.
 *
 * Revision 1.127  2005/10/28 16:45:42  dwilliss
 * Added a GetBookmark to table iterator
 * deprecated get record num
 *
 * Revision 1.125  2005/10/12 16:47:48  dwilliss
 * Added a Write method that takes a bookmark
 *
 * Revision 1.1  2001/05/23 20:24:54  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBTABLE_H
#define INC_RVC_DBTABLE_H

#ifndef INC_RVC_DBASE_H
	#include <rvc/dbase.h>
#endif

#ifndef INC_RVC_DBFIELDCONTROL_H
	#include <rvc/dbfieldcontrol.h>
#endif

#ifndef INC_RVC_DBFIELDUSAGE_H
	#include <rvc/dbfieldusage.h>
#endif

#ifndef INC_RVC_DBFIELDFLAGS_H
	#include <rvc/dbfieldflags.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#ifndef INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif

#ifndef INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif

#ifndef	INC_MI32_ENUMSET_H
	#include <mi32/enumset.h>
#endif

#ifndef	INC_MI32_DATETIME_H
	#include <mi32/datetime.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_UOM_H
	#include <mi32/uom.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define INC_VECTOR
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
struct UNITCONV;
struct DBFIELDINFO;
namespace SML {
	class TABLEDATA;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
namespace RVC {
	struct DBCALLBACKSTRUCT;
	class SHAPE;
	class STYLE;
	class DBASE_ELEMENT;
	class DBTRANSINFO;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Table objects.  
class DBTABLE : public RVC::OBJECT {
	public:

		//! Database table attachment type 
		enum ATTACHMENT {
			ATTACHMENT_Unknown = 0,				//!< Your standard free-for-all
			ATTACHMENT_OneRecordPerElement,	//!< But may use record for multiple elements
			ATTACHMENT_OneElementPerRecord,	//!< But element may have multiple records
			ATTACHMENT_OneToOne,					//!< At most one record per element and one element per record
			ATTACHMENT_RelatedOnly,				//!< No attachments at all, only related
			ATTACHMENT_Identity,					//!< Record number is element number
			ATTACHMENT_NoRestrictions			//!< Your standard attached to element free-for-all
			};
			
		//! Predefined table ID's
		enum DEFINEDID {
			DEFINEDID_Internal =		 -1,			//!< The "Built-in" table.  Not valid for TIN internal tables
			DEFINEDID_InternalNode = -2,			//!< Not valid for TIN internal tables
			DEFINEDID_Invalid = 		-98,
			DEFINEDID_New =	 		 -9
			};
			
		//! Flags for finding attached record methods (i.e. RVCDBTABLE::GetAttachedRecords())
		enum FINDFLAG {
			FINDFLAG_AllAttached = 0,
			FINDFLAG_OnlyDirect = 2,
			FINDFLAG_FirstMatch = 8
			};

		//! Flags for searching on an index methods (i.e. RVCDBTABLE::SearchIndex())
		enum INDEXSEARCH {
			INDEXSEARCH_ReturnIndexOffset = 	0x00000000,
			INDEXSEARCH_CheckExists = 			0x00000001,
			INDEXSEARCH_ReturnRecordNumber = 0x00000002,
			INDEXSEARCH_ReturnInsertOffset = 0x00000004	
			};
			
		enum ITERATERECORD {
			ITERATERECORD_None = 			0x0000,
			ITERATERECORD_SkipDeleted = 	0x0001,
			ITERATERECORD_Sort = 			0x0002,
			ITERATERECORD_UniqueOnly = 	0x0004,
			ITERATERECORD_SaveIndex = 		0x0008,
			ITERATERECORD_SingleKey = 		0x8000
			};
			
		//! Type of external table
		enum LINKTYPE {
			LINKTYPE_Internal = 0,	//!< TNTmips internal format
			LINKTYPE_MIPS,				//!< MIPS external format (old RVF style) 
			LINKTYPE_DBASE3,			//!< dBase III+ format (external only)
			LINKTYPE_INFO,	 			//!< INFO format (external only)
			LINKTYPE_DBASE4,			//!< dBase IV (external only)
			LINKTYPE_RBASE, 			//!< R:BASE (external only)
			LINKTYPE_DBASE2,			//!< Dbase 2 (external only)
			LINKTYPE_ODBC,				//!< Link to ODBC server (external only)
			LINKTYPE_CSV,				//!< Comma Seperated values (export only)
			LINKTYPE_TAB,				//!< MapInfo TAB files
			LINKTYPE_ORACLE,			//!< Oracle server
			LINKTYPE_RDBMS,			//!< OLEDB or PostgreSQL
			LINKTYPE_LAS,				//!< LiDAR format 1.1 file
			LINKTYPE_NUMTYPES			//!< The number of link types
			};

		//! Record referencing type.
		enum RECORDREFTYPE {
			RECORDREFTYPE_New =			0,		//!< Reference to "new" record
			RECORDREFTYPE_Absolute =	1,		//!< Reference number is absolute
			RECORDREFTYPE_Bookmark =	2		//!< Reference number is bookmark value
			};

		//! Flags for element referenced tables  (TIN is the only one so far)
		enum REFELEM {
			REFELEM_All			= 0x00000000,
			REFELEM_Node		= 0x00000010,
			REFELEM_Edge 		= 0x00000020,
			REFELEM_Triangle	= 0x00000040,
			REFELEM_Hull		= 0x00000080	
			};
			
		//! Result values for CompareStructures() method
		enum STRUCTCOMPARE {
			STRUCTCOMPARE_DoNotMatch = 0,		//!< Table structures do not match
			STRUCTCOMPARE_Exact,					//!< Table structures match exactly
			STRUCTCOMPARE_Compatible,			//!< Table structures are compatible, record fields can be converted without loss of data
			STRUCTCOMPARE_SameTable				//!< Both tables point to the same linked table
			};
		
		//! Table capability flags, passed to RVCDBTABLE::HasCapability()
		enum CAPABILITY {
			CAPABILITY_Create	= 2048,
			CAPABILITY_Link,
			CAPABILITY_ReadMemo,
			CAPABILITY_WriteMemo,
			CAPABILITY_WriteRecord,
			CAPABILITY_MultiTable,		
			CAPABILITY_Server,
			CAPABILITY_Import,
			CAPABILITY_Export,
			CAPABILITY_DeleteRecord,
			CAPABILITY_UserWritable,
			CAPABILITY_CanSetUserWrite,
			CAPABILITY_ChangeFields,
			CAPABILITY_AddFields,
			CAPABILITY_DeleteFields,
			CAPABILITY_AddRecord,
			CAPABILITY_RandomAccess,
			CAPABILITY_ReadBinary,
			CAPABILITY_WriteBinary,
			CAPABILITY_WriteIfOpenForWrite,	//!< Will tell us if we _could_ get write permission if we tried, even if currently open for read only
			CAPABILITY_BookmarksAreNotAbsolute //!< If a table has this "capability", then bookmarks are hash values, not absolute record numbers
			};
			
		enum PERMISSIONS {
			PERMISSION_View = 0,
			PERMISSION_Modify,
			PERMISSION_Delete,
			PERMISSION_ViewStructure,
			PERMISSION_ModifyStructure,
			PERMISSION_Duplicate,
			PERMISSION_ReadACL,
			PERMISSION_ChangeACL
			};

		//! Table usage, defines specialized tables
		enum USAGE {
			USAGE_UserDefined =			0,
			USAGE_StdStats =				2,
	 		USAGE_ElementID =				3,
			USAGE_NodeTurn =				4,
			USAGE_ScaleSelect =			5,
			USAGE_Section =				6,
			USAGE_Internal =				7,
			USAGE_PointStyle =			8,
			USAGE_LineStyle =				9,
			USAGE_PolyStyle =				10,
			USAGE_TextStyle =				11,
	 		USAGE_MultiStyle =			12,
	 		USAGE_EditLog =				13,
	 		USAGE_Selection =				14,
			USAGE_FlightPath =			15,
			USAGE_PointOfInterest =		16,
			USAGE_LegendElement =		17,
			USAGE_StyleParameters =		18,
			USAGE_SelectParameters =	19,
			USAGE_ExtrudedPoints =		20,
			USAGE_InfluenceSphere =		21,
			USAGE_Sketch =					22,	//!< user-defined table on a sketch layer
			USAGE_ClumpRaster =			23,	//!< Output of Raster Clump process
			USAGE_Enumeration =			24,
			USAGE_Shape =					25,	//!< The 'directly' attached table for the shape object
			USAGE_PickList =				26,	//!< Pick list table
			USAGE_Properties =			27,	//!< Properties, typically for spatial object
			USAGE_FieldOfView =			28,	//!< Field of view (for viewshed)
			USAGE_Schema =					29,
			USAGE_LAST
			};

		enum VIEWMODE {
			VIEWMODE_Tabular			= 0,
			VIEWMODE_SingleRecord	= 1
			};
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class BOOKMARK;
		class CURSOR;
		class FIELDINFO;
		class GETOBJFILTER_ISSELECTABLE;
		class RECORD;
		friend class RECORD;
		class RECORD_ID_ELEMENT;
		class RECORD_ID_POINT;
		class RECORD_LEGENDELEMENT;
		class RECORD_NODETURN;
		class RECORD_SCALESELECT;
		class RECORD_SELECTION;
		class RECORD_STYLE_MULTI;
		class RECORD_STYLE_ELEMENT;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		typedef ENUMSET<USAGE, USAGE_LAST> USAGESET;
		typedef std::vector<FIELDINFO> FIELDINFOARRAY;
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class LINKPARMS;
		class MAKEPARMS;
		class MAKEPARM_ID_ELEMENT;
		class MAKEPARM_ID_POINT;
		class MAKEPARM_STYLE_ELEMENT;
		class MAKEPARM_SCALESELECT;
		class MAKEPARM_SELECTION;
		class MAKEPARM_LEGENDELEMENT;
		class MAKEPARM_NODETURN;
		class MAKEPARM_ORACLE;
		class RECORDITERATOR;
		class CONST_RECORDITERATOR;
		class OBSERVER;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Container for general field value.
		class FIELDVALUE {
			public:

				enum TYPE {
					TYPE_Invalid,
					TYPE_Double,
					TYPE_String,
					TYPE_DateTime,
					TYPE_Color
					};

				//! Default constructor, creates "invalid" value.
				FIELDVALUE (
					) : m_type(TYPE_Invalid), m_pValue(0) { }
				//! Copy constructor.
				FIELDVALUE (
					const FIELDVALUE &rhs
					) { Copy(rhs); }
				//! Construct from double.
				FIELDVALUE (
					double value
					) : m_type(TYPE_Double), m_pValue(new double(value)) { }
				//! Construct from MISTRING.
				FIELDVALUE (
					const MISTRING &value
					) : m_type(TYPE_String), m_pValue(new MISTRING(value)) { }
				//! Construct from DATETIME.
				FIELDVALUE (
					const DATETIME &value
					) : m_type(TYPE_String), m_pValue(new DATETIME(value)) { }
				//! Construct from COLOR.
				FIELDVALUE (
					const COLOR &value
					) : m_type(TYPE_String), m_pValue(new COLOR(value)) { }
				//! Destructor.
				~FIELDVALUE ()
					{ Free(); }

				//! Assignment from FIELDVALUE.
				FIELDVALUE& operator= (const FIELDVALUE &rhs)
					{ if (this != &rhs) { Free(); Copy(rhs); } return (*this); }
				//! Assignment from double.
				FIELDVALUE& operator= (double rhs)
					{ Free(); m_type = TYPE_Double; m_pValue = new double(rhs); return (*this); }
				//! Assignment from MISTRING.
				FIELDVALUE& operator= (const MISTRING &rhs)
					{ Free(); m_type = TYPE_String; m_pValue = new MISTRING(rhs); return (*this); }
				//! Assignment from DATETIME.
				FIELDVALUE& operator= (const DATETIME &rhs)
					{ Free(); m_type = TYPE_DateTime; m_pValue = new DATETIME(rhs); return (*this); }
				//! Assignment from COLOR.
				FIELDVALUE& operator= (const COLOR &rhs)
					{ Free(); m_type = TYPE_Color; m_pValue = new COLOR(rhs); return (*this); }
				//! Comparison for equality.
				bool operator== (const FIELDVALUE &rhs) const
					{ return (IsEqual(rhs)); }
				//! Comparison for inequality.
				bool operator!= (const FIELDVALUE &rhs) const
					{ return (!IsEqual(rhs)); }
				//! Comparison for sort ordering.
				bool operator< (const FIELDVALUE &rhs) const;

				//! Get value as string.
				MISTRING GetAsString (
					int precision = -1			//!< Precision, -1 to strip trailing 0's
					) const;

				//! Get COLOR value.
				COLOR GetColor () const
					{ return ((m_type == TYPE_Color) ? *static_cast<const COLOR*>(m_pValue) : COLOR()); }
				//! Get DATETIME value.
				DATETIME GetDateTime () const
					{ return ((m_type == TYPE_DateTime) ? *static_cast<const DATETIME*>(m_pValue) : DATETIME()); }
				//! Get double value.
				double GetDouble () const
					{ return ((m_type == TYPE_Double) ? *static_cast<const double*>(m_pValue) : 0); }
				//! Get MISTRING value.
				const MISTRING& GetString () const
					{ return ((m_type == TYPE_String) ? *static_cast<const MISTRING*>(m_pValue) : MISTRING::GetEmpty()); }
				//! Get type.
				TYPE GetType () const
					{ return (m_type); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TYPE m_type;
				void *m_pValue;

				void Copy (const FIELDVALUE &rhs);
				void Free ();
				bool IsEqual (const FIELDVALUE &rhs) const;
				#endif
			};

		//! Container for field definition.
		class FIELDINFO {
			public:

				//! Pseudo fields for FIELDINFO.next_field.  Some database schemas
				//! require a 2 part key where the primary key is made up of 2 fields
				//! but the foreign key is made up of 1 field and the name of the field
				//! or one field and a "domid" which is looked up from a 3rd table by
				//! using the table/field name.  We look those up on import and record
				//! them in the FIELDINFO.domid.  I don't know of anything that would
				//! use the table name, but I added it for completeness 
				enum NEXTFIELD {
					NEXTFIELD_ThisFieldName = 0xFFFF,	//!< Name of this field is the key data 
					NEXTFIELD_ThisTableName = 0xFFFE,	//!< Name of this table is the key data 
					NEXTFIELD_DomID			= 0xFFFD,	//!< Key data is the "domid" field of this field 
					NEXTFIELD_PseudoField	= 0xFFFC		//!< If next_field is > this, it's one of these 
					};
					
				// Note: format is stored as a UINT8 in the fieldinfo
				enum FORMAT {			
					FORMAT_Undefined = 0,
					FORMAT_OGC_WKB = 1,		//!< OGC Well Known Binary
					FORMAT_OGC_WKT,			//!< OGC Well Known Text
					FORMAT_ImageJPG,				//!< A JPEG file stored in a blob
					FORMAT_ImagePNG,				//!< A PNG file stored in a blob
					};

				//! Flags for GetTypeDisplay.
				enum GETTYPEDISPLAY {
					GETTYPEDISPLAY_General =			0x00,		//!< Generalize types
					GETTYPEDISPLAY_UserCreatable =	0x01,		//!< User-creatable types
					GETTYPEDISPLAY_Technical =			0x02,		//!< Technical label for each defined type
					GETTYPEDISPLAY_Computed =			0x04,		//!< Include "computed" indication if so
					};

				//! Get TEXTID for format.
				static TEXTID GetFormatTextID (
					FORMAT format,					//!< Field format
					bool bAbbr = false			//!< Use abbreviation
					);

				//! Get display string for specified field type/flags.
				static MISTRING GetTypeDisplay (
					DBFIELDTYPE fieldtype,		//!< Field type
					DBFIELDFLAGS fieldflags,	//!< Field flags
					GETTYPEDISPLAY getflags		//!< Request flags
					);

				//! Get TEXTID for usage.
				static TEXTID GetUsageTextID (
					DBFIELDUSAGE usage,			//!< Field usage
					bool bAbbr = false			//!< Use abbreviation
					);

				//! Default constructor
				FIELDINFO ();

				//! Copy constructor
				FIELDINFO (
					const FIELDINFO &rhs
					);

				~FIELDINFO ();

				//! Assignment
				FIELDINFO& operator= (
					const FIELDINFO &rhs
					);

				//! Access field control settings for modification.
				DBFIELDCONTROL& AccessFieldControl ()
					{ return (m_FieldControl); }

				//! Add specified flags.
				void AddFlags (
					DBFIELDFLAGS flags
					);

				//! Clear the field info of current values
				void Clear ();

				//! Clear specified flags.
				void ClearFlags (
					DBFIELDFLAGS flags
					);

				//! Clear a link from this field to another table
				void ClearLink ();

				//! Clear the next field number that this field is linked to
				void ClearNextLinkField ();

				//! Compare two field types and return best field type to convert to
				//! @return this->GetType() if exact match, DBFIELDTYPE_??? depending if the types are
				//! not the best match but still matchable, DBFIELDTYPE_Invalid if types do not match
				DBFIELDTYPE CompareFieldTypes (
					const FIELDINFO &rhs,
					bool &ExactMatch					//!< Set to 'true' if the two fields match exactly, 'false' if they do not
					) const;

				//! Get the AutoIncrement number for the next added record
				//! @return Next AutoIncrement number
				UINT32 GetAutoIncrement () const;

				//! Get number of decimal places to display for floating point fields
				//! @return Number of decimal places to display		
				int GetDecimalPlaces () const;

				//! Get actual number of decimal places used, will return 0 for non-floating point fields.
				int GetDecimalPlacesUsed () const;

				//! Get the display units for this field
				//! @return Display unit
				int GetDispUnits () const;
					
				//! Get number of columns to display
				//! @return Number of columns to display		
				int GetDispWidth () const;
					
				//! Get the domain ID for this field
				//! @return Domain ID
				const RVC::OBJECTNAME& GetDomainID () const;
					
				//! Get the flags for this field
				//! @return Field flags.
				DBFIELDFLAGS GetFlags () const;
					
				//! Get the format of the field.
				//! Currently only used for BLOB fields to record what kind of data is
				//!	stored in the blob.
				//! @return Field usage (FIELDINFO::FORMAT_...)
				FORMAT GetFormat () const;
					
				const DBFIELDCONTROL& GetFieldControl () const
					{ return (m_FieldControl); }

				//! Get descriptor for field.
				const RVC::DESCRIPTOR& GetDescriptor () const
					{ return (m_Descriptor); }

				//! Get the name of this field
				//! \deprecated Call GetDescriptor().GetShortName()
				//! @return Field name
				const RVC::OBJECTNAME& GetShortName () const;
					
				//! Get the name of this field
				//! @return Field name
				const MISTRING& GetFullName () const
					{ return (m_Descriptor.GetFullName()); }

				//! Get the next field number that this field is linked to
				//! @return Next link field number
				INT32 GetNextLinkField () const;
					
				INT32 GetOffset () const;

				//! Get the next field pseudo number that this field is linked to
				//! @return Next link field pseudo number (FIELDINFO::NEXTFIELD_...)
				NEXTFIELD GetPseudoNextLinkField () const;
					
				//! Get the field number that this field is linked to, use HasRelation() to 
				//! determine if the field is related to another table / field
				//! @return Linked field number
				INT32 GetRelatedField () const;
					
				//! Get the table number that this field is linked to, use HasRelation() to 
				//! determine if the field is related to another table / field
				//! @return Related table number
				INT32 GetRelatedTable () const;
					
				//! Get the size of the field in bytes
				//! @return Size of field in bytes
				INT32 GetSize () const;
					
				//! Get the type of the field
				//! @return Type of field.
				DBFIELDTYPE GetType () const;

				//! Get display string for this field type.
				MISTRING GetTypeDisplay (
					GETTYPEDISPLAY flags
					) const;
			
				//! Get the numeric units for this field for unit conversion
				void GetUnits (
					UNITCONV &conv,
					bool bGetUnitsShown = false
					) const;

				//! Get the numeric units for this field. 
				//! By default this method returns the units stored in the database.
				//! You can also get user's preferred units to view the field in, but
				//! that's mostly for historical reasons, as this preference is usually
				//! stored in a subobject of the table, not in the field info.
				UOM::UNIT GetUnits (
					bool bGetUnitsShown = false
					) const;

				int GetUnitType () const;

				//! Get the usage of the field
				//! @return Field usage.
				DBFIELDUSAGE GetUsage () const;

				//! Get array of valid formats.
				void GetValidFormats (
					DBFIELDUSAGE usage,							//!< Field usage
					SIMPLE_ARRAY<FORMAT> &ValidFormats		//!< Array of formats returned, will be empty if only FORMAT_Undefined is valid
					) const;

				//! Get array of valid usages.
				void GetValidUsages (
					SIMPLE_ARRAY<DBFIELDUSAGE> &ValidUsages	//!< Array of valid usages returned, will be empty if only DBFIELDUSAGE_Undefined is valid
					) const;

				//! Get FIELDVALUE::TYPE for field.
				FIELDVALUE::TYPE GetValueType () const;

				//! Is the field related to another table / field
				//! @return 'True' if field is related
				bool HasRelation () const;
			
				//! Does the field have a next key field link?
				//! @return 'True' if field has a next key field
				bool HasNextKey () const;
			
				//! Does the field have a next key pseudo field link?
				//! @return 'True' if field has a next pseudo key field
				bool HasPseudoNextKey () const;

				//! Determine if field is date or time.
				bool IsDateTime () const;

				//! Determine if field is numeric floating-point type.
				bool IsFloat () const;
					
				//! Is the field numeric?  (not a string that can be converted to a number)
				//! @return 'True' if field is a numeric field
				bool IsNumeric () const;

				//! Is the field a string?
				//! @return 'True' if field is a string.
				bool IsString () const;

				//! Reset position information about the field			
				void ResetPosition ();
					
				//! Set the AutoIncrement number for the next added record
				void SetAutoIncrement (
					UINT32 count
					);

				void SetFieldControl (
					const DBFIELDCONTROL &FieldControl
					) { m_FieldControl = FieldControl; }
			
				void SetDescription (
					const MISTRING &Description
					) { m_Descriptor.SetDescription(Description); }

				void SetDescriptor (
					const RVC::DESCRIPTOR &Descriptor
					) { m_Descriptor = Descriptor; }

				//! Set the number of decimal places for floating point fields
				void SetDecimalPlaces (
					int decplaces
					);
					
				//! Set the display units for this field
				void SetDispUnits (
					int unit
					);
					
				//! Set the display width (number of columns) for the field
				void SetDispWidth (
					int width
					);
					
				//! Set the Domain ID for this field
				void SetDomainID (
					const RVC::OBJECTNAME &name
					);
					
				//! Set field flags
				void SetFlags (
					DBFIELDFLAGS flags
					);
					
				//! get the format of the field.
				//! Currently only used for BLOB fields to record what kind of data is
				//!	stored in the blob.
				void SetFormat (
					FORMAT format
					);
					
				//! Set this field to link to a field in another table
				void SetLink (
					INT32 TableNum,
					INT32 FieldNum,
					INT32 NextFieldNum = -1
					);
					
				//! Set this field to link to a field in another table
				void SetLink (
					INT32 TableNum,
					INT32 FieldNum,
					NEXTFIELD NextFieldNum
					);
					
				//! Set the name of this field
				void SetName (
					const RVC::OBJECTNAME &name
					);
					
				//! Set the name of this field
				void SetName (
					const MISTRING &name
					);
					
				//! Set the name of this field
				//! The TEXTID version is necessary to resolve an abmiguious function 
				//! call since MISTRING and RVC::OBJECTNAME both have implicit ctors
				//! from TEXTID
				void SetName (
					TEXTID name
					);
					
				//! Set the next field number that this field is linked to
				void SetNextLinkField (
					INT32 NextFieldNum
					);
					
				//! Set this field to link to a field in another table
				void SetNextLinkField (
					NEXTFIELD NextFieldNum
					);
					
				//! Set the offset into the buffer for data of this field.
				//! You don't normally have to set this.  It will be set for you when
				//! you create the table.
				void SetOffset (
					int offset
					);

				//! Set the size of this field in bytes
				void SetSize (
					INT32 size
					);
					
				//! Set the field type.
				void SetType (
					DBFIELDTYPE type			//!< Field type
					);

				//! Set the field units
				//! Note: this sets the unit type and the "Units Stored"
				//! but leaves the "Units Shown" alone.  
				void SetUnits (
					const UNITCONV &conv
					);

				//! Set the field units
				//! This method allows you to choose to set only the units stored or
				//! the units shown.  The "units shown" is mostly for historical 
				//! reasons.
				void SetUnits (
					const UOM::UNIT &units,
					bool bSetUnitsStored = true,
					bool bSetUnitsShown = true
					);
					
				//! Set the field units
				//! This method allows you to choose to set only the units stored or
				//! the units shown.  The "units shown" is mostly for historical 
				//! reasons.
				void SetUnits (
					UOM::IDNUM units,
					bool bSetUnitsStored = true,
					bool bSetUnitsShown = true
					);
					
				//! Set the field units
				//! Note: This will only set the "Units Shown", which is only used by
				//!	Single Record View.  And it will refuse to change it if the UnitType
				//!	of the conv parameter does not match the field's unit type.
				void SetUnitsShown (
					const UNITCONV &conv
					);
								
				//! Set the field usage
				void SetUsage (
					DBFIELDUSAGE usage
					);
			
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DBFIELDINFO *m_pFieldInfo;
				RVC::DESCRIPTOR m_Descriptor;
				DBFIELDCONTROL m_FieldControl;

				const DBFIELDINFO* GetFieldInfoPtr () const { return (m_pFieldInfo); }
				DBFIELDINFO* GetFieldInfoPtr () { return (m_pFieldInfo); }
				
				friend class DBTABLE;
				friend class DBTABLE::CURSOR;
				friend class DBTABLE::RECORD;
				friend class SHAPE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};


		//! Get TEXTID for attachment.
		static TEXTID GetAttachmentTextID (
			ATTACHMENT attachment			//!< Attachment type
			);

		//! Get (single-line) string for displaying field type.
		static MISTRING GetFieldTypeString (
			DBFIELDTYPE FieldType,
			DBFIELDFLAGS FieldFlags,
			CHAR_ENCODING encoding,
			bool bIncludeRange = false
			);
		
		//! Determine if a database type supports a given capability.
		static bool HasDBTypeCapability (
			LINKTYPE LinkType,
			CAPABILITY Capability
			);

		//! Default constructor
		DBTABLE ();

		//! Copy constructor
		DBTABLE (								//! Copy Constructor
			const DBTABLE &rhs
			);

		//! Destructor
		virtual ~DBTABLE ();

		//! Assignment
		DBTABLE& operator= (
			const DBTABLE &rhs
			);

		//! Add a record to the table
		//! @return Record number for the new record
		//! Note: You should not depend on this returning the record number.
		//!	If the table is a link to any external format, it may be returning
		//!	a bookmark number instead.  Eventually this method will be modified
		//! 	to only return ERRVALUE.  This version would be deprecated, but 
		//!	it's the one you would call if you don't care what the new record
		//!	number is.
		INT32 AddRecord (
			RECORD &Record
			);
		
		//! Add a record to the table
		ERRVALUE AddRecord (
			RECORD &Record,		//!< Record to write
			BOOKMARK &Bookmark	//!< Bookmark of new record RETURNED
			);
		
		//! Create an index on a field in the table
		ERRVALUE BuildIndex (
			INT32 FieldNum,
			DBKEYFLAGS flags = DBKEYFLAG_Indexed		//!< Type of key to make
			);

		//! Set table attachment type
		//! Note: This method resolves existing attached/related records and
		//! tries to keep everything correct.  
		ERRVALUE ChangeAttachmentType (
			ATTACHMENT Attachment,
			ELEMTYPE ElemType = ELEMTYPE_Empty	
			);

		//! Combine rhs table with this table
      ERRVALUE Combine (
         const DBTABLE &rhs,
         DBASE::JOINFLAGS flags,
         DBASE &dest,          //!< Database in which to store table
         const RVC::DESCRIPTOR &DestDescriptor	//!< Descriptor of new table
         );

		//! Compare two table structures
		//! @return STRUCTCOMPARE enum value describing if and how the table structures match
		STRUCTCOMPARE CompareStructures (
			const DBTABLE &rhs
			) const;
			
		//! Determine the minimum ATTACHMENT type based on current attachmets.
		//!
		//! Looks at current element to record attachments to determine the
		//! minimum setting for attachment.  
		//!
		//! If the table is RelatedOnly the returned type will be one of the
		//!	attached (non-related) attachment types.  Call ChangeAttachmentType
		//!	to actually resolve the attachments.
		//! If the table is already OneToOne or Identity it will return that.
		//! It will never return suggest Identity unless the table is already 
		//!	Identity.
		ATTACHMENT ComputeMinimumAttachmentType (
			ELEMTYPE ElemType = ELEMTYPE_Empty
			) const;

		//! Copy the direct record attachments from one table to another
		ERRVALUE CopyRecordAttachmentsTo (
			DBTABLE &DestTable,
			const SIMPLE_ARRAY<INT32> &SrcToDestTrans
			) const;

		//! Copy a table to another database
		//! \param DestDatabase The database to copy to
		//! \param pDestDescriptor The new table's descriptor.  If not specified,
		//!		the new table will get the same descriptor as the source table.
		//! \param pRecNumTrans Optional, will contain returned
		//!		translation from source record numbers to destination record
		//!		numbers.  This is only an issue if the source table has deleted
		//!		records, or if the Filter is used to only copy a portion of the
		//!		table.
		//! \param pNewTableNumber Pointer to INT32 into which to return the new
		//!		table number.
		//! \param Filter Delegate to call to determine if a record should be
		//!		copied.  Example:
		//!			DELEGATE_BOOL_UINT32(&bits, &BITSET::GetBit)
		//! \param Credentials The credentials to use when creating the table
		//! \param flags Supported copy flags:
		//!		DBASE::COPYFLAG_RemoveStdAttribTables -- will silently refuse to copy the table if it's a standard attribute table
		//!		DBASE::COPYFLAG_TableLinks -- If the table is a link to an external table, it will just make a new link in the destination database
		//!		DBASE::COPYFLAG_MinimizeTableSize
		//!		DBASE::COPYFLAG_FreezeComputedFields -- Any computed fields will be computed and frozen (no longer computed).
		ERRVALUE CopyTo (
			DBASE &DestDatabase,
			RVC::DESCRIPTOR *pDestDescriptor = 0, //!< New table's descriptor (will use source table's if not specified)
			INT32 *pNewTableNumber = 0,
			ELEMNUMTRANS *pRecNumTrans = 0,
			DELEGATE_BOOL_UINT32 Filter = 0,
			ATTACHMENT attachment = ATTACHMENT_Unknown,	//!< New table attachment, Unknown to determine from source table
			DBASE::COPYFLAG flags = DBASE::COPYFLAG_None,
			const RVC::CREDENTIALS *pCredentials = 0
			) const;

		//! Drop a table (delete)
		ERRVALUE Drop (
			bool CleanDBList						//!< Remove the entries in the DBlist that refer to this table
			);

		//! Delete a record from the table using the record number
		ERRVALUE DeleteRecord (
			INT32 RecordNum
			);

		//! Delete a record from the table using the cursor and a bookmark
		ERRVALUE DeleteRecord (
			CURSOR &cursor,
			const BOOKMARK &bookmark
			);

		//! Find the field corresponding to the name
		//! @return Field number of >= 0, R_FieldNotFound if field does not exist, < 0 Error code
		INT32 FindField (
			const MISTRING &name
			) const;
			
		//! Find the field cooresponding to the name
		//! @return Field number of >= 0, R_FieldNotFound if field does not exist, < 0 Error code
		INT32 FindFieldByShortName (
			const RVC::OBJECTNAME &name
			) const;
			
		//! Find a field with a specific usage.
		//! Returns -1 if the table does not have a field with the specified 
		//! usage.  If the table has mutliple fields of the specified usage, it
		//! returns the first one.
		INT32 FindFieldByUsage (
			DBFIELDUSAGE Usage
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			double value,								//!< Buffer containing value to compare to
			SIMPLE_ARRAY<INT32> &FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const char* string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32> &FoundRecList,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const MIUNICODE* string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32> &FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const MISTRING &string,					//!< Buffer containing string to compare to
			SIMPLE_ARRAY<INT32> &FoundRecList
			) const;
			
		//! Find the records that match the record data for a given field
		//! @return Number of records found or <0 if error 
		INT32 FindRecords (
			INT32 FieldNum,
			const RECORD &record,					//!< Record containing stuff to compare to
			SIMPLE_ARRAY<INT32> &FoundRecList,
			INT32 RecordFieldNum = -1				//!< Record field number to use, default is 'FieldNum'
			) const;
			
		//! Get record numbers in this table attached to specified element.
		//! @return Number of records attached or < 0 if error 
		INT32 GetAttachedRecords (
			ELEMNUM ElemNum,
			ELEMTYPE ElemType,
			SIMPLE_ARRAY<INT32> &RecordList,
			FINDFLAG FindFlags = FINDFLAG_AllAttached,
			DBTABLE *pSourceTable = 0	//!< Source table if ElemType == ELEMTYPE_Record
			) const;

		//! Get record numbers in this table attached to specified element.
		//! @return Number of records attached or < 0 if error 
		INT32 GetAttachedRecords (
			const RVC::ELEMIDX &elemidx,
			SIMPLE_ARRAY<INT32> &RecordList,
			FINDFLAG FindFlags = FINDFLAG_AllAttached,
			DBTABLE *pSourceTable = 0	//!< Source table if ElemType == ELEMTYPE_Record
			) const;

		//! Get table attachment type
		//! @return Table attachment type
		ATTACHMENT GetAttachmentType () const;

		//! Get the computed field query for a field
		ERRVALUE GetComputedFieldQuery (
			INT32 FieldNum, 
			MISTRING &QueryStr
			) const;

		//! Get table encoding.
		//! @return Table encoding
		CHAR_ENCODING GetEncoding () const;
			
		//! Get the index for a field, will build one if needed
		ERRVALUE GetFieldIndex (
			INT32 FieldNum,
			SIMPLE_ARRAY<INT32> &Index, 				//!< List of record numbers in sorted order
			BITSET &DuplicateBits						//!< Bit set to true if value in the record for that field is equal to the previous record
			);

		//! Get the RVC::DESCRIPTOR for a single field
		ERRVALUE GetFieldDescriptor (
			INT32 FieldNum,
			RVC::DESCRIPTOR &Descriptor
			) const;

		//! Get the FIELDINFO for a single field
		ERRVALUE GetFieldInfo (
			INT32 FieldNum,
			FIELDINFO &FieldInfo
			) const;

		//! Get the FIELDINFO's for the table
		ERRVALUE GetFieldInfo (
			FIELDINFOARRAY &FieldArray
			) const;

		//! Get FIELDVALUE::TYPE for field.
		FIELDVALUE::TYPE GetFieldValueType (
			UINT32 FieldNum							//!< Field number
			) const;

		//! Get the list of fields linked to the starting field		
		ERRVALUE GetLinkFields (
			INT32 StartFieldNum,								//!< Field to start checking on
			SIMPLE_ARRAY<INT32> &FieldNumArray			//!< One item 'StartFieldNum' if no next_key links, > 1 items if next_key links exists
			) const;		 

		//! Get the link type of this table
		LINKTYPE GetLinkType () const;

		//! Get number of computed fields.
		INT32 GetNumComputedFields () const;

		//! Get the number of deleted records in the table
		INT32 GetNumDeletedRecords () const;

		//! Get the number of field in the table
		//! @return Number of fields
		INT32 GetNumFields () const;

		//! Get number of non-deleted records in table.
		INT32 GetNumNonDeletedRecords () const
			{ return (GetNumRecords() - GetNumDeletedRecords()); }

		//! Get the number of records in the table
		//! More accurately, this method returns the maximum record number + 1.
		//! If the table has any deleted records, they are not subtracted from
		//! the count.
		//! @return Number of records
		INT32 GetNumRecords () const;

		//! Get reference element (This table belongs to what element?)
		//! @return Reference flags
		REFELEM GetReferenceElement () const;
			
		//! Get the records from this table attached to the record in the basetable
		//! @return Number of records attached or <0 if error 
		INT32 GetRelatedRecords (
			INT32 RecordNum,												//!< Record in the base table to get attachments for
			INT32 BaseTableNum,											//!< Basetable that contains the record stated above
			SIMPLE_ARRAY<INT32> &RecordList,							//!< List of records that are related RETURNED
			FINDFLAG FindFlags = FINDFLAG_AllAttached				//!< find flags
			) const;

		//! Get the records from this table attached to the given record
		//! @return Number of records attached or <0 if error 
		INT32 GetRelatedRecords (
			const RECORD &record,										//!< Record to get attachments for
			SIMPLE_ARRAY<INT32> &RecordList,							//!< List of records that are related RETURNED
			FINDFLAG FindFlags = FINDFLAG_AllAttached				//!< find flags
			) const;

		//! Get the metadata for the table.
		//! Note that this method differs from RVC::OBJECT::GetMetadata in that
		//! this one will make sure that the returned metadata includes 
		//! descriptors for all the fields, even if the actual object doesn't
		//! have metadata for them.
		ERRVALUE GetTableMetadata (
			SIMPLE_ARRAY<UINT8> &buffer
			) const;

		//! Get table number
		//! @return Table number
		INT32 GetTableNumber () const;
			
		//! Get table usage
		//! @return Table usage
		USAGE GetUsage () const;
			
		//! Get the default view mode
		VIEWMODE GetViewMode () const;

		//! Get view position of table
		//! @return Table view position
		LRECT2D GetWindowRect () const;
			
		//! Get table capability
		//! @return Table capability
		bool HasCapability (
			CAPABILITY capability
			) const;

		//! Is the table hidden?
		//! @return 'True' if the table is hidden
		bool IsHidden () const;

		//! Is table an internal (no storage representation) table
		//! @return 'True' if the table is internal
		bool IsInternal () const;

		//! Is table missing.  Occurs with linked tables
		//! @return 'True' if the table is missing
		bool IsMissing () const;
			
		//! Is table a prompt table.  Used for adding elements
		//! @return 'True' if the table is a prompt table
		bool IsPrompt () const;

		//! Is table a UserDefined table.  (Currently USAGE_UserDefined||USAGE_Sketch||USAGE_Shape)
		//! @return 'True' if the table is a user defined table
		bool IsUserDefined () const;

		//! Is table valid.  Occurs with deleted tables
		//! @return 'True' if the table is valid
		bool IsValid () const;

		//! Can the table be written to?
		//! @return 'True' if the table is writable
		bool IsWritable () const;

		//! Link to an external table, field info returned in the LINKPARMS
		ERRVALUE Link (
			DBASE &parent,
			const DESCRIPTOR &Descriptor,
			const LINKPARMS &LinkParms,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a table
		ERRVALUE Make (
			DBASE &parent,
			const DESCRIPTOR &Descriptor,
			MAKEPARMS &MakeParms,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new table with one field listing all distinct values of
		//! a specific field.
		ERRVALUE MakeDistinctTableFromField (
			DBASE &parent,
			const DBTABLE &SourceTable,
			INT32 Field,					//! Field number from source table
			const DESCRIPTOR &Descriptor,
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Open an existing table using the table name
		ERRVALUE Open (
			const DBASE &parent,
			const MISTRING &TableName,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an existing table using the table name
		ERRVALUE Open (
			const DBASE &parent,
			const RVC::OBJECTNAME &ShortName,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an existing table using the table index
		ERRVALUE Open (
			const DBASE &parent,
			INT32 TableNum,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open a table using another instance of that table
		ERRVALUE Open (
			const DBTABLE &twin,
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an DBTABLE instance using a RVC::OBJITEM instance
		//!
		//! This method will attach and open the file and database table object if possible
		ERRVALUE Open (
			const RVC::OBJITEM &ObjItem,			//!< This contains the file and rvc object location information
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Open an <b>attached</b> DBTABLE instance.
		//! The instance must be attached for this to work.
		ERRVALUE Open (
#ifdef RVC_CREDENTIALS_MANDATORY
			OPENMODE OpenMode,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials //!< User credentials passed in, NULL for default behavior
#else
			OPENMODE OpenMode = OPENMODE_Read,	//!< Mode in which to open the rvc object
			MDLGPARENT dlgparent = 0,					//!< Parent dialog handle, 0 for default handle
			const CREDENTIALS *pCredentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Open an existing table
		// This is a private version used by SML, which really already has the
		// table open, but since RVC interface classes aren't available inside
		// RVCSYS, it only has internal handles.  
		ERRVALUE Open (
			const SML::TABLEDATA* TableData,
			OPENMODE mode = OPENMODE_Read
			// doesn't need credentials table really already open.
			);
	#endif  // doxygen

		//! Open object to contain child objects of table.
		//! The object actually opened will depend on whether the table is real or virtual.
		//! For a normal table the container will be the table itself.
		//! For the internal table (-1) the container will be the database.
		//! For other virtual tables (eg vector label style) the container will be a subobject of the database.
		//! @return TRUE if container exists, FALSE if not, < 0 if error.
		int OpenChildContainer (
			OBJECT &ObjContainer,					//!< Container object returned if exists
			bool bCreate = false						//!< Whether to create if does not exist
			);

		//! Read a record from the table using a bookmark (relative to the default bookmark cursor) into a pre-allocated buffer
		ERRVALUE Read (
			const BOOKMARK &Bookmark, 
			RECORD &Record
			) const { return (Read(0, 0, &Bookmark, Record, RECACC_Bookmark)); }

		//! Read a record from the table using a record number into a pre-allocated buffer
		ERRVALUE Read (
			INT32 RecordNum, 
			RECORD &Record
			) const { return (Read(0, RecordNum, 0, Record, RECACC_RecordNum)); }

		//! Read a record from the table using a cursor into a pre-allocated buffer
		ERRVALUE Read (
			CURSOR &cursor,
			RECORD &Record
			) const { return (Read(&cursor, 0, 0, Record, RECACC_Cursor)); }
			
		//! Read a record from the table using a cursor / bookmark pair into a pre-allocated buffer
		ERRVALUE Read (
			CURSOR &cursor,
			const BOOKMARK &bookmark,
			RECORD &Record
			) const { return (Read(&cursor, 0, &bookmark, Record, RECACC_Bookmark)); }
			
		//! Read a record from the table using a record number into a pre-allocated buffer
		ERRVALUE ReadDefault (
			RECORD &Record
			) const { return (Read(0, RECORD_New, 0, Record, RECACC_RecordNum)); }

		ERRVALUE ReadFieldControls (
			std::vector<DBFIELDCONTROL> &FieldControls
			) const;

		INT32 ReadInverseDBList (
			INT32 RecNum,
			SIMPLE_ARRAY<DBLISTELMT> &List
			);

		//! Get a constant iterator to the first viable record in the table
		void RecordBegin (
			CONST_RECORDITERATOR &Iterator, 
			ITERATERECORD IterateRecord = ITERATERECORD_SkipDeleted,
			INT32 FieldNum = 0
			) const;

		//! Get an iterator to the first viable record in the table
		void RecordBegin (
			RECORDITERATOR &Iterator, 
			ITERATERECORD IterateRecord = ITERATERECORD_SkipDeleted,
			INT32 FieldNum = 0
			);

		//! Remove duplicate records from the table
		ERRVALUE RemoveDuplicateRecords ();

		//! Remove unattached records from the table
		ERRVALUE RemoveUnattachedRecords ();

		//! Set the number of records in the table.  Does not work with ATTACHMENT_Identity tables
		ERRVALUE Resize (
			INT32 NewNumRecords,
			bool CleanDBList
			);

		//! Search field index with a value for the record match
		//! @return Number of matches or error < 0
		template <class _CT>
		INT32 SearchIndex (
			INT32 FieldNum,
			const _CT &item,
			const FIELDINFO &finfo, 
			INT32 &offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8	//!< Encoding to use (ignored except for non-unicode string fields)
			) const { return (SearchIndex(FieldNum, &item, &finfo, &offset, flag)); }

		//! Search field index with a value that matches the field to search for the record match
		//! @return Number of matches or error < 0
		template <class _CT>
		INT32 SearchIndex (
			INT32 FieldNum,
			const _CT &item,				//!< MUST MATCH 'FieldNum' field type and size
			INT32 &offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8	//!< Encoding to use (ignored except for non-unicode string fields)
			) const { return (SearchIndex(FieldNum, &item, 0, &offset, flag, Encoding)); }

		//! Search field index with a double value for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			double value,
			INT32 &offset, 
			INDEXSEARCH flag
			) const;
			
		//! Search field index with a string value for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			const MISTRING &value,
			INT32 &offset, 
			INDEXSEARCH flag
			) const;
			
		//! Search field index with a value in the RECORD for the match
		//! @return Number of matches or error < 0
		INT32 SearchIndex (
			INT32 FieldNum,
			const RECORD &Record, 
			INT32 RecFieldNum,
			INT32 &offset, 
			INDEXSEARCH flag
			) const;
		
		//! Set table attachment type
		//! Note: This just blindly sets the attachment type without resolving
		//! anything.  Call ChangeAttachmentType if you need to resolve existing
		//! relationships and attachments
		ERRVALUE SetAttachmentType (
			ATTACHMENT Attachment
			);

		//! Set the computed field query for a field
		ERRVALUE SetComputedFieldQuery (
			INT32 FieldNum,
			const MISTRING &QueryStr
			);
			
		//! Set table encoding
		void SetEncoding (
			CHAR_ENCODING Encoding
			);

		//! Set the RVC::DESCRIPTOR for a single field
		//! Note, there is no field number.  The ShortName of the descriptor
		//! should match the short name of an existing field
		ERRVALUE SetFieldDescriptor (
			const RVC::DESCRIPTOR &Descriptor
			);

		//! Modify specific table FIELDINFO
		ERRVALUE SetFieldInfo (
			INT32 FieldNum,
			const FIELDINFO &FieldInfo
			);

		//! Modify all table FIELDINFO's
		ERRVALUE SetFieldInfo (
			FIELDINFOARRAY &FieldArray
			);

		//! Modify all table FIELDINFO's
		ERRVALUE SetFieldInfo (
			FIELDINFOARRAY &FieldArray,
			const SIMPLE_ARRAY<int> &NewToOld
			);

		//! Set whether table is "hidden".
		ERRVALUE SetHidden (
			bool hidden
			);

		//! Set whether or not this table should "Prompt when adding elements"
		ERRVALUE SetPromptOnAddElement (
			bool prompt = true
			);

		//! Set the reference element type
		ERRVALUE SetReferenceElement (
			REFELEM RefElem
			);

		//! Set the "read only" bit in the table header
		//! (will go away in favor of ACL)
		ERRVALUE SetReadOnly (
			bool bReadOnly = true
			);

		//! Set table usage
		ERRVALUE SetUsage (
			DBTABLE::USAGE usage
			);

		//! Set the default view mode of the table
		ERRVALUE SetViewMode (
			VIEWMODE ViewMode
			);

		//! Set view position of table
		void SetWindowRect (
			const LRECT2D &rect
			);

		//! If a table is an ODBC link to Access or Excel, this method will
		//! convert it to an OLE-DB link.  WARNING! Doing so will render the
		//! link unusable in TNT 2005:71 and earlier.
		ERRVALUE UpgradeLinkToOLEDB (
			);

		//! Determine if the current user has permission to access this table
		//! \return 0 If user has access, error code < 0 if not
		ERRVALUE ValidateAccess (
			PERMISSIONS Permission,
			bool bGenerateErrorMsg = true	//!< Pass false to not SetErrPosn on error
			) const;

		//! Determine if the current user has permission to a given field in this table.
		//! \return 0 If user has access, error code < 0 if not
		ERRVALUE ValidateAccess (
			INT32 FieldNum,
			PERMISSIONS Permission,
			bool bGenerateErrorMsg = true	//!< Pass false to not SetErrPosn on error
			) const;

		//! Make sure that a table and field names are valid and unique
		//! This version assumes the table already exists.  There is also a 
		//! ValidateTableName method on DBASE for use when creating a new table
		ERRVALUE ValidateTableName (
			OBJECTNAME &NewTableName,
			SIMPLE_ARRAY<OBJECTNAME> &NewFieldNames
			);

		//! Write a record in the table using a record number from a buffer
		ERRVALUE Write (
			INT32 RecordNum, 
			RECORD &Record,
			UINT32 flags = 0
			) { return (Write(0, RecordNum, 0, Record, RECACC_RecordNum, flags)); }

		//! Write a record to the table using a bookmark (relative to the default bookmark cursor) 
		ERRVALUE Write (
			const BOOKMARK &Bookmark, 
			RECORD &Record,
			UINT32 flags = 0
			) { return (Write(0, 0, &Bookmark, Record, RECACC_Bookmark, flags)); }

		//! Write a record in the table using a cursor from a buffer
		ERRVALUE Write (
			CURSOR &cursor,
			RECORD &Record,
			UINT32 flags = 0
			) { return (Write(&cursor,0, 0, Record, RECACC_Cursor, flags)); }

		//! Write a record in the table using a cursor / bookmark pair from a buffer
		ERRVALUE Write (
			CURSOR &cursor,
			const BOOKMARK &bookmark,
			RECORD &Record,
			UINT32 flags = 0
			) { return (Write(&cursor, 0, &bookmark, Record, RECACC_Bookmark, flags)); }

		ERRVALUE WriteFieldControls (
			const std::vector<DBFIELDCONTROL> &FieldControls
			);

	protected:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		DBASE XXXdelete_ParentObj;
		DATABASEHANDLE* m_DBHandle;
		OBJITEM m_DBObjItem;
		int m_TableID;		// Actually the table number
		
		DATABASEHANDLE* GetDBHandle() const { return (m_DBHandle); }

		virtual bool v_IsValidTable (const DBTABLE &table);
		virtual bool v_IsObjectOpen () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum RECACC {
			RECACC_RecordNum,
			RECACC_Cursor,
			RECACC_Bookmark
			};

		static INT32 TestIsJoinable (
			const DBTABLE &src,
			const DBTABLE &dest,
			DBTABLETRANS &TableTrans,
			DBASE::COPYFLAG CopyFlags,
			DBASE::JOINFLAGS JoinFlags
			);
			
		ERRVALUE AppendTo (
			DBASE &ObjDbTgt,
			DBTRANSINFO &TransInfo,
			DBTABLETRANS &TableTrans,
			INT32 &NextElemID,
			DBASE::JOINFLAGS flags
			) const;

		ERRVALUE CopyComputedFieldExpressions (
			const DBTABLE &SrcTable,										//!< Source table
			const SIMPLE_ARRAY<INT32> &FieldTransSrcToTgt,			//!< Field number translation from source to target
			DELEGATE_VOID_MISTRING_NONCONST delegateAdjust = 0		//!< Delegate to adjust expression string
			);

		ERRVALUE CopyRecordAttachmentsLow (
			const DBTABLE *pSrcTable1,
			DBTABLETRANS *pTableTrans1,
			const DBTABLE *pSrcTable2,
			DBTABLETRANS *pTableTrans2,
			DBASE::JOINFLAGS JoinFlags
			);

		INT32 GetRecordSize () const;

		ERRVALUE Read (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			RECORD &Record,
			RECACC flags
			) const;

		INT32 SearchIndex (
			INT32 FieldNum,
			void* buf,
			const FIELDINFO* finfo,
			INT32 &offset, 
			INDEXSEARCH flag = INDEXSEARCH_ReturnIndexOffset,
			CHAR_ENCODING = CHAR_ENCODING_UTF8
			) const;

		static void TableCallbackFunc (DBCALLBACKSTRUCT *cb, void *data);

		ERRVALUE Write (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			RECORD &Record,
			RECACC flags,
			UINT32 TableFlags
			);

		ERRVALUE WriteBinary (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			INT32 FieldNum,
			const SIMPLE_ARRAY<UINT8> &binary,
			RECACC flags
			);

		ERRVALUE WriteMemo (
			CURSOR* cursor,
			INT32 RecordNum,
			const BOOKMARK* bookmark,
			INT32 FieldNum,
			const MISTRING &string,
			RECACC flags
			);

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual ERRVALUE v_GetDescriptor (RVC::DESCRIPTOR &Descriptor, const CREDENTIALS *pCredentials) const;
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual ERRVALUE v_GetParent (RVC::OBJECT &ObjParent) const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS *pCredentials);

		friend class CURSOR;
		friend class DBASE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};


//! Class to represent database bookmarks used in ODBC
class DBTABLE::BOOKMARK {
	public:
		
		//! Default constructor
		explicit BOOKMARK (
			UINT32 Bookmark = 0
			) : m_Bookmark(Bookmark) {}
	
		//! Copy constructor
		BOOKMARK (
			const BOOKMARK &rhs
			) : m_Bookmark(rhs.m_Bookmark) {}

		//! Assignment
		BOOKMARK &operator= (
			const BOOKMARK &rhs
			) {
			if (this != &rhs) m_Bookmark = rhs.m_Bookmark;
			return (*this);
			}
	
		bool operator== (
			const BOOKMARK &rhs
			) const { return (m_Bookmark == rhs.m_Bookmark); }
		
		UINT32 GetValue (
			) const { return m_Bookmark; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_Bookmark;
	
		friend class DBTABLE;
		friend class DBTABLE::CURSOR;
		friend class DBTABLE::RECORD;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Class to represent database table cursors used in ODBC
class DBTABLE::CURSOR {
	public:
		enum FUNCTION {
			FUNCTION_None =	 		0x00000000,
			FUNCTION_Unique = 		0x00000001,
			FUNCTION_Sorted = 		0x00000002,
			FUNCTION_Equal =			0x00000004,
			FUNCTION_GreaterThan =	0x00000008,
			FUNCTION_LessThan =		0x00000010,
			FUNCTION_NotEqual =		0x00000020,
			FUNCTION_AND =				0x00000040,
			FUNCTION_OR =				0x00000080,
			FUNCTION_NoClear =		0x00000100,	//!< Add to previous function
			FUNCTION_ForUpdate =		0x00000200	//!< To allow writes
			};
		
		//! Flags for GetIndex
		enum INDEXFLAG {
			INDEXFLAG_None = 0,
			INDEXFLAG_IndexOffset = 0x00000001, //<! Give index offset indstead of record number
			INDEXFLAG_LeaveRunBit = 0x00000002 //<! Leave the high bit set on the record number if the record is the first in a run of same-value records
			};

		//! Default constructor
		explicit CURSOR (
			const DBTABLE& table
			);
			
		//! Destructor
		~CURSOR ();
			
		//! Equality
		bool operator== (
			const CURSOR &rhs
			) const;
			
		//! Get the current bookmark		
		ERRVALUE GetBookmark (
			BOOKMARK& Bookmark
			) const;
			
		//! Get the offset into the rowset
		//! Note that INDEXFLAG_LeaveRunBit is meaningless for ODBC and OLE DB
		ERRVALUE GetIndex (
			INT32& Index,
			INDEXFLAG flags = INDEXFLAG_None
			) const;

	#ifndef NO_DEPRECATED
		//! Get the current record number or < 0 if error
		//! \deprecated Use GetBookmark instead.  
		//! This method will NOT work for ODBC or RDBMS.
		DEPRECATED INT32 GetRecordNum () const;
	#endif
			
		//! Is the cursor "done"
		//! Note: this will return true right after a Rewind() until you call
		//! Next().
		bool IsDone (
			) const { return (!IsValid()); }

		//! Is the cursor valid
		//! @return True for a valid cursor
		bool IsValid () const;

		//! Increment cursor
		//! @return > 0 if still valid, 0 if not, < 0 error
		//! \code
		//! DBTABLE::CURSOR Cursor(Table);
		//! while (Cursor.Next() > 0) {
		//!    ...
		//!    }
		//! \endcode
		int Next ();
			
		//! Reset cursor to the beginning
		//! Note: After calling Rewind, the cursor is actually pointing
		//! one record before the beginning.  It is intended that you should
		//! use it like this...
		//! \code
		//! DBTABLE::CURSOR Cursor(Table);
		//!   ...
		//! Cursor.Rewind();	// only necessary if you've already used Cursor
		//! while (Cursor.Next() > 0) {
		//!    ...
		//!    }
		//! \endcode
		ERRVALUE Rewind ();

		//! Seek to a spot in the table.
		//! \return 1 if value found, 0 if not found but seek to closest match
		//! This method will NOT work for ODBC or RDBMS.
		INT32 SeekValue (
			INT32 FieldNum,
			const void* vbuf,
			const FIELDINFO* finfo,
			bool bSingleField = true
			);

		//! Seek to a spot in the table.
		//! \return 1 if value found, 0 if not found but seek to closest match
		//! This method will NOT work for ODBC or RDBMS.
		INT32 SeekValue (
			INT32 FieldNum,
			const MISTRING& value
			);

		//! Set cursor behavior
		//! Note: the only functions that will work with this method are Sort
		//! and Unique.  Sort has its own function which is better.
		ERRVALUE SetFunction (
			INT32 FieldNum,
			FUNCTION Function
			);
	
		//! Set cursor behavior
		ERRVALUE SetSort (
			const SIMPLE_ARRAY<INT32>& FieldList	//!< fields to sort on
			);
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const DBTABLE& m_Table;

		class PRIV;

		PRIV *m_Cursor;	

		CURSOR (const CURSOR &rhs);
		CURSOR& operator=(const CURSOR &rhs);
		
		friend class DBTABLE;
		friend class DBTABLE::RECORD;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	

//! Database table object selection filter that excludes all objects except for tables.
class DBTABLE::GETOBJFILTER_ISSELECTABLE : public RVC::GETOBJFILTER {
	public:
		GETOBJFILTER_ISSELECTABLE() : GETOBJFILTER() { }
		virtual ~GETOBJFILTER_ISSELECTABLE() { }

	private:
		virtual bool v_IsSelectable (
			const RVC::OBJITEM& objitem,
			int position
			);

		virtual bool v_HasSelectableSubObjects (
			const RVC::OBJITEM& objitem
			);

		virtual bool v_IsValidParent (
			const RVC::OBJITEM& objitem
			);
	};

//=====================================================================================================================

//! Parameters to link to external database tables.
class DBTABLE::LINKPARMS {
	public:

		//! Link flags.
		enum LINKFLAG {
			LINKFLAG_None = 				0,
			LINKFLAG_KeepPassword = 	0x00800000
			};

		//! Construct with FILEPATH.
		LINKPARMS (
			const FILEPATH& filepath,					//!< Local file path
			DBTABLE::LINKTYPE LinkType,				//!< Link type
			CHAR_ENCODING Encoding,						//!< External table character encoding
			LINKFLAG LinkFlag = LINKFLAG_None,		//!< Flags if needed
			char Delimiter = ','							//!< Field delimiter for CSV and similar
			);

		//! Construct with link string.
		LINKPARMS (
			const MISTRING& LinkString,				//!< Link string, for RDBMS must include separate 'TABLE=' to identify table being linked to
			DBTABLE::LINKTYPE LinkType,				//!< Link type
			CHAR_ENCODING Encoding,						//!< External table character encoding
			LINKFLAG LinkFlag = LINKFLAG_None,		//!< Flags if needed
			char Delimiter = ','							//!< Field delimiter for CSV and similar
			);
	
		//! Destructor
		~LINKPARMS ();

		char GetDelimiter () const
			{ return (m_Delimiter); }

		//! Get the encoding of the external table.
		CHAR_ENCODING GetEncoding () const
			{ return (m_Encoding); }

		LINKFLAG GetLinkFlag () const
			{ return (m_LinkFlag); }

		const MISTRING& GetLinkString () const
			{ return (m_LinkString); }

		//! Get the type of external link
		//! @return External link type
		DBTABLE::LINKTYPE GetLinkType () const
			{ return (m_LinkType); }

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_LinkString;
		DBTABLE::LINKTYPE m_LinkType;
		CHAR_ENCODING m_Encoding;
		LINKFLAG m_LinkFlag;
		char m_Delimiter;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
//	DBTABLE::MAKEPARMS and derived classes

//! Database table creation parameters class.  Manages the necessary parameters to create a database table
class DBTABLE::MAKEPARMS {
	public:

		//! Constructor
		MAKEPARMS (
			const DBTABLE::FIELDINFOARRAY& FieldInfoArray,
			DBTABLE::ATTACHMENT Attachment = ATTACHMENT_RelatedOnly,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8,
			DBTABLE::USAGE Usage = USAGE_UserDefined
			) :
			m_FieldInfoArray(FieldInfoArray),
			m_Attachment(Attachment),
			m_Encoding(Encoding),
			m_Usage(Usage),
			m_RefElem(REFELEM_All),
			m_LinkObject(0),
			m_LinkType(LINKTYPE_Internal),
			m_Delimiter(0),
			m_bPromptTable(false),
			m_UseFieldNamesAsHeading(false)
			{}
			
		//! Constructor
		MAKEPARMS (
			DBTABLE::ATTACHMENT Attachment = ATTACHMENT_RelatedOnly,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8,
			DBTABLE::USAGE Usage = USAGE_UserDefined
			) :
			m_Attachment(Attachment),
			m_Encoding(Encoding),
			m_Usage(Usage),
			m_RefElem(REFELEM_All),
			m_LinkObject(0),
			m_LinkType(LINKTYPE_Internal),
			m_Delimiter(0),
			m_bPromptTable(false),
			m_UseFieldNamesAsHeading(false)
			{}
			
		//! Template Constructor
		MAKEPARMS (
			const DBTABLE& Template
			);
			
		virtual ~MAKEPARMS (){}

		//! Get attachment type
		//! @return Table attachment type
		DBTABLE::ATTACHMENT GetAttachmentType (
			) const { return (m_Attachment); }

		//! Get the delimiter for CSV tables
		//! if not set, ',' is assumed
		char GetDelimiter (
			) const { return (m_Delimiter); }

		//! Get encoding
		//! @return Table encoding
		CHAR_ENCODING GetEncoding (
			) const { return (m_Encoding); }

		//! Get field info array
		//! @return Table field info array
		const DBTABLE::FIELDINFOARRAY& GetFieldInfo (
			) const { return (m_FieldInfoArray); }

		//! Get table link file
		//! @return Link table file to create
		const MISTRING& GetLinkFile (
			) const { return (m_NewLinkFile); }
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Get object to create a link under the table			
		const RVC::OBJECT* GetLinkObject (
			) const { return (m_LinkObject); }
		#endif // GENERATING_DOXYGEN_OUTPUT
			
		//! Get table link type
		//! @return Table link type
		DBTABLE::LINKTYPE GetLinkType (
			) const { return (m_LinkType); }
			
		//! Get whether or not this table should "Prompt when adding elements"
		bool GetPromptOnAddElement (
			) const { return m_bPromptTable; }

		//! Get reference element (This table belongs to what element?)
		//! @return Reference flags
		DBTABLE::REFELEM GetReferenceElement (
			) const { return (m_RefElem); }
			
		//! Get table usage
		//! @return Table usage
		DBTABLE::USAGE GetUsage (
			) const { return (m_Usage); }

		//! Get whether to use field names as heading where supported.
		bool GetUseFieldNamesAsHeading () const
			{ return (m_UseFieldNamesAsHeading); }

		//! Set table attachment type
		void SetAttachmentType (
			ATTACHMENT Attachment
			) {m_Attachment = ((Attachment == ATTACHMENT_Unknown) ? ATTACHMENT_NoRestrictions : Attachment); }

		//! Set the delimiter
		//! Only used for CSV tables and only necessary if changing it from
		//! something other than ','
		void SetDelimiter (
			char delim
			) { m_Delimiter = delim; }

		//! Set table encoding
		void SetEncoding (
			CHAR_ENCODING Encoding
			) {m_Encoding = Encoding; }

		//! Set table field info array
		void SetFieldInfo (
			const FIELDINFOARRAY& FieldInfoArray
			) {m_FieldInfoArray = FieldInfoArray; }

		//! Set table link file, see SetLinkType()
		void SetLinkFile (
			const MISTRING& filename
			) {m_NewLinkFile = filename; }
			
		//! Set table link type, this will create a new table of this link type			
		void SetLinkType (
			LINKTYPE LinkType
			) {m_LinkType = LinkType; }
			
		//! Set whether or not this table should "Prompt when adding elements"
		void SetPromptOnAddElement (
			bool prompt = true
			) { m_bPromptTable = prompt; }

		//! Set table reference elements
		void SetReferenceElement (
			REFELEM RefElem
			) {m_RefElem = RefElem; }
			
		//! Set table usage
		void SetUsage (
			USAGE Usage
			) {m_Usage = Usage; }

		//! Set whether to use field names as heading where supported.
		void SetUseFieldNamesAsHeading (
			bool UseFieldNamesAsHeading
			) { m_UseFieldNamesAsHeading = UseFieldNamesAsHeading; }
			
	protected:

		DBTABLE::FIELDINFOARRAY& GetFieldArray (
			) { return (m_FieldInfoArray); }
			
		void SetLinkObject (
			const OBJECT& object
			) {m_LinkObject = &object; }
			
		void SetupRelatedFields (
			const DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBFIELDFLAGS RelFieldFlags,				//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			INT32 FirstFieldNum,
			INT32 NumStaticFields,
			bool UseRelatedSingle
			);

		//! Validate parameters.
		//! @return TRUE if modified, FALSE if not, < 0 if error.
		int Validate ();
			
	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FIELDINFOARRAY m_FieldInfoArray;
		DBTABLE::ATTACHMENT m_Attachment;
		CHAR_ENCODING m_Encoding;
		DBTABLE::USAGE m_Usage;
		DBTABLE::REFELEM m_RefElem;
		const OBJECT* m_LinkObject;
		DBTABLE::LINKTYPE m_LinkType;
		MISTRING m_NewLinkFile;
		char m_Delimiter;	
		bool m_bPromptTable;
		bool m_UseFieldNamesAsHeading;

		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Element ID tables
class DBTABLE::MAKEPARM_ID_ELEMENT : public DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_ID_ELEMENT ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Point ID tables
class DBTABLE::MAKEPARM_ID_POINT : public DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_ID_POINT ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Element Style tables.
//! Handles both single and multi-style tables, set of fields depends on USAGE specified in construction.
class DBTABLE::MAKEPARM_STYLE_ELEMENT : public DBTABLE::MAKEPARMS {
	public:

		//! Setup static fields in array based on usage
		static void SetupStaticFields (
			FIELDINFOARRAY& FieldInfoArray,				//!< Field array to populate, will be resized if needed
			USAGE Usage											//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle, USAGE_TextStyle, USAGE_MultiStyle
			);

		//! Constructor
		MAKEPARM_STYLE_ELEMENT (
			const DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBFIELDFLAGS RelFieldFlags,					//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle, USAGE_TextStyle, USAGE_MultiStyle
			bool bUseRelatedSingle
			);

		//! Constructor with specified style object for link.
		MAKEPARM_STYLE_ELEMENT (
			const DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBFIELDFLAGS RelFieldFlags,					//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			const RVC::STYLE& StyleObj,					//!< Style object to link table to
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle, USAGE_TextStyle, USAGE_MultiStyle
			bool bUseRelatedSingle
			);

		//! Set style object to link to.
		void SetStyleLink (
			const RVC::STYLE& StyleObj						//!< Style object to link table to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		void Setup (
			const DBTABLE& RelTable,					//!< Table to relate to
			INT32 RelFieldNum,								//!< Field in table to relate to
			DBFIELDFLAGS RelFieldFlags,					//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			USAGE Usage,										//!< USAGE_LineStyle, USAGE_PointStyle, USAGE_PolyStyle
			bool bUseRelatedSingle
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Scale Select tables
class DBTABLE::MAKEPARM_SCALESELECT : public DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_SCALESELECT (
			const DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBFIELDFLAGS RelFieldFlags,				//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			bool bUseRelatedSingle
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Attribute Select tables
class DBTABLE::MAKEPARM_SELECTION : public DBTABLE::MAKEPARMS {
	public:	
		//! Constructor
		MAKEPARM_SELECTION (
			const DBTABLE& RelTable,
			INT32 RelFieldNum,
			DBFIELDFLAGS RelFieldFlags,				//!< DBFIELDFLAG_RangeAbove, DBFIELDFLAG_RangeBelow
			bool bUseRelatedSingle
			);
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM for legend element table.
class DBTABLE::MAKEPARM_LEGENDELEMENT : public DBTABLE::MAKEPARMS {
	public:
		//! Constructor for OneToOne attachment to element.
		MAKEPARM_LEGENDELEMENT ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM for Node Turn Impedance table.
class DBTABLE::MAKEPARM_NODETURN : public DBTABLE::MAKEPARMS {
	public:
		//! Constructor for OneToOne attachment to element.
		MAKEPARM_NODETURN ();
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! MAKEPARM's for Oracle table creation
class DBTABLE::MAKEPARM_ORACLE : public DBTABLE::MAKEPARMS {
	public:
		MAKEPARM_ORACLE (
			) : MAKEPARMS(ATTACHMENT_RelatedOnly, CHAR_ENCODING_UTF8) { 
			SetLinkType(LINKTYPE_ORACLE); 
			}
			
		MAKEPARM_ORACLE (
			const DBTABLE::FIELDINFOARRAY& FieldInfoArray
			) : MAKEPARMS(FieldInfoArray, ATTACHMENT_RelatedOnly, CHAR_ENCODING_UTF8) {
			SetLinkType(LINKTYPE_ORACLE); 
			}
	
		MAKEPARM_ORACLE (
			const DBTABLE& Template
			) : MAKEPARMS(Template) {
			SetLinkType(LINKTYPE_ORACLE);
			SetAttachmentType(ATTACHMENT_RelatedOnly);
			SetUsage(USAGE_UserDefined);
			}
	};
	

//===================================================================================================================
//	DBTABLE::RECORD and derived classes

//! Class to contain a database record
class DBTABLE::RECORD {
	public:

		//! Flags for reading field methods (i.e. RVCDBTABLE::ReadField())
		enum FIELDFLAG {
			FIELDFLAG_None = 				0x0000,
			FIELDFLAG_NaN = 				0x0010,	//!< Return NaN for unknown values 
			FIELDFLAG_NoUnitConv = 		0x0020,	//!< Don't do any unit conversion 
			FIELDFLAG_LocalTime = 		0x0080,	//!< Show FIELD::TYPE_DateTime in local time zone
			FIELDFLAG_Strip0 =			0x0100,	//!< Strip trailing 0s when convert numeric field to string
			};

		//! Default constructor			
		RECORD ();
			
		//! Copy constructor			
		RECORD (
			const RECORD &rhs
			);
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD (
			const DBTABLE& table
			);

		//! Destructor			
		virtual ~RECORD ();
			
		//! Assignment
		RECORD& operator= (
			const RECORD &rhs
			);		
		
		//! Equality - compares if the information in the two records are the same, including memo and computed fields
		bool operator== (
			RECORD &rhs
			);		
		
		//bcolombe: added new CompareRecords function which works same as operator== but takes threshold value
		bool CompareRecords (
			RECORD &rhs,
			double threshhold = 0.0
			);

		//! Inequality - compares if the information in the two records are not the same, including memo and computed fields
		bool operator!= (
			RECORD &rhs
			) { return (!(*this == rhs)); }
		
		//! Initialize this record to work with this table, used to setup internal items
		ERRVALUE AssignTable (
			const DBTABLE& table
			);
			
		//! Clear the record
		void Clear ();

		//! Clear attachment.
		void ClearAttachment ();

		//! Compare two entries in a field to determine if they are equal
		bool CompareField (
			INT32 FieldNum,
			const RECORD& record,
			double threshhold = 0.0
			) const;

		//! Compare two entries in a field to determine sort order
		//! returns <0 if "this" is < record, >0 if this > record, = if same
		INT32 CompareFieldOrdered (
			INT32 FieldNum,
			const RECORD& record,
			double threshhold = 0.0
			) const;

		//! Compute a hash value for the current record.
		//! This is used by the database join code to quickly compare two records
		//! for equality.
		INT32 ComputeHash () const;

		//! Copy the fields that match in name and type from one record to another.
		//! Unlike using the assignment operator, which assumes the same table
		//! (or at least same structure), this method will cope with the 
		//! possibility that the source record may have different field sizes, 
		//! indexes, count and offsets than the destination record.
		ERRVALUE CopyMatchingFieldsTo (
			RECORD& DestRecord
			) const;

		//! Copy the field values from one record to another.
		//! Unlike using the assignment operator, which assumes the same table
		//! (or at least same structure), this method will cope with the 
		//! possibility that the source record may have different field sizes
		//! and offsets than the destination record.
		//! Note: it assumes that the records come from tables with the same
		//! number of fields and won't do any field reordering.
		ERRVALUE CopyTo (
			RECORD& DestRecord
			) const;

		//! Create (unattached) record with specified fields.
		ERRVALUE Create (
			const FIELDINFOARRAY &FieldInfo
			);

		//! Initialize the record with the constraints set for the table			
		ERRVALUE Initialize ();

		//! Read an entire record into a byte array		
		void GetData (
			SIMPLE_ARRAY<UINT8>& Data
			) const;

		//! Obtain the DBTABLE::FIELDINFO for the specified field
		//! @return Valid FIELDINFO if 'FieldNum' is valid, invalid FIELDINFO if 'FieldNum' is out of range
		const FIELDINFO& GetFieldInfo (
			INT32 FieldNum
			) const;

		//! Get the current record number referred to by this instance
		//! @return Record number if >= 0 or -1 if not associated with a record in a table
		INT32 GetRecordNumber () const;

		const DBTABLE *GetTable () const
			{ return (m_Table); }

		//! Get the current table number refered to by this instance
		//! @return Table number if >= 0 or TABLE_Invalid if not associated with a table
		INT32 GetTableNumber () const;

		//! Read field from this record into a FIELDVALUE.
		ERRVALUE GetValue (
			INT32 FieldNum,
			FIELDVALUE& value,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			) const;

		//! Read a field from this record into an MISTRING		
		ERRVALUE GetValue (
			INT32 FieldNum,
			MISTRING& string,
			FIELDFLAG FieldFlag = FIELDFLAG_None,
			int DecimalPlaces = -1,						//!< Precision, -1 to use table definition, -2 for automatic
			const UNITCONV *pUnitConv = 0
			) const;

		//! Read a field from this record and return a double
		//! @return Double value converted from the field specified
		double GetValue (
			INT32 FieldNum,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			) const;
			
		//! Read the value of a COLOR field
		ERRVALUE GetValue (
			INT32 FieldNum,
			COLOR& color
			) const;

		//! Read the value of a Date or DateTime field
		ERRVALUE GetValue (
			INT32 FieldNum,
			DATETIME& datetime
			) const;

		//! Read a binary field from this record into a SIMPLE_ARRAY<UINT8>
		//! This will return an empty array if the field is not a DBFIELDTYPE_Binary
		ERRVALUE GetValueBinary (
			INT32 FieldNum,
			SIMPLE_ARRAY<UINT8>& binary
			) const;

		//! Read an image out of a field.
		//!
		//! This will construct and return a SPATMOD::IMAGE::SOURCE from a
		//! field with USAGE_ImageBLOB or USAGE_ImageFile.  The caller is
		//! responsible for freeing the stage when they're done with it.
		//!
		//! If the bForThumbnail parameter, it will add a FILTER_ZOOM stage
		//! which will resize the image such that the maxium dimension is
		//! MaxThumbnailSize.
		//! 
		//! When requesting a thumbnail, if the field is followed by a field
		//! of USAGE_ImageThumbnail, it will use the thumbnail field instead
		//! of the requested field.  
		//! 
		//! If the requested field doesn't have an image USAGE, 
		//! this method will return EBadFuncParm.
		ERRVALUE GetValueImage (
			INT32 FieldNum,
			SPATMOD::IMAGE::STAGE*& pImage,		//!< Image returned
			bool bForThumbnail = false,
			INT32 MaxThumbnailSize = 100
			) const;

		//! Get FIELDVALUE::TYPE for field.
		FIELDVALUE::TYPE GetValueType (
			UINT32 FieldNum							//!< Field number
			) const;

		//! Determine if 'this' is assigned to a table
		//! @return 'True' if assigned to a table, 'false' if not
		bool IsAssigned () const;

		//! Write a set of fields from another RECORD field to this record if the tables are related 
		//! using the key field / next field association
		ERRVALUE SetKeyValues (
			INT32 StartFieldNum,
			const RECORD& SourceRecord
			);
			
		//! Write an entire record from a byte array		
		void SetData (
			const SIMPLE_ARRAY<UINT8>& Data
			);

		//! Write a field from a FIELDVALUE to this record.
		ERRVALUE SetValue (
			INT32 FieldNum,
			const FIELDVALUE& value,
			FIELDFLAG FieldFlag = FIELDFLAG_None
			);

		//! Write a field from a COLOR to this record (for COLOR fields only)
		ERRVALUE SetValue (
			INT32 FieldNum,
			const COLOR& color
			);

		//! Write the value of a Date or DateTime field 
		ERRVALUE SetValue (
			INT32 FieldNum,
			const DATETIME& datetime
			) ;

		//! Write a field from a MISTRING to this record
		ERRVALUE SetValue (
			INT32 FieldNum,
			const MISTRING& string,
			FIELDFLAG FieldFlag = FIELDFLAG_None,
			const UNITCONV *pUnitConv = 0
			);

		//! Write a field from a double to this record
		void SetValue (
			INT32 FieldNum,
			double value,
			bool NoUnitConv = false
			);
			
		//! Write a field from another RECORD field to this record
		ERRVALUE SetValue (
			INT32 FieldNum,
			const RECORD& SourceRecord,
			INT32 SourceFieldNum
			);
			
		//! Write a binary field to this record from a SIMPLE_ARRAY<UINT8>
		ERRVALUE SetValueBinary (
			INT32 FieldNum,
			const SIMPLE_ARRAY<UINT8>& binary
			) const;

	protected:
	
		virtual bool IsValidTable (const DBTABLE& /*table*/) { return (true); }
	
		const void *const GetFieldPtr (
			INT32 FieldNum
			) const;

		void *GetFieldPtr (
			INT32 FieldNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct BINARY {
			SIMPLE_ARRAY<UINT8> m_Binary;
			INT32 m_FieldNum;
			bool m_HasChanged;
			
			BINARY () : m_FieldNum(-1), m_HasChanged(false) {}
			};

		struct MEMO {
			MISTRING m_Memo;
			INT32 m_FieldNum;
			bool m_HasChanged;
			
			MEMO () : m_FieldNum(-1), m_HasChanged(false) {}
			};

		mutable void *m_Buffer;
		const DBTABLE* m_Table;
		DBTABLE::FIELDINFOARRAY m_DBFields;
		INT32 m_RecordNum;
		CURSOR* m_Cursor;
		BOOKMARK m_Bookmark;
		UINT32 m_dbioflags;
		int m_BufSize;
		mutable BITSET m_CompFieldRead;
		mutable MILIST<MEMO> m_MemoList;
		class TABLEOBSERVER;
		friend class TABLEOBSERVER;
		TABLEOBSERVER* m_TableObserver;
		mutable MILIST<BINARY> m_BinaryList;
		CHAR_ENCODING m_TableEncoding;
		
		INT32 CompareBinaryField (INT32 FieldNum, const RECORD& record) const;
		INT32 CompareMemoField (INT32 FieldNum, const RECORD& record) const;
		ERRVALUE ReadComputed (INT32 FieldNum) const;
		ERRVALUE ReadMemo (MEMO& Memo, INT32 FieldNum) const;
		ERRVALUE ReadBinary (BINARY& Binary, INT32 FieldNum) const;
		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Element ID table records
class DBTABLE::RECORD_ID_ELEMENT : public DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_ID_ELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_ID_ELEMENT (
			const RECORD_ID_ELEMENT &rhs
			) : RECORD(rhs) {}
			
		//! Copy constructor		
		RECORD_ID_ELEMENT (
			const RECORD &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from DBTABLE - used to setup internal items
		RECORD_ID_ELEMENT (
			const DBTABLE& table
			) : RECORD(table) {}

		//! Destructor			
		virtual ~RECORD_ID_ELEMENT (
			) {}
	
		//! Assignment
		RECORD_ID_ELEMENT& operator= (
			const RECORD_ID_ELEMENT &rhs
			) {
			RECORD::operator=(rhs);
			return (*this);
			}
		
		//! Get Current ID value
		//!	@return Current ID
		INT32 GetCurrent (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(0))); }
			
		//! Get Original ID value
		//!	@return Original ID
		INT32 GetOriginal (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(1))); }
			
		//! Set Current ID value
		void SetCurrent (
			INT32 current
			) {*static_cast<INT32*>(GetFieldPtr(0)) = current; }
			
		//! Set Original ID value
		void SetOriginal (
			INT32 original
			) {*static_cast<INT32*>(GetFieldPtr(1)) = original; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_ElementID); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Point ID table records
class DBTABLE::RECORD_ID_POINT : public DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_ID_POINT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_ID_POINT (
			const RECORD_ID_POINT &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_ID_POINT (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_ID_POINT (
			) {}
	
		//! Assignment
		RECORD_ID_POINT& operator= (
			const RECORD_ID_POINT &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
		
		//! Get Current ID value
		//!	@return Current ID
		INT32 GetCurrent (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(0))); }
			
		//! Set Current ID value
		void SetCurrent (
			INT32 current
			) {*static_cast<INT32*>(GetFieldPtr(0)) = current; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_ElementID); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Record for table containing single element style.
class DBTABLE::RECORD_STYLE_ELEMENT : public DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_STYLE_ELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_STYLE_ELEMENT (
			const RECORD_STYLE_ELEMENT &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_STYLE_ELEMENT (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_STYLE_ELEMENT (
			) {}
	
		//! Assignment
		RECORD_STYLE_ELEMENT& operator= (
			const RECORD_STYLE_ELEMENT &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		static INT32 GetRelationFieldNum (
			) { return (SAFIELD_SINGLE_RelField); }
		
		static INT32 GetSelectionFieldNum (
			) { return (SAFIELD_SINGLE_DrawFlag); }
	
		INT32 GetStyleIndex (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_SINGLE_StyleIndex))); }

		const RVC::OBJECTNAME& GetStyleName (
			) const { return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_SINGLE_StyleName))); }

		bool IsSelected (
			) const { return (*static_cast<const INT8*>(GetFieldPtr(SAFIELD_SINGLE_DrawFlag)) != 0); }

		ERRVALUE SetRelationValues (
			const RECORD& record
			) { return (SetKeyValues(SAFIELD_SINGLE_RelField, record)); }
		
		void SetSelected (
			bool Selected
			) {*static_cast<INT8*>(GetFieldPtr(SAFIELD_SINGLE_DrawFlag)) = Selected; }
			
		void SetStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_SINGLE_StyleIndex)) = index; }

		void SetStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_SINGLE_StyleName)) = name; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (
			const DBTABLE& table
			) {
			DBTABLE::USAGE usage = table.GetUsage();
			return (usage == DBTABLE::USAGE_PointStyle || usage == DBTABLE::USAGE_LineStyle || usage == DBTABLE::USAGE_PolyStyle);
			}
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//-------------------------------------------------------------------------------------------------------------------

//! Record containing multiple element styles.
class DBTABLE::RECORD_STYLE_MULTI : public DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_STYLE_MULTI (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_STYLE_MULTI (
			const RECORD_STYLE_MULTI &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_STYLE_MULTI (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_STYLE_MULTI (
			) {}
	
		//! Assignment
		RECORD_STYLE_MULTI& operator= (
			const RECORD_STYLE_MULTI &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		INT32 GetLineStyleIndex (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_LineStyleIndex))); }

		const RVC::OBJECTNAME& GetLineStyleName (
			) const { return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_LineStyleName))); }

		INT32 GetPointStyleIndex (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_PointStyleIndex))); }

		const RVC::OBJECTNAME& GetPointStyleName (
			) const { return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PointStyleName))); }

		INT32 GetPolyStyleIndex (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_PolyStyleIndex))); }

		const RVC::OBJECTNAME& GetPolyStyleName (
			) const { return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PolyStyleName))); }

		static INT32 GetRelationFieldNum (
			) { return (SAFIELD_CAD_RelField); }
		
		static INT32 GetSelectionFieldNum (
			) { return (SAFIELD_CAD_DrawFlag); }
	
		INT32 GetStyleIndex (
			RVC::STYLETYPE StyleType
			) const {
			switch (StyleType) {
				case STYLETYPE_Point: return (GetPointStyleIndex());
				case STYLETYPE_Line: return (GetLineStyleIndex());
				case STYLETYPE_Polygon: return (GetPolyStyleIndex());
				case STYLETYPE_Text: return (GetTextStyleIndex());
				}
			return (-1);
			}

		const RVC::OBJECTNAME& GetStyleName (
			RVC::STYLETYPE StyleType
			) const {
			switch (StyleType) {
				case STYLETYPE_Point: return (GetPointStyleName());
				case STYLETYPE_Line: return (GetLineStyleName());
				case STYLETYPE_Polygon: return (GetPolyStyleName());
				}
			return (GetTextStyleName());		//! Good as anything for a default
			}

		INT32 GetTextStyleIndex (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(SAFIELD_CAD_TextStyleIndex))); }

		const RVC::OBJECTNAME& GetTextStyleName (
			) const { return (*static_cast<const OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_TextStyleName))); }

		bool IsSelected (
			) const { return (*static_cast<const INT8*>(GetFieldPtr(SAFIELD_CAD_DrawFlag)) != 0); }

		void SetLineStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_LineStyleIndex)) = index; }

		void SetLineStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_LineStyleName)) = name; }

		void SetPointStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_PointStyleIndex)) = index; }

		void SetPointStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PointStyleName)) = name; }

		void SetPolyStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_PolyStyleIndex)) = index; }

		void SetPolyStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_PolyStyleName)) = name; }

		ERRVALUE SetRelationValues (
			const RECORD& record
			) { return (SetKeyValues(SAFIELD_CAD_RelField, record)); }
		
		void SetSelected (
			bool Selected
			) {*static_cast<INT8*>(GetFieldPtr(SAFIELD_CAD_DrawFlag)) = Selected; }
			
		void SetStyleIndex (
			RVC::STYLETYPE StyleType,
			INT32 index
			) {
			switch (StyleType) {
				case STYLETYPE_Point: SetPointStyleIndex(index); break;
				case STYLETYPE_Line: SetLineStyleIndex(index); break;
				case STYLETYPE_Polygon: SetPolyStyleIndex(index); break;
				case STYLETYPE_Text: SetTextStyleIndex(index); break;
				}
			return;
			}

		void SetStyleName (
			RVC::STYLETYPE StyleType,
			const RVC::OBJECTNAME& name
			) {
			switch (StyleType) {
				case STYLETYPE_Point: SetPointStyleName(name); break;
				case STYLETYPE_Line: SetLineStyleName(name); break;
				case STYLETYPE_Polygon: SetPolyStyleName(name); break;
				case STYLETYPE_Text: SetTextStyleName(name); break;
				}
			return;
			}

		void SetTextStyleIndex (
			INT32 index
			) {*static_cast<INT32*>(GetFieldPtr(SAFIELD_CAD_TextStyleIndex)) = index; }

		void SetTextStyleName (
			const RVC::OBJECTNAME& name
			) {*static_cast<OBJECTNAME*>(GetFieldPtr(SAFIELD_CAD_TextStyleName)) = name; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_MultiStyle); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_SCALESELECT : public DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_SCALESELECT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_SCALESELECT (
			const RECORD_SCALESELECT &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_SCALESELECT (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_SCALESELECT (
			) {}
	
		//! Assignment
		RECORD_SCALESELECT& operator= (
			const RECORD_SCALESELECT &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		double GetMaxScale (
			) const { return (*static_cast<const double*>(GetFieldPtr(SCALESELFIELD_MaxScale))); }

		double GetMinScale (
			) const { return (*static_cast<const double*>(GetFieldPtr(SCALESELFIELD_MinScale))); }

		static INT32 GetRelationFieldNum (
			) { return (SCALESELFIELD_RelField); }
	
		void SetMaxScale (
			double scale
			) {*static_cast<double*>(GetFieldPtr(SCALESELFIELD_MaxScale)) = scale; }

		void SetMinScale (
			double scale
			) {*static_cast<double*>(GetFieldPtr(SCALESELFIELD_MinScale)) = scale; }

		ERRVALUE SetRelationValues (
			const RECORD& record
			) { return (SetKeyValues(SCALESELFIELD_RelField, record)); }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_ScaleSelect); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_SELECTION : public DBTABLE::RECORD {
	public:
		//! Default constructor		
		RECORD_SELECTION (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_SELECTION (
			const RECORD_SELECTION &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from RVCDBTABLE - used to setup internal items
		RECORD_SELECTION (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_SELECTION (
			) {}
	
		//! Assignment
		RECORD_SELECTION& operator= (
			const RECORD_SELECTION &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		bool IsSelected (
			) const { return (*static_cast<const UINT8*>(GetFieldPtr(GetSelectionFieldNum())) != 0); }

		static INT32 GetRelationFieldNum (
			) { return (1); }
	
		static INT32 GetSelectionFieldNum (
			) { return (0); }
	
		void SetSelected (
			bool Selected
			) {*static_cast<UINT8*>(GetFieldPtr(GetSelectionFieldNum())) = Selected; }

		ERRVALUE SetRelationValues (
			const RECORD& record
			) { return (SetKeyValues(GetRelationFieldNum(), record)); }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_Selection); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

class DBTABLE::RECORD_LEGENDELEMENT : public DBTABLE::RECORD {
	public:

		enum FIELDNUM {
			FIELDNUM_ID =			0,
			FIELDNUM_PriLabel =	1,
			FIELDNUM_SecLabel =	2
			};

		//! Default constructor		
		RECORD_LEGENDELEMENT (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_LEGENDELEMENT (
			const RECORD_LEGENDELEMENT &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from DBTABLE - used to setup internal items
		RECORD_LEGENDELEMENT (
			const DBTABLE& table
			) : RECORD(table) {}
			
		//! Destructor			
		virtual ~RECORD_LEGENDELEMENT (
			) {}
	
		//! Assignment
		RECORD_LEGENDELEMENT& operator= (
			const RECORD_LEGENDELEMENT &rhs
			) {		
			RECORD::operator=(rhs);
			return (*this);
			}
			
		//! Get field number for ID field.
		static INT32 GetFieldNumID (
			) { return (FIELDNUM_ID); }
	
		//! Get field number for primary label field.
		static INT32 GetFieldNumPrimaryLabel (
			) { return (FIELDNUM_PriLabel); }
	
		//! Get field number for secondary label field.
		static INT32 GetFieldNumSecondaryLabel (
			) { return (FIELDNUM_SecLabel); }

		//! Get unique record ID.
		UINT32 GetID (
			) const { return (*static_cast<const UINT32*>(GetFieldPtr(FIELDNUM_ID))); }

		//! Get string for primary label.
		ERRVALUE GetPrimaryLabel (
			MISTRING& str
			) const { return (GetValue(FIELDNUM_PriLabel,str)); }
	
		//! Get string for secondary label.
		ERRVALUE GetSecondaryLabel (
			MISTRING& str
			) const { return (GetValue(FIELDNUM_SecLabel,str)); }
	
		//! Set string for primary label.
		ERRVALUE SetPrimaryLabel (
			const MISTRING& str
			) { return (SetValue(FIELDNUM_PriLabel,str)); }
	
		//! Set string for secondary label.
		ERRVALUE SetSecondaryLabel (
			const MISTRING& str
			) { return (SetValue(FIELDNUM_SecLabel,str)); }
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_LegendElement); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//-------------------------------------------------------------------------------------------------------------------

//! Specialized record class to manage Node Turn table records
class DBTABLE::RECORD_NODETURN : public DBTABLE::RECORD {
	public:
	
		//! Default constructor		
		RECORD_NODETURN (
			) : RECORD() {}
	
		//! Copy constructor		
		RECORD_NODETURN (
			const RECORD_NODETURN &rhs
			) : RECORD(rhs) {}
			
		//! Copy constructor		
		RECORD_NODETURN (
			const RECORD &rhs
			) : RECORD(rhs) {}
			
		//! Constructor from DBTABLE - used to setup internal items
		RECORD_NODETURN (
			const DBTABLE& table
			) : RECORD(table) {}

		//! Destructor			
		virtual ~RECORD_NODETURN (
			) {}
	
		//! Assignment
		RECORD_NODETURN& operator= (
			const RECORD_NODETURN &rhs
			) {
			RECORD::operator=(rhs);
			return (*this);
			}
			
		//! Set node impedance to invalidate turn
		void DisableTurn (
			) {*static_cast<double*>(GetFieldPtr(2)) = -1.0; }
		
		//! Get Line number coming into this node
		//!	@return Line In Number
		INT32 GetLineIn (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(0))); }
			
		//! Get Line number coming out of this node
		//!	@return Line Out Number
		INT32 GetLineOut (
			) const { return (*static_cast<const INT32*>(GetFieldPtr(1))); }
			
		//! Get Node Impedance
		//!	@return Node Impedance
		double GetImpedance (
			) const { return (*static_cast<const double*>(GetFieldPtr(2))); }
			
		//! Set Line number coming into this node
		void SetLineIn (
			INT32 LineIn
			) {*static_cast<INT32*>(GetFieldPtr(0)) = LineIn; }
			
		//! Set Line number coming out of this node
		void SetLineOut (
			INT32 LineOut
			) {*static_cast<INT32*>(GetFieldPtr(1)) = LineOut; }
		
		//! Set Node Impedance
		void SetImpedance (
			double Impedance
			) {*static_cast<double*>(GetFieldPtr(2)) = Impedance; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool IsValidTable (const DBTABLE& table) { return (table.GetUsage() == DBTABLE::USAGE_NodeTurn); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Constant record iterator class
//!
//! This class is used to iterate through a table's records.  This construct replaces the GetNumRecords() and cursor routines.
//! See methods RecordBegin() and RecordEnd() for obtaining a record iterator
class DBTABLE::CONST_RECORDITERATOR {
	public:
		//! Default constructor
		CONST_RECORDITERATOR (
			) : 
			m_TableObj(0),
			m_Record(0),
			m_NeedRecord(true),
			m_CheckSingle(false),
			m_FieldNum(0),
			m_Cursor(0)
			{}

		//! Destructor
		virtual ~CONST_RECORDITERATOR (
			) {
			delete m_Cursor;
			delete m_Record;
			}

		//! Dereference
		const RECORD& operator* (
			) const;

		//! Dereference arrow
		const RECORD* operator-> (
			) const { return (&**this); }

		//! Pre-increment
		CONST_RECORDITERATOR& operator++ (
			);

		bool Done (
			) const { return (m_Cursor == 0 || !m_Cursor->IsValid()); }
			
		ERRVALUE GetBookmark (
			DBTABLE::BOOKMARK& bookmark
			) const { return (m_Cursor->GetBookmark(bookmark)); }

	protected:
		mutable RECORD* m_Record;
		INT32 m_FieldNum;
		CURSOR* m_Cursor;
		mutable bool m_NeedRecord;
		bool m_CheckSingle;
		const DBTABLE* m_TableObj;
		
		void SetupIterator (const DBTABLE* table, ITERATERECORD IterateRecord, INT32 FieldNum);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CONST_RECORDITERATOR (const CONST_RECORDITERATOR &rhs);
		CONST_RECORDITERATOR& operator= (const CONST_RECORDITERATOR &rhs);
		
		friend class DBTABLE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Record iterator class
//!
//! This class is used to iterate through a table's records.  This construct replaces the GetNumRecords() and cursor routines.
//! See methods RecordBegin() and RecordEnd() for obtaining a record iterator
class DBTABLE::RECORDITERATOR : public DBTABLE::CONST_RECORDITERATOR {
	public:

		//! Default constructor
		RECORDITERATOR (
			) : CONST_RECORDITERATOR() {}

		//! Dereference
		RECORD& operator* (
			) { return (*const_cast<RECORD*>(&CONST_RECORDITERATOR::operator*())); }

		//! Dereference arrow
		RECORD* operator-> (
			) { return (&**this); }

		//! Pre-increment
		RECORDITERATOR& operator++ (
			) {
			CONST_RECORDITERATOR::operator++();
			return (*this);
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RECORDITERATOR (CONST_RECORDITERATOR &rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! OBSERVER class for DBTABLE objects.  The SUBJECT class is DBTABLE.
class DBTABLE::OBSERVER : public RVC::OBJECTOBSERVER {
	public:

		//! Types of table structural changes made, used in RVCDBTABLE::OBSERVER::OnStructChanged()
		enum STRUCTCHANGE {
			STRUCTCHANGE_None = 					0x00000000,
			STRUCTCHANGE_Name = 					0x00000001,
			STRUCTCHANGE_Desc = 					0x00000002,
			STRUCTCHANGE_ViewMode = 			0x00000004,
			STRUCTCHANGE_Field = 				0x00000008,			//!< Something in one or more fields changed 
			STRUCTCHANGE_FieldsAdded = 		0x00000010,
			STRUCTCHANGE_Flags = 				0x00000020,
			STRUCTCHANGE_AttachmentType = 	0x00000040,
			STRUCTCHANGE_Encoding = 			0x00000080,
			STRUCTCHANGE_DBEditorUpdate = 	0x00000100,
			STRUCTCHANGE_DBEditorRebuild = 	0x00000200
			};	

		explicit OBSERVER (
			DBTABLE& dbtable
			) : OBJECTOBSERVER(dbtable) {}
			
		virtual ~OBSERVER (
			) {}
			
		//! Called before a multi-record change
		virtual void OnBulkModifyBegin (
			) { return; }
			
		//! Called after a multi-record change
		virtual void OnBulkModifyEnd (
			) { return; }
			
		//! Called when a record is added to the table
		virtual void OnRecordAdd (
			INT32											//!< RecordNum
			) { return; }
			
		//! Called when a record is about to be altered
		virtual void OnRecordChangeBegin (
			INT32											//!< RecordNum
			) { return; }
			
		//! Called when a record has been altered
		virtual void OnRecordChangeEnd (
			INT32											//!< RecordNum
			) { return; }
			
		//! Called when a record is about to be deleted
		virtual void OnRecordDeleteBegin (
			INT32											//!< RecordNum
			) { return; }
			
		//! Called when a record has been deleted
		virtual void OnRecordDeleteEnd (
			INT32											//!< RecordNum
			) { return; }
			
		//! Called when table structure information has changed
		virtual void OnStructChanged (
			STRUCTCHANGE								//!< SubReason
			) { return; }

		//! Called when the table becomes valid or invalid
		virtual void OnTableValid (
			bool											//!< WasInvalid
			) { return; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual void OnNotifyRaw (UINT32 reason, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBTABLE::INDEXSEARCH)
DEFINE_ENUM_OP_BITWISE(DBTABLE::ITERATERECORD)
DEFINE_ENUM_OP_BITWISE(DBTABLE::FINDFLAG)
DEFINE_ENUM_OP_BITWISE(DBTABLE::REFELEM)
DEFINE_ENUM_OP_BITWISE(DBTABLE::RECORD::FIELDFLAG)
DEFINE_ENUM_OP_BITWISE(DBTABLE::FIELDINFO::GETTYPEDISPLAY)
DEFINE_ENUM_OP_BITWISE(DBTABLE::CURSOR::FUNCTION)
DEFINE_ENUM_OP_BITWISE(DBTABLE::CURSOR::INDEXFLAG)
DEFINE_ENUM_OP_INCREMENT(DBTABLE::USAGE)
#endif

}		// End of RVC namespace

#endif 	// INC_RVC_DBTABLE_H
