/**
 * \file rvc/geotable.h 
 * \brief RVC::GEOTABLE class definition
 *
 * \if NODOC
 * $Id: geotable.h_v 1.3 2014/07/03 21:45:57 mju Exp $
 *
 * $Log: geotable.h_v $
 * Revision 1.3  2014/07/03 21:45:57  mju
 * Add calc nested class.
 *
 * Revision 1.2  2014/04/02 17:38:42  mju
 * Add reset.
 *
 * Revision 1.1  2014/03/28 21:40:21  mju
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_GEOTABLE_H
#define INC_RVC_GEOTABLE_H

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
#include <mi32/latlonfm.h>
#endif


namespace RVC {

//=====================================================================================================================

class GEOTABLE {
	public:

		//! Default constructor
		GEOTABLE ();

		//! Copy constructor
		GEOTABLE (
			const GEOTABLE& rhs
			);

		//! Destructor
		~GEOTABLE ();

		//! Assignment.
		GEOTABLE& operator= (
			const GEOTABLE& rhs
			);

		//! Setup to begin calculating points.
		ERRVALUE CalcBegin (
			DBTABLE& ObjTable						//!< Source table
			);

		//! End point calculation.
		void CalcEnd ();

		//! Calculate 2D point.
		ERRVALUE CalcPoint2D (
			const DBTABLE::RECORD& record,
			INT32 ElemNum,
			DPOINT2D& point
			) const;

		//! Calculate 3D point.
		ERRVALUE CalcPoint3D (
			const DBTABLE::RECORD& record,
			INT32 ElemNum,
			DPOINT3D& point
			) const;

		const SPATREF::COORDREFSYS& GetCRS () const
			{ return (m_CRS); }

		const DRECT3D& GetExtents () const
			{ return (m_Extents); }

		const MISTRING& GetFieldNameX () const
			{ return (m_FieldNameX); }

		const MISTRING& GetFieldNameY () const
			{ return (m_FieldNameY); }

		const MISTRING& GetFieldNameZ () const
			{ return (m_FieldNameZ); }

		DMSTYPE GetLatLonFormat () const
			{ return (m_LatLonFormat); }

		const MISTRING& GetScript () const
			{ return (m_Script); }

		UINT8 GetUnitCodeXY () const
			{ return (m_UnitCodeXY); }

		UINT8 GetUnitCodeZ () const
			{ return (m_UnitCodeZ); }

		void GetXml (
			XMLNODE *nodeParent
			) const;

		//! Reset all values to defaults.
		void Reset ();

		void SetCRS (
			const SPATREF::COORDREFSYS& crs
			) { m_CRS = crs; }

		void SetExtents (
			const DRECT3D& extents
			) { m_Extents = extents; }

		void SetFieldNameX (
			const MISTRING& FieldNameX
			) { m_FieldNameX = FieldNameX; }

		void SetFieldNameY (
			const MISTRING& FieldNameY
			) { m_FieldNameY = FieldNameY; }

		void SetFieldNameZ (
			const MISTRING& FieldNameZ
			) { m_FieldNameZ = FieldNameZ; }

		void SetFieldNumX (
			INT32 FieldNumX
			) { m_FieldNumX = FieldNumX; }

		void SetFieldNumY (
			INT32 FieldNumY
			) { m_FieldNumY = FieldNumY; }

		void SetFieldNumZ (
			INT32 FieldNumZ
			) { m_FieldNumZ = FieldNumZ; }

		void SetLatLonFormat (
			DMSTYPE LatLonFormat
			) { m_LatLonFormat = LatLonFormat; }

		void SetScript (
			const MISTRING& Script
			) { m_Script = Script; }

		void SetUnitCodeXY (
			UINT8 UnitCodeXY
			) { m_UnitCodeXY = UnitCodeXY; }

		void SetUnitCodeZ (
			UINT8 UnitCodeZ
			) { m_UnitCodeZ = UnitCodeZ; }

		void SetXml (
			const XMLNODE *nodeParent
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CALC;

		CALC *m_pCalc;
		MISTRING m_Script;
		MISTRING m_FieldNameX;
		MISTRING m_FieldNameY;
		MISTRING m_FieldNameZ;
		SPATREF::COORDREFSYS m_CRS;
		DRECT3D m_Extents;
		INT32 m_FieldNumX;
		INT32 m_FieldNumY;
		INT32 m_FieldNumZ;
		DMSTYPE m_LatLonFormat;
		UINT8 m_UnitCodeXY;
		UINT8 m_UnitCodeZ;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

}		// End namespace RVC

#endif 	// INC_RVC_GEOTABLE_H
