/**
 * \file rvc/imagemultiband.h
 * \brief RVC::IMAGEMULTIBAND definitions
 *
 * \if NODOC
 * $Id: imagemultiband.h_v 1.1 2015/02/16 17:37:31 mju Exp $
 *
 * $Log: imagemultiband.h_v $
 * Revision 1.1  2015/02/16 17:37:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_IMAGEMULTIBAND_H
#define	INC_RVC_IMAGEMULTIBAND_H

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Utility class to find set of coincident raster image bands in OBJITEMLIST.
class IMAGEMULTIBAND {
	public:

		enum FLAGS {
			FLAG_Default =				0,
			FLAG_Strict =				0x01,		//!< Return error if any non-suitable item found
			FLAG_AllowPalette =		0x02,		//!< Allow rasters with color palette subobject (implied by FLAG_Strict)
			FLAG_NeedSameCellType =	0x04,		//!< Require all rasters to have the same cell type
			};

		//! Find coincident image bands.
		//! This will scan the specified list, starting with the item referenced and continuing until a non-coincident/suitable
		//! item is found.
		//! @return Number of bands found > 0, 0 if first item is not suitable object, < 0 if error
		//! Note, return value can be 1, indicating first item is suitable but next item is not coincident or suitable
		static int FindCoincidentBands (
			const OBJITEMLIST &ObjItemList,							//!< List to scan
			const OBJITEMLIST::CONST_ITERATOR &itFirstItem,		//!< Iterator for first item in list to scan
			FLAGS flags = FLAG_Default
			);
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(IMAGEMULTIBAND::FLAGS)
#endif

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_IMAGEMULTIBAND_H
