/**
 * \file rvc/opimageautocorr.h
 * \brief RVC::OP::IMAGE_AUTOCORRELATE definitions.
 *
 * \if NODOC
 * $Id: opimageautocorr.h_v 1.7 2014/12/29 18:06:47 mju Exp $
 *
 * $Log: opimageautocorr.h_v $
 * Revision 1.7  2014/12/29 18:06:47  mju
 * Doxygen exclusion for deprecated items.
 *
 * Revision 1.6  2013/04/22 18:02:32  mju
 * Use trans2d_affine.
 *
 * Revision 1.5  2007/08/20 15:14:01  honerkamp
 * remove illegal qualifier
 *
 * Revision 1.4  2007/02/21 01:16:28  honerkamp
 * optional correlation parameter can be passed
 *
 * Revision 1.3  2006/11/13 21:37:05  mju
 * Use int32 consistently for parms.
 *
 * Revision 1.2  2006/11/13 21:03:43  mju
 * Forgot to make all methods static.
 *
 * Revision 1.1  2006/11/13 20:47:09  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPIMAGEAUTOCORR_H
#define	INC_RVC_OPIMAGEAUTOCORR_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_AFFINE;
namespace RVC {
	class RASTER;
	}
#endif

namespace RVC {
namespace OP {

class IMAGE_AUTOCORRELATE {
	public:

		static int AdjustPoint (
			RVC::RASTER& ObjRasterSrc,
			const DPOINT2D& PointSrc,
			const TRANS2D_AFFINE& trans,
			RVC::RASTER& ObjRasterTgt,
			DPOINT2D& PointTgt,
			double& RetCorrelation,
			INT32 SourceSize,			//!< size of the correlation window in input image
			INT32 TargetSize			//!< size of the searchable window in output image
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		DEPRECATED_MSG("Use TRANS2D_AFFINE variant") static int AdjustPoint (
			RVC::RASTER& ObjRasterSrc,
			const DPOINT2D& PointSrc,
			const MAT3X3 ItoO,
			const MAT3X3 OtoI,
			RVC::RASTER& ObjRasterTgt,
			DPOINT2D& PointTgt,
			double& RetCorrelation,
			INT32 SourceSize,			//!< size of the correlation window in input image
			INT32 TargetSize			//!< size of the searchable window in output image
			);
		#endif

		static int AdjustPointEpipolar (
			RVC::RASTER& ObjRasterLeft,
			RVC::RASTER& ObjRasterRight,
			const DPOINT2D& PointSrc,
			DPOINT2D& PointTgt,
			INT32 SourceSize,
			INT32 TargetWidth,
			INT32 VertParallax,
			double MinCorr,
			double MinSigma,
			double *RetCorrelation = NULL  //optional correlation of calculated point 
			);

		static int EvaluateDeviation (
			RVC::RASTER& ObjRaster,
			INT32 SourceSize,
			const DPOINT2D& pos,
			double& RetSigma
			);
	};

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPIMAGEAUTOCORR_H
