/**
 * \file rvc/opimagecontrastmatch.h
 * \brief RVC::OP::IMAGE_CONTRAST_MATCH definitions
 *
 * \if NODOC
 * $Id: opimagecontrastmatch.h_v 1.1 2014/11/06 21:55:14 mju Exp $
 *
 * $Log: opimagecontrastmatch.h_v $
 * Revision 1.1  2014/11/06 21:55:14  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPIMAGECONTRASTMATCH_H
#define	INC_RVC_OPIMAGECONTRASTMATCH_H

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
#include <mi32/smlbase.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MIJOB;
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================
//		IMAGE_CONTRAST_MATCH definition - Begin

//! Match image contrast to reference image.
class IMAGE_CONTRAST_MATCH {
	public:

		enum MATCHAREA {
			MATCHAREA_Overlap = 0,
			MATCHAREA_All,
			};

		//! Install SML interface.
		static void SML_Install (SML::CONTEXT *context);

		//! Constructor.
		IMAGE_CONTRAST_MATCH ();

		//! Destructor.
		~IMAGE_CONTRAST_MATCH ();

		//! Read all parameters from job file.
		ERRVALUE JobRead (MIJOB &job);

		//! Write parameters to job file.
		ERRVALUE JobWrite (MIJOB &job);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set matching area.
		//! Default is to match using overlap area.
		void SetMatchArea (
			MATCHAREA MatchArea
			) { m_MatchArea = MatchArea; }

		ERRVALUE SetObjInput (
			const RVC::OBJITEM &ObjItemInput
			) { m_ObjItemInput = ObjItemInput; return (0); }

		ERRVALUE SetObjOutput (
			const RVC::OBJITEM &ObjItemOutput
			) { m_ObjItemOutput = ObjItemOutput; return (0); }

		ERRVALUE SetObjReference (
			const RVC::OBJITEM &ObjItemReference
			) { m_ObjItemReference = ObjItemReference; return (0); }

		//! Set whether to copy contrast tables.
		//! Default is to not copy.
		void SetCopyContrastTables (
			bool bCopy
			) { m_bCopyContrastTables = bCopy; }

		//! Set output pyramiding.
		//! Default is to average.
		void SetPyramid (
			RVC::IMAGE::PYRAMID PyramidMode
			) { m_PyramidMode = PyramidMode; }

		//! Set output compression.
		//! Default is uncompressed.
		void SetCompression (
			RVC::RASTER::COMPTYPE CompressionType,
			UINT16 QualityRatio
			) { m_CompressionType = CompressionType; m_CompressionQualityRatio = QualityRatio; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		RVC::OBJITEM m_ObjItemInput;
		RVC::OBJITEM m_ObjItemReference;
		RVC::OBJITEM m_ObjItemOutput;
		MATCHAREA m_MatchArea;
		RVC::IMAGE::PYRAMID m_PyramidMode;
		RVC::RASTER::COMPTYPE m_CompressionType;
		UINT16 m_CompressionQualityRatio;
		bool m_bCopyContrastTables;

		static void SML_ContrastMatchReference (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);

		ERRVALUE CheckSettings ();
		ERRVALUE RunSML_ContrastMatchReference (SML::CONTEXT *context, SML::ARG *args, int numargs);

		IMAGE_CONTRAST_MATCH (const IMAGE_CONTRAST_MATCH&);
		IMAGE_CONTRAST_MATCH& operator= (const IMAGE_CONTRAST_MATCH&);
		#endif
	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPIMAGECONTRASTMATCH_H
