/**
 * \file rvc/uidlgimagehistogram.h
 * \brief RVC::UI::DlgImageHistogram functions
 *
 * \if NODOC
 * $Id: uidlgimagehistogram.h_v 1.3 2014/05/14 13:34:00 mju Exp $
 *
 * $Log: uidlgimagehistogram.h_v $
 * Revision 1.3  2014/05/14 13:34:00  mju
 * Make dialog class public, replacing global functions.
 *
 * Revision 1.2  2013/08/01 16:22:37  mju
 * Add open fn for multi-raster.
 *
 * Revision 1.1  2013/06/28 21:22:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIDLGIMAGEHISTOGRAM_H
#define	INC_RVC_UIDLGIMAGEHISTOGRAM_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_FORMTABBED_H
#include <mgui/formtabbed.h>
#endif

#ifndef	INC_MGUI_GRAPHHISTOGRAM_H
#include <mgui/graphhistogram.h>
#endif

#ifndef	INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_RVC_UIFILTERGETOBJECTIMAGE_H
#include <rvc/uifiltergetobjectimage.h>
#endif

#ifndef	INC_RVC_UIFORMOBJECTSINGLE_H
#include <rvc/uiformobjectsingle.h>
#endif

#ifndef	INC_MI32_STATUNIVAR_H
#include <mi32/statunivar.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define INC_VECTOR
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class REGION2D;
namespace RVC {
	class OBJITEM;
	class HISTOGRAM;
	}
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! Dialog to display image histograms.
class DLG_IMAGE_HISTOGRAM : public MGUI::DLGSHELL {
	public:

		//! Set region for all open dialogs.
		static ERRVALUE AllSetRegion (
			const REGION2D& region
			);

		//! Create and open dialog with specified single image.
		//! @return Pointer to dialog, will be automatically set to destroy self on close.
		static DLG_IMAGE_HISTOGRAM* CreateAndOpen (
			MDLGPARENT dlgparent,
			const RVC::OBJITEM& objitem
			);

		//! Determine if any histogram dialogs are open.
		static bool HasAnyOpen ();

		//! Default constructor.
		DLG_IMAGE_HISTOGRAM ();

		//! Destructor.
		virtual ~DLG_IMAGE_HISTOGRAM ();

		//! Clear region.
		void ClearRegion ();

		//! Set single object.
		void SetObject (
			const RVC::OBJITEM& objitem
			);

		//! Set region.
		ERRVALUE SetRegion (
			const REGION2D& region
			);

		//! Setup dialog to display set of objects corresponding to color model.
		ERRVALUE SetupMulti (
			COLOR::MODEL ColorModel,
			const MILIST<const RVC::OBJITEM*>& ObjItemPtrList
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_IMAGE_HISTOGRAM THISCLASS;

		enum AREAMODE {
			AREAMODE_Auto,
			AREAMODE_Both,
			AREAMODE_Full,
			AREAMODE_Region
			};

		enum CMDID {
			CMDID_Scale_Log = 1000,
			CMDID_Count_Cumulative,
			CMDID_ComputeFull,
			CMDID_SaveAsImage,
			CMDID_SaveAsText,
			CMDID_Area_Full,
			CMDID_Area_Region,
			CMDID_Area_Both,
			CMDID_Help,
			CMDID_BASE_Stats =	0x1000,
			};

		enum COMPONENTID {
			COMPONENTID_All = -1,
			COMPONENTID_Red = 0,
			COMPONENTID_Green,
			COMPONENTID_Blue,
			COMPONENTID_Real,
			COMPONENTID_Imaginary,
			COMPONENTID_Magnitude,
			COMPONENTID_Phase,
			COMPONENTID_Cyan,
			COMPONENTID_Magenta,
			COMPONENTID_Yellow,
			COMPONENTID_Black,
			COMPONENTID_Alpha,
			COMPONENTID_Hue,
			COMPONENTID_Saturation,
			COMPONENTID_Intensity,
			COMPONENTID_Brightness,
			COMPONENTID_Value,
			COMPONENTID_COUNT
			};

		class STATITEM {
			public:
				STATITEM ();
				~STATITEM ();
				const COLOR& GetColorGraph () const
					{ return (m_colorGraph); }
				const COLOR& GetColorLabel () const
					{ return (m_colorLabel); }
				COMPONENTID GetComponentID () const
					{ return (m_ComponentID); }
				const TEXTID GetLabel () const
					{ return (GetComponentLabel(m_ComponentID)); }
				RVC::RASTER::COMPONENT GetRasterComponent () const
					{ return (m_RasterComponent); }
				UINT16 GetSampleIntervalColumns () const
					{ return (m_SampleColumns); }
				UINT16 GetSampleIntervalRows () const
					{ return (m_SampleRows); }
				const STAT_UNIVARIATE& GetStats (bool bRegion) const
					{ return (bRegion ? m_StatsRegion : m_StatsFull); }
				const STAT_UNIVARIATE& GetStatsFull () const
					{ return (m_StatsFull); }
				const STAT_UNIVARIATE& GetStatsRegion () const
					{ return (m_StatsRegion); }
				bool IsSampled () const
					{ return (m_SampleColumns > 1 || m_SampleRows > 1); }
				void SetComponentID (COMPONENTID id);
				void SetRasterComponent (RVC::RASTER::COMPONENT component);
				ERRVALUE SetStats (const RVC::HISTOGRAM& ObjHistogram, bool bRegion);
			private:
				STAT_UNIVARIATE m_StatsFull;
				STAT_UNIVARIATE m_StatsRegion;
				COLOR m_colorGraph;
				COLOR m_colorLabel;
				COMPONENTID m_ComponentID;
				RVC::RASTER::COMPONENT m_RasterComponent;
				UINT16 m_SampleColumns;
				UINT16 m_SampleRows;
			};

		RVC::UI::FILTER_GETOBJECT_IMAGE m_FilterImage;
		RVC::UI::FORM_OBJECT_SINGLE m_FormObjSingle;
		MGUI::FORM_EDIT_STRING m_TextObjMulti;
		MGUI::CTRL_TOOLBAR m_Toolbar;
		MGUI::MENU m_MenuSettings;
		MGUI::MENU::ITEMHND m_hndSepComponent;
		MGUI::MENU::ITEMHND m_hndLblComponent;
		MGUI::FORM_TABBED m_FormTabbed;
		MGUI::GRAPH_HISTOGRAM m_Graph;
		MGUI::FORM_LIST m_ListStats;
		MGUI::CTRL_TEXT m_TextProperties;
		std::vector<RVC::OBJITEM> m_ObjItemsMulti;
		std::vector<STATITEM> m_StatItems;
		REGION2D m_Region;
		int m_StatsRowMinimum;
		int m_StatsRowMaximum;
		int m_StatsRowMean;
		int m_StatsRowStdDev;
		int m_StatsRowMedian;
		int m_StatsRowMode;
		int m_StatsRowMost;
		int m_StatsRowCount;
		int m_StatsRowNulls;
		int m_StatsRowBinInterval;
		int m_FieldIdxFull[4];
		int m_FieldIdxRegion[4];
		int m_ComponentIdx;
		UINT32 m_GraphNumComponents;
		AREAMODE m_GraphAreaMode;
		bool m_bHasStatsRegion;
		bool m_bIsFloat;

		static TEXTID GetComponentLabel (COMPONENTID id);

		ERRVALUE CalcImageRegion (RVC::IMAGE& ObjImage, REGION2D& RgnImage) const;
		ERRVALUE ComputeFullHistogram ();
		bool GraphIdxIsRegion (UINT32 idx) const;
		const STATITEM& GraphIdxStatItem (UINT32 idx) const;
		COLOR OnGraphGetColor (UINT32 idx) const;
		COLOR OnGraphGetColorLabel (UINT32 idx) const;
		MISTRING OnGraphGetLabel (UINT32 idx) const;
		const STAT_UNIVARIATE& OnGraphGetStats (UINT32 idx) const;
		bool OnListStatsDrawItem (MGUI::FORM_LIST::CUSTOMDRAW& cd) const;
		bool OnListStatsToolTip (int itemnum, int fieldnum, MGUI::CTRL_TOOLTIP& tooltip) const;
		void OnMenuComponent (MGUI::ID cmdid, bool bToggleOn);
		void OnMenuSettings (MGUI::ID cmdid, bool bToggleOn);
		void OnMenuStats (MGUI::ID cmdid, bool bToggleOn);
		void OnObjSelected ();
		ERRVALUE SaveAsText ();
		ERRVALUE SetupNewObject ();
		void Update ();
		void UpdateFieldStats (UINT32 c, bool bRegion);
		void UpdateListStats ();
		void UpdateMenuAreaMode ();
		void UpdateMenuStats ();

		ERRVALUE v_CreateContent ();
		void v_OnOpen ();
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIDLGIMAGEHISTOGRAM_H
