/**
 * \file rvc/uifiltergetobjectdbtable.h
 * \brief RVC::UI::FILTER_GETOBJECT_DBTABLE definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectdbtable.h_v 1.2 2013/05/09 21:46:06 mju Exp $
 *
 * $Log: uifiltergetobjectdbtable.h_v $
 * Revision 1.2  2013/05/09 21:46:06  mju
 * Add filter_getobject_dbtable_style.
 *
 * Revision 1.1  2013/05/09 20:42:51  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTDBTABLE_H
#define	INC_RVC_UIFILTERGETOBJECTDBTABLE_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting database table objects.
class FILTER_GETOBJECT_DBTABLE : public MGUI::DLG_GETOBJECT_FILTER {
	public:

		//! Default constructor, allows OBJTYPE_Table.
		FILTER_GETOBJECT_DBTABLE ();

		//! Construct with specific set of allowed object types.
		//! If any non-dbtable types are specified they will always be included by this class.
		FILTER_GETOBJECT_DBTABLE (
			const RVC::OBJTYPESET& ObjTypeSetSelectable		//!< Set of allowed object types
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_DBTABLE ();

		//! Set whether to allow "hidden" tables.
		//! By default, hidden tables are not selectable.
		void SetAllowHidden (
			bool bAllowHidden = true
			) { m_bAllowHidden = bAllowHidden; }

		//! Set whether to allow "internal" tables.
		//! By default, internal tables are not selectable.
		void SetAllowInternal (
			bool bAllowInternal = true
			) { m_bAllowInternal = bAllowInternal; }

		//! Set whether table needs to be non-empty.
		//! By default, empty tables are selectable.
		void SetNeedNonEmpty (
			bool bNeedNonEmpty = true
			) { m_bNeedNonEmpty = bNeedNonEmpty; }

		//! Set needed "usages".
		//! By default, all usages are allowed.
		void SetNeedUsage (
			const DBTABLE::USAGESET& UsageSet
			) { m_UsageSet = UsageSet; }

		//! Set whether table needs to be "user-defined" (not built-in).
		void SetNeedUserDefined (
			bool bNeedUserDefined = true
			) { m_bNeedUserDefined = bNeedUserDefined; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef MGUI::DLG_GETOBJECT_FILTER BASECLASS;
		typedef FILTER_GETOBJECT_DBTABLE THISCLASS;

		DBTABLE::USAGESET m_UsageSet;
		bool m_bAllowHidden;
		bool m_bAllowInternal;
		bool m_bNeedUserDefined;
		bool m_bNeedNonEmpty;

		int IsSelectableDbTable (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================

//! Filter for selecting database style table objects.
//! Will only allow selection if a "style" object exists under the table.
class FILTER_GETOBJECT_DBTABLE_STYLE : public FILTER_GETOBJECT_DBTABLE {
	public:

		//! Default constructor, allows OBJTYPE_Table.
		FILTER_GETOBJECT_DBTABLE_STYLE ();

		//! Destructor.
		virtual ~FILTER_GETOBJECT_DBTABLE_STYLE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FILTER_GETOBJECT_DBTABLE BASECLASS;
		typedef FILTER_GETOBJECT_DBTABLE_STYLE THISCLASS;

		int IsSelectableDbTableStyle (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================
}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTDBTABLE_H
