/**
 * \file rvc/uifiltergetobjectgeometric.h
 * \brief RVC::UI::FILTER_GETOBJECT_GEOMETRIC definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectgeometric.h_v 1.4 2013/08/13 22:00:49 mju Exp $
 *
 * $Log: uifiltergetobjectgeometric.h_v $
 * Revision 1.4  2013/08/13 22:00:49  mju
 * Add elemtype parm to ctors.
 *
 * Revision 1.3  2013/05/08 16:58:57  mju
 * Add dox details.
 *
 * Revision 1.1  2013/05/07 21:29:38  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTGEOMETRIC_H
#define	INC_RVC_UIFILTERGETOBJECTGEOMETRIC_H

#ifndef	INC_RVC_UIFILTERGETOBJECTGEOSPATIAL_H
#include <rvc/uifiltergetobjectgeospatial.h>
#endif

#ifndef	INC_RVC_GEOMETRIC_H
#include <rvc/geometrc.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting "geometric" objects.
//! By default, the object must contain at least one non-label element. This can be controlled using
//! SetAllowEmpty or SetAllowIfOnlyLabels. Specific required element type(s) may also be set using
//! SetNeedElemTypes.
class FILTER_GETOBJECT_GEOMETRIC : public FILTER_GETOBJECT_GEOSPATIAL {
	public:

		//! Default constructor, allows all "geometric" object types.
		FILTER_GETOBJECT_GEOMETRIC (
			ELEMTYPE NeedElemType = ELEMTYPE_Empty				//!< Specific element type needed if any
			);

		//! Construct with specific set of allowed object types.
		//! If any non-geometric types are specified they will always be included by this class.
		explicit FILTER_GETOBJECT_GEOMETRIC (
			const RVC::OBJTYPESET& ObjTypeSetSelectable,		//!< Set of allowed object types
			ELEMTYPE NeedElemType = ELEMTYPE_Empty				//!< Specific element type needed if any
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_GEOMETRIC ();

		//! Set to allow "empty" geometric object (has no elements).
		//! By default, empty objects are not allowed.
		void SetAllowEmpty (
			bool AllowEmpty = true
			) { m_bAllowEmpty = AllowEmpty; }

		//! Set to allow geometric object with only "label" elements.
		//! By default, object must have at least one non-label element.
		void SetAllowIfOnlyLabels (
			bool AllowIfOnlyLabels = true
			) { m_bAllowIfOnlyLabels = AllowIfOnlyLabels; }

		//! Set which element type(s) are needed and minimum number.
		void SetNeedElemTypes (
			const ELEMTYPESET& ElemTypeSet,
			INT32 MinCount = 1
			) { m_ElemTypeSet = ElemTypeSet; m_MinElements = MinCount; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FILTER_GETOBJECT_GEOSPATIAL BASECLASS;
		typedef FILTER_GETOBJECT_GEOMETRIC THISCLASS;

		ELEMTYPESET m_ElemTypeSet;
		INT32 m_MinElements;
		bool m_bAllowEmpty;
		bool m_bAllowIfOnlyLabels;

		int IsGeometricSelectable (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTGEOMETRIC_H
