/**
 * \file rvc/uiformdbfieldpicklist.h
 * \brief RVC::UI::FORM_DB_FIELD_PICKLIST definition.
 *
 * \if NODOC
 * $Id: uiformdbfieldpicklist.h_v 1.10 2014/11/26 21:33:05 mju Exp $
 *
 * $Log: uiformdbfieldpicklist.h_v $
 * Revision 1.10  2014/11/26 21:33:05  mju
 * Add formstyle_Automatic and other styles for CSV lists.
 * Move private part to implementation file.
 *
 * Revision 1.9  2011/12/27 16:50:06  mju
 * Remove redundant hdr.
 *
 * Revision 1.8  2010/02/22 18:52:24  mju
 * Specify table/field FOR when create.
 *
 * Revision 1.6  2010/02/19 22:40:49  mju
 * Add dlgEditSettings.
 *
 * Revision 1.3  2010/02/12 22:47:53  mju
 * Redesign to use tree so can support hierarchical.
 *
 * Revision 1.1  2009/08/21 13:32:53  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMDBFIELDPICKLIST_H
#define	INC_RVC_UIFORMDBFIELDPICKLIST_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Form for picking database field value from list.
class FORM_DB_FIELD_PICKLIST : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Automatic =				0x0000,	//!< Automatically determine style based on DBFIELDCONTROL values for field
			FORMSTYLE_SelectSingle =			0x0001,	//!< Single selection from tree/list
			FORMSTYLE_SelectMulti =				0x0002,	//!< Allow selection of multiple items
			FORMSTYLE_ComboBox =					0x0004,	//!< Use combobox for selection
			FORMSTYLE_SelectCSV_AllowDup =	0x0008,	//!< Selection returned in CSV list with duplicates allowed
			FORMSTYLE_SelectCSV_Unique =		0x0010,	//!< Selection returned in CSV list with no duplicates
			FORMSTYLE_AllowSelectNone =		0x0100,	//!< User is allowed choice of <none> even if "not null" set on field
			FORMSTYLE_NoSelectNone =			0x0200,	//!< Don't include choice of <none> even if allowed for field
			FORMSTYLE_UseGroupBox =				0x0400,	//!< Place form in groupbox even if no label
			FORMSTYLE_NoEditSettings =			0x0800,	//!< Don't include Settings choice on popup menu
			FORMSTYLE_MASK_SelectCSV =			FORMSTYLE_SelectCSV_AllowDup | FORMSTYLE_SelectCSV_Unique,
			FORMSTYLE_MASK_Select =				FORMSTYLE_SelectSingle | FORMSTYLE_SelectMulti | FORMSTYLE_ComboBox | FORMSTYLE_MASK_SelectCSV
			};

		class SETTINGS {
			public:
				static const SERIALIZERITEM* SerialGetItemDef ();

				SETTINGS ();
				~SETTINGS ();

				INT32 GetFieldNumDisplay (
					) const { return (m_FieldNumDisplay); }
				INT32 GetFieldNumGroupBy (
					) const { return (m_FieldNumGroupBy); }
				INT32 GetFieldNumSortBy (
					) const { return (m_FieldNumSortBy); }
				const LPOINT2D& GetGridSize (
					) const { return (m_GridSize); }
				bool GetStyleSampleShow (
					) const { return (m_StyleSampleShow); }
				INT32 GetStyleSampleSize (
					) const { return (m_StyleSampleSize); }
				bool IsChanged (
					) const { return (m_IsChanged); }
				bool IsEqual (
					const SETTINGS& rhs
					) const;
				void ResetChanged (
					) { m_IsChanged = false; }
				void SetFieldNumDisplay (
					INT32 FieldNumDisplay
					) { if (FieldNumDisplay != m_FieldNumDisplay) {m_IsChanged = true; m_FieldNumDisplay = FieldNumDisplay; } }
				void SetFieldNumGroupBy (
					INT32 FieldNumGroupBy		//!< Field number, -1 for automatic, -2 for none
					) { if (FieldNumGroupBy != m_FieldNumGroupBy) {m_IsChanged = true; m_FieldNumGroupBy = FieldNumGroupBy; } }
				void SetFieldNumSortBy (
					INT32 FieldNumSortBy
					) { if (FieldNumSortBy != m_FieldNumSortBy) {m_IsChanged = true; m_FieldNumSortBy = FieldNumSortBy; } }
				void SetGridSize (
					INT32 x,
					INT32 y
					) { if (x != m_GridSize.x || y != m_GridSize.y) { m_IsChanged = true; m_GridSize.x = x; m_GridSize.y = y; } }
				void SetGridSize (
					const LPOINT2D& GridSize
					) { if (GridSize != m_GridSize) { m_IsChanged = true; m_GridSize = GridSize; } }
				void SetStyleSampleShow (
					bool StyleSampleShow
					) { if (StyleSampleShow != m_StyleSampleShow) { m_IsChanged = true; m_StyleSampleShow = StyleSampleShow; } }
				void SetStyleSampleSize (
					INT32 StyleSampleSize
					) { if (StyleSampleSize != m_StyleSampleSize) { m_IsChanged = true; m_StyleSampleSize = StyleSampleSize; } }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef SETTINGS THISCLASS;
				bool m_IsChanged;
				bool m_StyleSampleShow;
				INT32 m_StyleSampleSize;
				INT32 m_FieldNumDisplay;
				INT32 m_FieldNumSortBy;
				INT32 m_FieldNumGroupBy;
				LPOINT2D m_GridSize;
				#endif
			};

		//! Dialog to change picklist form settings.
		//! @return TRUE if changed, FALSE if not, < 0 if error.
		static int DlgEditSettings (
			MDLGPARENT dlgparent,
			const RVC::DBTABLE& TableFor,
			INT32 FieldNumFor,
			FORM_DB_FIELD_PICKLIST::SETTINGS& settings,		//!< Settings passed/returned
			bool ForGrouping = false
			);

		//! Constructor.
		FORM_DB_FIELD_PICKLIST ();

		//! Destructor.
		virtual ~FORM_DB_FIELD_PICKLIST ();

		//! Create with default settings.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MISTRING& label,
			const RVC::DBTABLE& TableFor,			//!< Table picklist is FOR
			INT32 FieldNumFor,						//!< Field picklist is FOR
			FORMSTYLE FormStyle = FORMSTYLE_Automatic,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create with specified settings.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MISTRING& label,
			const RVC::DBTABLE& TableFor,			//!< Table picklist is FOR
			INT32 FieldNumFor,						//!< Field picklist is FOR
			const SETTINGS& settings,
			FORMSTYLE FormStyle = FORMSTYLE_Automatic,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get current settings.
		const SETTINGS& GetSettings () const
			{ return (m_Settings); }

		//! Get table picking from.
		const RVC::DBTABLE& GetTablePickFrom () const;

		//! Get value for selected item in single-selection mode, if any.
		ERRVALUE GetValueSelected (
			RVC::DBTABLE::RECORD& RecordRet,				//!< Record to place value into
			INT32 FieldNumRet									//!< Field in RecordRet to contain value
			) const;

		//! Determine if has any item selected.
		bool HasSelection () const;

		//! Determine if value from specified record is selected.
		bool IsValueSelected (
			const RVC::DBTABLE::RECORD& RecordTest,	//!< Record containing value to test
			INT32 FieldNumTest								//!< Field in RecordTest containing value to test
			) const;

		//! Set delegate for when selection changed.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set value selected.
		ERRVALUE SetValueSelected (
			const RVC::DBTABLE::RECORD& RecordSet,		//!< Record to get value to set from
			INT32 FieldNumSet,								//!< Field in RecordSet to set from
			bool notify = true								//!< Notify via delegate if changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;

		PRIV *m_pPriv;
		SETTINGS m_Settings;
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_DB_FIELD_PICKLIST::FORMSTYLE)
#endif

inline bool operator== (
	const FORM_DB_FIELD_PICKLIST::SETTINGS& lhs,
	const FORM_DB_FIELD_PICKLIST::SETTINGS& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const FORM_DB_FIELD_PICKLIST::SETTINGS& lhs,
	const FORM_DB_FIELD_PICKLIST::SETTINGS& rhs
	) { return (!lhs.IsEqual(rhs)); }


//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFORMDBFIELDPICKLIST_H
