/**
 * \file rvc/uiformfieldelementid.h
 * \brief RVC::UI::FORM_FIELD_ELEMENTID definition
 *
 * \if NODOC
 * $Id: uiformfieldelementid.h_v 1.3 2015/05/04 19:51:19 mju Exp $
 *
 * $Log: uiformfieldelementid.h_v $
 * Revision 1.3  2015/05/04 19:51:19  mju
 * Add style enum and parm to create.
 *
 * Revision 1.2  2015/04/13 19:00:52  mju
 * Add accessLabel.
 *
 * Revision 1.1  2015/04/13 17:58:24  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFORMFIELDELEMENTID_H
#define	INC_RVC_UIFORMFIELDELEMENTID_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTUPT
namespace RVC {
	class OBJITEM;
	class DBASE_ELEMENT;
	}
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! Form to display combobox for choosing field to use for "element ID".
class FORM_FIELD_ELEMENTID : MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =		0,
			STYLE_NoNone =			0x01,		//!< Disallow "None" choice, should not combine with STYLE_NoElemNum
			STYLE_NoElemNum =		0x02,		//!< Disallow "Element Number" choice, should not combine with STYLE_NoNone
			};

		//! Constructor.
		FORM_FIELD_ELEMENTID ();

		//! Destructor.
		virtual ~FORM_FIELD_ELEMENTID ();

		//! Access label control, typically for alignment.
		MGUI::CTRL_LABEL& AccessLabel ()
			{ return (m_Cbx.GetLabel()); }

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE &paneParent,
			const MISTRING &label,
			STYLE style = STYLE_Default,				//!< Form style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get name of selection if table/field.
		//! @return Table.Field in string or empty string if selection is not a field.
		MISTRING GetSelectionName () const;

		//! Get name of selected table and field.
		//! @return if selection is table/field, false if not.
		bool GetSelectedTableField (
			MISTRING &TableName,			//!< Table name returned, empty if selection is not a field
			MISTRING &FieldName			//!< Field name returned, empty if selection is not a field
			) const;

		//! Determine if selection is "element number".
		bool IsSelectionElemNum () const
			{ return (m_Cbx.GetSelectedItemID() == ELEMENTID_ElemNum); }

		//! Determine if selection is "none".
		bool IsSelectionNone () const
			{ return (m_Cbx.GetSelectedItemID() == ELEMENTID_None); }

		//! Set delegate for selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnSelection = delegate; }

		//! Set associated geometric object and element type.
		void SetObjGeometric (
			const RVC::OBJITEM &objitem,	//!< Object location
			ELEMTYPE ElemType,				//!< Element type
			bool notify = true				//!< Whether to notify delegate if selection changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_FIELD_ELEMENTID THISCLASS;

		enum ELEMENTID {
			ELEMENTID_None = 0,
			ELEMENTID_ElemNum = -2,
			ELEMENTID_Choose = -3
			};

		MGUI::FORM_COMBOBOX m_Cbx;
		MISTRINGLIST m_FieldsUsed;
		DELEGATE_VOID_NOPARMS m_DelegateOnSelection;
		DBASE_ELEMENT *m_pObjDbGeometric;
		STYLE m_style;

		void OnCbx ();
		ERRVALUE PromptField ();
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_FIELD_ELEMENTID::STYLE)
#endif

//=====================================================================================================================

} // End namespace UI
} // End namespace RVC

#endif	// INC_RVC_UIFORMFIELDELEMENTID_H
