/**
 * \file rvc/uiformobjectsingle.h
 * \brief RVC::UI::FORM_OBJECT_SINGLE class definitions
 *
 * \if NODOC
 * $Id: uiformobjectsingle.h_v 1.10 2013/05/09 15:27:46 mju Exp $
 *
 * $Log: uiformobjectsingle.h_v $
 * Revision 1.10  2013/05/09 15:27:46  mju
 * Don't use const for filter pointer.
 *
 * Revision 1.9  2013/05/09 15:14:29  mju
 * In setFilter use pointer and take ownership.
 *
 * Revision 1.8  2013/05/09 15:07:24  mju
 * Add create method with pointer to filter to take ownership.
 *
 * Revision 1.7  2013/05/09 14:52:15  mju
 * Support mgui::dlg_getobject_filter and deprecate use of rvc::getobjfilter.
 *
 * Revision 1.6  2012/10/18 12:54:55  mju
 * Change setObjTypeSet to setFilters.
 *
 * Revision 1.1  2006/07/21 20:50:19  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMOBJECTSINGLE_H
#define	INC_RVC_UIFORMOBJECTSINGLE_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
#include <mgui/dlggetobjectstyle.h>
#endif

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class GETOBJFILTER;
	}
#endif

namespace RVC {
namespace UI {
	
//===================================================================================================================

//! Form to manage single object selection, containing button and text.
class FORM_OBJECT_SINGLE : public MGUI::FORM_BUTTONTEXT {
	public:
	
		//! Constructor.
		FORM_OBJECT_SINGLE ();

		//! Destructor.
		virtual ~FORM_OBJECT_SINGLE ();
			
		//! Clears the selection.
		void Clear (
			bool notify							//!< Notify 
			);

		//! Create form with filter reference.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& prompt,								//!< Prompt string to use with DlgGetObjects
			const MGUI::DLG_GETOBJECT_FILTER& filter,		//!< Object selection filter reference (MUST REMAIN VALID)
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			int width = 0,											//!< Width in "em" characters, 0 for default
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create form and take ownership of filter.
		//! The form will take ownership of the specified filter pointer and the value will be returned as NULL.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& prompt,								//!< Prompt string to use with DlgGetObjects
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,			//!< Object selection filter pointer (must not be NULL)
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			int width = 0,											//!< Width in "em" characters, 0 for default
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create form with legacy filter.
		DEPRECATED void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			int width,												//!< Width in "em" characters
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER *pGetObjFilter = 0,			//!< Filter for DlgGetObjects
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create form with legacy filter and default width.
		DEPRECATED void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,								//!< Label for button
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER *pGetObjFilter = 0,			//!< Filter for DlgGetObjects
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get selected object.
		const RVC::OBJITEM& GetObjItem () const
			{ return (m_ObjItem); }
			
		//! Select object via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify							//!< Send notification to delegate after selection
			);
			
		//! Set delegate to call when change to selected object occurs.
		void SetDelegateOnChangeSelected (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSelected = delegate; }

		//! Set new filter.
		//! Form will take ownership of new filter and value will be returned as NULL.
		//! If the currently selected object is not allowed by the new filter, the additional parameters affect whether
		//! the user will be prompted for a new selection and/or whether to call the delegate to notify.  If the
		//! user cancels the selection (or is not prompted) the existing selection will be cleared.
		//! If no object is currently selected, the user will not be prompted and the delegate will not be called.
		void SetFilter (
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,	//!< Pointer to filter, will be returned as NULL
			bool SelectIfCurrentNotAllowed,			//!< Reselect if current selection no longer allowed
			bool NotifyIfChangeSelected				//!< Notify if selection changed
			);

		//! Set object.
		void SetObjItem (
			const RVC::OBJITEM& ObjItem,	//!< Object location to set
			bool notify							//!< Send notification to delegate after selection
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::OBJITEM m_ObjItem;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSelected;
		MISTRING m_PromptStr;
		const MGUI::DLG_GETOBJECT_FILTER *m_pFilter;
		MGUI::DLG_GETOBJECT_STYLE m_DlgStyle;
		bool m_bOwnFilter;

		void OnBtnSelect ();

		FORM_OBJECT_SINGLE (const FORM_OBJECT_SINGLE&);
		FORM_OBJECT_SINGLE& operator= (const FORM_OBJECT_SINGLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMOBJECTSINGLE_H
