/**
 * \file rvc/uiformselectelements.h
 * \brief RVC::FORM_SELECT_ELEMENTS_... class definitions.
 *
 * \if NODOC
 * $Id: uiformselectelements.h_v 1.13 2014/12/12 16:05:14 mju Exp $
 *
 * $Log: uiformselectelements.h_v $
 * Revision 1.13  2014/12/12 16:05:14  mju
 * Add v_onUpdateCurSelectParmItem.
 *
 * Revision 1.12  2014/09/15 13:13:02  mju
 * Change onObjectRemoved parms.
 *
 * Revision 1.11  2014/09/11 20:29:25  mju
 * Add ability to enable/disable "apply to all" button.
 *
 * Revision 1.10  2014/05/30 17:59:07  mju
 * Add support for applying selection to all.
 *
 * Revision 1.9  2013/05/02 21:53:33  mju
 * Support options for separation by polygon and simplify separation controls.
 *
 * Revision 1.8  2013/01/15 15:52:03  mju
 * Remove unused region member.
 *
 * Revision 1.7  2012/07/31 17:46:18  mju
 * Add toggle to control whether to clear list after run.
 *
 * Revision 1.6  2012/07/27 20:48:30  mju
 * Adjust layout for separation.
 *
 * Revision 1.5  2012/07/26 21:27:10  mju
 * Add controls for separation.
 *
 * Revision 1.4  2012/07/25 21:40:01  mju
 * Begin update to allow separation, integrate region selection with general form.
 *
 * Revision 1.3  2011/12/28 14:02:55  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.2  2011/10/21 20:26:18  mju
 * Remove redundant hdrs.
 *
 * Revision 1.1  2011/09/01 15:38:27  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMSELECTELEMENTS_H
#define	INC_RVC_UIFORMSELECTELEMENTS_H

#ifndef	INC_RVC_UIFORMOBJECTLIST_H
#include <rvc/uiformobjectlist.h>
#endif

#ifndef	INC_RVC_GEOMETRC_H
#include <rvc/geometrc.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef	INC_MGUI_FORMBOOK_H
#include <mgui/formbook.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class ELEMTYPESET;
#endif


namespace RVC {
namespace UI {
	
//=====================================================================================================================

//! Form to select elements of single type.
//! Consists of combobox to choose selection method and "Select" pushbutton.
class FORM_SELECT_ELEMENTS_SINGLETYPE : public MGUI::FORM_COMPOSITE {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_SINGLETYPE ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_SINGLETYPE ();
	
		void SetGeometricObject (
			const RVC::OBJITEM &ObjItem
			);
			
	protected:

		ERRVALUE CreateBase (MGUI::LAYOUT_PANE_BASE &ParentPane, bool AllowNone = true);
		TEXTID GetTextID (ELEMTYPE ElemType);
		void SetElemSelectParmsItemBase (RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM &item);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_ELEMENTS_SINGLETYPE THISCLASS;

		MGUI::CTRL_COMBOBOX m_CbxElemSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnSelect;
		RVC::OBJITEM m_ObjItem;
		
		void OnBtnSelect ();
		void OnCbxElemSelect ();
		void UpdateState ();

		virtual RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM& v_AccessCurSelectParmItem () = 0;
		virtual ELEMTYPE v_GetCurElemType () const = 0;
		virtual void v_OnUpdateCurSelectParmItem () = 0;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Form to select elements of specified fixed type.
class FORM_SELECT_ELEMENTS_FIXEDTYPE : public FORM_SELECT_ELEMENTS_SINGLETYPE {
	public:

		enum STYLE {
			STYLE_Default =				0,
			STYLE_AllowApplyToAll =		0x01,
			};
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_FIXEDTYPE ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_FIXEDTYPE ();

		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			ELEMTYPE ElemType,
			STYLE style,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		void EnableApplyToAll (
			bool bEnable
			) { m_bEnableApplyToAll = bEnable; EnableApplyToAll(); }

		//! Get selection parameter item.
		const RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM& GetElemSelectParmsItem () const
			{ return (m_SelectItem); }

		//! Get label for alignment.
		MGUI::CTRL_LABEL& GetLabel ()
			{ return (m_Label); }

		void SetDelegateApplyToAll (
			DELEGATE_VOID_ELEMTYPE delegate
			);
			
		void SetElemSelectParmsItem (
			const RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM &item
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_ELEMENTS_FIXEDTYPE THISCLASS;

		MGUI::CTRL_LABEL m_Label;
		MGUI::CTRL_PUSHBUTTON m_BtnApplyToAll;
		RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM m_SelectItem;
		DELEGATE_VOID_ELEMTYPE m_DelegateApplyToAll;
		ELEMTYPE m_ElemType;
		bool m_bEnableApplyToAll;

		void EnableApplyToAll ();
		void OnBtnApplyToAll ();

		virtual RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM& v_AccessCurSelectParmItem ();
		virtual ELEMTYPE v_GetCurElemType () const;
		virtual void v_OnUpdateCurSelectParmItem ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_SELECT_ELEMENTS_FIXEDTYPE::STYLE)
#endif
	
//=====================================================================================================================

//! Form to select elements of user-chosen type.
class FORM_SELECT_ELEMENTS_USERTYPE : public FORM_SELECT_ELEMENTS_SINGLETYPE {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_USERTYPE ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_USERTYPE ();
	
		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			RVC::OBJTYPE ObjType,
			const ELEMTYPESET &ValidTypes,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);
			
		const RVC::GEOMETRIC::ELEMSELECTPARMS& GetElemSelectParms ();
			
		void SetElemSelectParms (
			const RVC::GEOMETRIC::ELEMSELECTPARMS &parms
			);
			
		void SetElemTypeSet (
			const ELEMTYPESET &ValidTypes
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_ELEMENTS_USERTYPE THISCLASS;

		MGUI::CTRL_COMBOBOX m_CbxElemType;
		RVC::GEOMETRIC::ELEMSELECTPARMS m_SelectParms;
		ELEMTYPE m_ElemType;
		RVC::OBJTYPE m_ObjType;

		void OnCbxElemType ();

		virtual RVC::GEOMETRIC::ELEMSELECTPARMS::ITEM& v_AccessCurSelectParmItem ();
		virtual ELEMTYPE v_GetCurElemType () const;
		virtual void v_OnUpdateCurSelectParmItem ();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Form with options to select elements from geometric object.
class FORM_SELECT_ELEMENTS_GEOMETRIC : public MGUI::FORM_COMPOSITE {
	public:

		enum STYLE {
			STYLE_Default =							0,
			STYLE_NoExtents =							0x01,
			STYLE_AddExtentsApplyToAllBtn =		0x02,
			STYLE_AllowSeparation =					0x04,
			STYLE_AllowExtentsByPolygons =		0x08,
			STYLE_AddSelectionApplyToAllBtn =	0x10,
			};
				
		//! Constructor.
		explicit FORM_SELECT_ELEMENTS_GEOMETRIC (STYLE StyleAllowed = STYLE_Default);

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_GEOMETRIC ();
			
		void Clear ();
	
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			RVC::OBJTYPE DestObjType,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			STYLE style = STYLE_Default
			);

		void EnableApplySelectionToAll (
			bool bEnable
			) { v_EnableApplySelectionToAll(bEnable); }
			
		const RVC::GEOMETRIC::ELEMSELECTPARMS& GetElemSelectParms ();

		const OBJITEM& GetObjItem () const
			{ return (m_ObjItem); }
			
		void SetDelegateOnApplyRegionToAll (
			DELEGATE_VOID_NOPARMS delegate
			);

		void SetDelegateOnApplySelectionToAll (
			DELEGATE_VOID_ELEMTYPE delegate
			) { v_SetDelegateOnApplySelectionToAll(delegate); }
			
		void SetElemSelectParms (
			const RVC::GEOMETRIC::ELEMSELECTPARMS &parms
			);
			
		void SetObject (
			const RVC::OBJITEM &ObjItem
			);

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::GEOMETRIC::ELEMSELECTPARMS& AccessSelectParms ()
			{ return (m_SelectParms); }

		const RVC::GEOMETRIC::ELEMSELECTPARMS& GetSelectParms () const
			{ return (m_SelectParms); }

		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_ELEMENTS_GEOMETRIC THISCLASS;
		class FILTER_EXTENTS_POLYGONS;

		MGUI::CTRL_GROUPBOX m_GbxElements;
		MGUI::CTRL_GROUPBOX m_GbxExtents;
		MGUI::CTRL_COMBOBOX m_CbxExtentsMode;
		MGUI::CTRL_TOGGLEBUTTON m_TglAddBorder;
		MGUI::FORM_BOOK m_BookExtents;
		MGUI::CTRL_PUSHBUTTON m_BtnRegionSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnRegionApplyToAll;
		MGUI::FORM_BUTTONTEXT m_BtxExtentsPolygons;
		MGUI::CTRL_GROUPBOX m_GbxSeparation;
		MGUI::CTRL_COMBOBOX m_CbxSeparate;
		MGUI::FORM_BUTTONTEXT m_BtxSeparateAttribute;
		MGUI::FORM_EDIT_STRING m_EditSeparatePrefix;
		MGUI::FORM_EDIT_STRING m_EditSeparateSuffix;
		MGUI::CTRL_TOGGLEBUTTON m_TglSeparateEachToOwnFile;

		RVC::OBJITEM m_ObjItem;
		RVC::OBJTYPE m_DestObjType;
		RVC::GEOMETRIC::ELEMSELECTPARMS m_SelectParms;
		STYLE m_StyleAllowed;

		void OnBtnRegionSelect ();
		void OnBtxExtentsPolygons ();
		void OnBtxSeparateAttribute ();
		void OnCbxExtentsMode ();
		void OnCbxSeparate ();
		void OnTglAddBorder ();
		ERRVALUE PromptExtentsPolygons (GEOMETRIC::ELEMSELECTPARMS::SEPARATION &SeparationValues);
		ERRVALUE PromptSeparateAttribute (const OBJITEM &ObjItemGeometric, ELEMTYPE ElemType, GEOMETRIC::ELEMSELECTPARMS::SEPARATION &values);
		void UpdateCtrlsSeparate ();

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq) = 0;
		virtual void v_EnableApplySelectionToAll (bool bEnable) = 0;
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms) = 0;
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate) = 0;
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms) = 0;
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem) = 0;
		virtual void v_SetupCbxSeparate (MGUI::CTRL_COMBOBOX &cbx) const;
	
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_SELECT_ELEMENTS_GEOMETRIC::STYLE)
#endif
	
//=====================================================================================================================

//! Form to select elements from vector object.	
class FORM_SELECT_ELEMENTS_VECTOR : public FORM_SELECT_ELEMENTS_GEOMETRIC {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_VECTOR ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_VECTOR ();
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_ELEMENTS_VECTOR THISCLASS;

		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Point;
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Line;
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Polygon;
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Label;

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq);
		virtual void v_EnableApplySelectionToAll (bool bEnable);
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate);
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem);
		virtual void v_SetupCbxSeparate (MGUI::CTRL_COMBOBOX &cbx) const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Form to select elements from CAD object.	
class FORM_SELECT_ELEMENTS_CAD : public FORM_SELECT_ELEMENTS_GEOMETRIC {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_CAD ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_CAD ();
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Element;

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq);
		virtual void v_EnableApplySelectionToAll (bool bEnable);
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate);
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Form to select elements from TIN object.	
class FORM_SELECT_ELEMENTS_TIN : public FORM_SELECT_ELEMENTS_GEOMETRIC {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_TIN ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_TIN ();
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Node;
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Edge;
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Triangle;

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq);
		virtual void v_EnableApplySelectionToAll (bool bEnable);
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate);
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Form to select elements from shape object.	
class FORM_SELECT_ELEMENTS_SHAPE : public FORM_SELECT_ELEMENTS_GEOMETRIC {
	public:
	
		//! Constructor.
		FORM_SELECT_ELEMENTS_SHAPE ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_SHAPE ();
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Element;

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq);
		virtual void v_EnableApplySelectionToAll (bool bEnable);
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate);
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Form to select elements from region object.	
class FORM_SELECT_ELEMENTS_REGION : public FORM_SELECT_ELEMENTS_GEOMETRIC {
	public:

		//! Constructor.
		FORM_SELECT_ELEMENTS_REGION ();

		//! Destructor.
		virtual ~FORM_SELECT_ELEMENTS_REGION ();
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORM_SELECT_ELEMENTS_FIXEDTYPE m_Element;

		virtual void v_CreateElementControls (MGUI::LAYOUT_PANE_BASE &ParentPane, STYLE style, MGUI::LAYOUT_EQUALIZER &eq);
		virtual void v_EnableApplySelectionToAll (bool bEnable);
		virtual void v_GetElemSelectParms (RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetDelegateOnApplySelectionToAll (DELEGATE_VOID_ELEMTYPE delegate);
		virtual void v_SetElemSelectParms (const RVC::GEOMETRIC::ELEMSELECTPARMS &parms);
		virtual void v_SetObject (const RVC::OBJITEM &ObjItem);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

//! Form to select geometric object(s) and element extraction options.	
class FORM_GEOMETRIC_EXTRACT : public MGUI::FORM_COMPOSITE {
	public:

		typedef fastdelegate::FastDelegate<void(const OBJITEM&)> DELEGATE_OBJITEM;
	
		//! Constructor.
		FORM_GEOMETRIC_EXTRACT ();

		//! Destructor.
		virtual ~FORM_GEOMETRIC_EXTRACT ();
			
		void Clear ();
	
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,							//!< Parent pane
			const RVC::OBJTYPESET &GeometricSet,						//!< Set of geometric objects to work on
			RVC::OBJTYPE DestObjType,
			const MISTRING &prompt,											//!< Prompt string
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,
			RVC::GETOBJFILTER *pGetObjFilter = 0,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			FORM_SELECT_ELEMENTS_GEOMETRIC::STYLE style = FORM_SELECT_ELEMENTS_GEOMETRIC::STYLE_Default
			);

		void CreateTglClearAfterRun ();
			
		const MILIST<RVC::GEOMETRIC::ELEMSELECTPARMS>& GetElemSelectParms ();
			
		const RVC::OBJITEMLIST& GetGeometricList () const
			{ return (m_GeometricList.GetObjectList()); }

		bool GetClearAfterRun () const
			{ return (m_TglClearAfterRun.GetValue()); }

		void SetDelegateOnObjectAdded (
			DELEGATE_OBJITEM delegate
			) { m_DelegateOnObjectAdded = delegate; }

		void SetDelegateOnObjectSelected (
			DELEGATE_OBJITEM delegate
			) { m_DelegateOnObjectSelected = delegate; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_GEOMETRIC_EXTRACT THISCLASS;

		RVC::UI::FORM_OBJECT_LIST m_GeometricList;
		MGUI::CTRL_TOGGLEBUTTON m_TglClearAfterRun;
		FORM_SELECT_ELEMENTS_VECTOR m_FormVector;
		FORM_SELECT_ELEMENTS_CAD m_FormCAD;
		FORM_SELECT_ELEMENTS_TIN m_FormTIN;
		FORM_SELECT_ELEMENTS_SHAPE m_FormShape;
		FORM_SELECT_ELEMENTS_REGION m_FormRegion;
		FORM_SELECT_ELEMENTS_GEOMETRIC *m_pFormActive;
		
		MILIST<RVC::GEOMETRIC::ELEMSELECTPARMS> m_SelectParms;
		INT32 m_CurItem;
		DELEGATE_OBJITEM m_DelegateOnObjectAdded;
		DELEGATE_OBJITEM m_DelegateOnObjectSelected;

		ERRVALUE ApplySelectionToAll (ELEMTYPE ElemType);
		void OnObjectAdded (const RVC::OBJITEM &ObjItem, INT32 ItemNum);
		void OnObjectRemoved (const RVC::OBJITEM &ObjItem, INT32 ItemNum);
		void OnObjectSelected (const RVC::OBJITEM &ObjItem, INT32 ItemNum);
		void OnObjectsRemoved ();
		void OnSelectionApplyToAll (ELEMTYPE ElemType);
		void OnTglClearAfterRun ();

		void SetRegionForAllObjects ();
		void UpdateCurItemValuesFromForm ();
		void UpdateFormFromCurItemValues ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
	
}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMSELECTELEMENTS_H
