/**
 * \file rvc/uiformtablefield.h
 * \brief RVC::UI::FORM_TABLEFIELD definitions
 *
 * \if NODOC
 * $Id: uiformtablefield.h_v 1.2 2014/03/18 12:59:02 mju Exp $
 *
 * $Log: uiformtablefield.h_v $
 * Revision 1.2  2014/03/18 12:59:02  mju
 * Add style_OnlyValidTables.
 *
 * Revision 1.1  2013/11/13 22:24:57  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMTABLEFIELD_H
#define	INC_RVC_UIFORMTABLEFIELD_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! Form for database table and field selection.
class FORM_TABLEFIELD : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =				0,
			STYLE_AllowNone =				0x01,		//!< Include "none" as choice for table
			STYLE_NoInternal =			0x02,		//!< Don't show "internal" tables
			STYLE_OnlyValidTables =		0x04,		//!< Only show valid tables in list
			};

		//! Delegate to determine if table or field is valid on selection or list population.
		//! Parameters are table or field Name and indicator of whether is from user selection (true) or for list creation (false).
		//! Must return true if valid, false if not.
		typedef fastdelegate::FastDelegate<bool(const MISTRING&, bool)> DELEGATE_SELECT_VALID;

		//! Constructor.
		FORM_TABLEFIELD ();

		//! Destructor.
		virtual ~FORM_TABLEFIELD ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& paneParent,	//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign,	//!< Sizing/alignment
			int listrows = 6,							//!< Number of rows in list
			STYLE style = STYLE_Default			//!< Form style
			);

		//! Get selected field name.
		const MISTRING& GetFieldName () const;

		//! Get selected table and field as single string.
		//! Returned string will be in form table.field with names in backquotes if contain spaces or other special characters.
		MISTRING GetTableField () const;

		//! Get selected table name.
		const MISTRING& GetTableName () const;

		//! Set database.
		void SetDatabase (
			const RVC::OBJITEM& ObjItemDb			//!< Database location
			);

		//! Set delegate for field list population and selection
		//! If no delegate is specified all fields will be selectable.
		void SetDelegateField (
			DELEGATE_SELECT_VALID delegate
			);

		//! Set delegate for table list population and selection
		void SetDelegateTable (
			DELEGATE_SELECT_VALID delegate
			);

		//! Set table and field.
		void SetTableField (
			const MISTRING& tablefield				//!< String in form of table.field
			);

		//! Set table and field.
		void SetTableField (
			const MISTRING& TableName,				//!< Table name
			const MISTRING& FieldName				//!< Field name
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_TABLEFIELD::STYLE)
#endif

//=====================================================================================================================

} // End namespace UI
} // End namespace RVC

#endif	// INC_RVC_UIFORMTABLEFIELD_H
