/**
 * \file rvc/uiimagepyramid.h
 * \brief RVC::UI::FORM_IMAGE_PYRAMID class.
 *
 * \if NODOC
 * $Id: uiimagepyramid.h_v 1.7 2015/03/03 18:21:14 mju Exp $
 *
 * $Log: uiimagepyramid.h_v $
 * Revision 1.7  2015/03/03 18:21:14  mju
 * Add create with label.
 *
 * Revision 1.6  2015/02/06 15:49:02  mju
 * Add get/setPyramid, iniRead/Write.
 * Deprecate get/setValues.
 *
 * Revision 1.5  2008/09/02 21:43:25  mju
 * Fix doxygen warning.
 *
 * Revision 1.4  2007/07/18 16:10:18  mju
 * Don't pass flags by ref in setvalues.
 *
 * Revision 1.2  2006/06/28 15:35:41  mju
 * Rename as form_image_pyramid.
 *
 * Revision 1.1  2006/06/23 14:37:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIIMAGEPYRAMID_H
#define	INC_RVC_UIIMAGEPYRAMID_H

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Form for setting IMAGE::PYRAMID values.
class FORM_IMAGE_PYRAMID : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_IMAGE_PYRAMID ();

		//! Destructor.
		virtual ~FORM_IMAGE_PYRAMID ();

		//! Create form with default label.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane				//!< Parent pane
			);

		//! Create form with specified label.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,				//!< Parent pane
			const MISTRING &label								//!< Label
			);

		//! Get label control.
		//! Should only be used for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel ()
			{ return (m_CbxPyramid.GetLabel()); }

		//! Get pyramid mode.
		RVC::IMAGE::PYRAMID GetPyramid () const
			{ return (static_cast<RVC::IMAGE::PYRAMID>(m_CbxPyramid.GetSelectedItemID())); }

		//! Get current values.
		//! \deprecated use GetPyramid instead.
		DEPRECATED void GetValues (
			bool &DoPyramid,										//!< Return, true if should make pyramid tiers, false if not
			RVC::IMAGE::PYRAMID &mode							//!< Return, pyramid mode
			);

		//! Read values from initialization file.
		void IniRead (
			INIHANDLE inihnd,
			const char *IniGroup,
			const char *IniField,
			bool notify = false
			);

		//! Write values to initialization file.
		void IniWrite (
			INIHANDLE inihnd,
			const char *IniGroup,
			const char *IniField
			) const;

		//! Set delegate to call on any change.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_CbxPyramid.SetDelegateOnSelection(delegate); }

		//! Set pyramid mode.
		void SetPyramid (
			RVC::IMAGE::PYRAMID pyramid,						//!< Pyramid mode
			bool notify												//!< Whether to notify delegate if changed
			);

		//! Set values.
		//! \deprecated use SetPyramid or IniRead instead.
		DEPRECATED void SetValues (
			bool DoPyramid,										//!< True if should make pyramid tiers, false if not
			RVC::IMAGE::PYRAMID mode,							//!< Pyramid mode
			bool notify												//!< Whether to notify delegate if changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::FORM_COMBOBOX m_CbxPyramid;

		FORM_IMAGE_PYRAMID (const FORM_IMAGE_PYRAMID &);
		FORM_IMAGE_PYRAMID& operator= (const FORM_IMAGE_PYRAMID &);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIIMAGEPYRAMID_H
