/**
 * \file rvc/uirecordpopulate.h
 * \brief User Interface for OP::RECORDPOPULATE
 *
 * \if NODOC
 * $Id: uirecordpopulate.h_v 1.1 2013/11/22 22:33:26 mju Exp $
 *
 * $Log: uirecordpopulate.h_v $
 * Revision 1.1  2013/11/22 22:33:26  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIRECORDPOPULATE_H
#define	INC_RVC_UIRECORDPOPULATE_H

#ifndef	INC_RVC_OPRECORDPOPULATE_H
#include <rvc/oprecordpopulate.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

class FORM_RECORDPOPULATE : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_RECORDPOPULATE ();

		//! Destructor.
		virtual ~FORM_RECORDPOPULATE ();

		//! Create form.
		//! The form will retain and update the specified RECORDPOPULATE instance, which must continue to exist
		//! for the life of the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& paneParent,			//!< Parent pane for form
			OP::RECORDPOPULATE& RecordPopulate			//!< Associated RECORDPOPULATE instance, must be initialized
			);

		//! Set delegate to call when the user makes a change.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

//=====================================================================================================================

//! Dialog to choose and manage settings for record population.
class DLG_RECORDPOPULATE : public MGUI::DLGSHELL {
	public:

		//! Constructor.
		DLG_RECORDPOPULATE ();

		//! Destructor.
		virtual ~DLG_RECORDPOPULATE ();

		//! Access RECORDPOPULATE service instance.
		OP::RECORDPOPULATE& AccessService ()
			{ return (m_RecordPopulate); }

		//! Initialize service instance, must be called before dialog created.
		ERRVALUE Initialize (
			const DBTABLE& ObjTable					//!< Table with records to be populated
			) { return (m_RecordPopulate.Initialize(ObjTable)); }

		//! Set delegate for creating custom controls at bottom of dialog.
		void SetDelegateCreateCustomBottom (
			MGUI::DELEGATE_CREATECUSTOMCONTROLS delegate
			) { m_DelegateCreateCustomBottom = delegate; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLGSHELL BASECLASS;
		typedef DLG_RECORDPOPULATE THISCLASS;

		RVC::OP::RECORDPOPULATE m_RecordPopulate;
		FORM_RECORDPOPULATE m_Form;
		MGUI::DELEGATE_CREATECUSTOMCONTROLS m_DelegateCreateCustomBottom;

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIRECORDPOPULATE_H
