/**
 * \file spatmod/imagefilteraddalpha.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_ADDALPHA definition.
 *
 * \if NODOC
 * $Id: imagefilteraddalpha.h_v 1.1 2016/06/22 21:28:32 mju Exp $
 *
 * $Log: imagefilteraddalpha.h_v $
 * Revision 1.1  2016/06/22 21:28:32  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERADDALPHA_H
#define	INC_SPATMOD_IMAGEFILTERADDALPHA_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLE_H
#include <spatmod/imagesample.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to add or adjust alpha values.
//! This filter will add an alpha/opacity component or adjust the (first) existing alpha components by multiplying
//! the specified AlphaValue. If the source does not contain any alpha components, one will be added of the specified
//! data type. In that case, if an integer data type is specified, the alpha value will be scaled so that 1.0
//! corresponds to the maximum possible value.
class FILTER_ADDALPHA : public FILTER {
	public:

		//! Constructor.
		FILTER_ADDALPHA (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			double AlphaValue,						//!< Alpha value to add or multiply by (0 to 1)
			DATATYPE AlphaType = DATATYPE_UINT8	//!< Data type for Alpha/Opacity sample if does not already exist
			);

		// Destructor.
		virtual ~FILTER_ADDALPHA ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_AlphaValue;
		SAMPLE m_AlphaSampleValue;
		UINT16 m_AlphaSampleIdx;
		bool m_bSrcHasAlpha;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_ADDALPHA (const FILTER_ADDALPHA&);
		FILTER_ADDALPHA& operator= (const FILTER_ADDALPHA&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERADDALPHA_H

