/**
 * \file spatmod/imagefilterfourier.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_FOURIER definition.
 *
 * \if NODOC
 * $Id: imagefilterfourier.h_v 1.3 2015/09/08 19:00:25 mju Exp $
 *
 * $Log: imagefilterfourier.h_v $
 * Revision 1.3  2015/09/08 19:00:25  mju
 * Redesign to use fft modes.
 *
 * Revision 1.2  2015/09/01 17:04:59  mju
 * Define enum op.
 *
 * Revision 1.1  2015/08/28 17:49:08  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERFOURIER_H
#define	INC_SPATMOD_IMAGEFILTERFOURIER_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_FFT_H
#include <mi32/fft.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to to perform fourier transform on image components.
//! Source image may consist of either "real" components or pairs of "complex" components.
//! Any "opacity" components will be excluded from result.
class FILTER_FOURIER : public FILTER {
	public:

		enum FLAGS {
			FLAG_None =				0,
			FLAG_Normalize =		0x01,		//!< Normalize output
			FLAG_ShiftOrigin =	0x02,		//!< Shift origin (DC level) to/from center for forward/reverse
			};

		//! Constructor.
		FILTER_FOURIER (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			FFT::MODE fftmode,						//!< Mode to control direction and result
			FLAGS flags = FLAG_None					//!< Flags
			);

		//! Destructor.
		virtual ~FILTER_FOURIER ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FFT m_fft;
		FFT::MODE m_Mode;
		FLAGS m_Flags;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &TgtIterator);
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_FOURIER (const FILTER_FOURIER &);
		FILTER_FOURIER& operator= (const FILTER_FOURIER &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILTER_FOURIER::FLAGS)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERFOURIER_H
