/**
 * \file spatmod/imagefilterlens.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LENS definition.
 *
 * \if NODOC
 * $Id: imagefilterlens.h_v 1.3 2015/12/22 23:45:27 bcolombe Exp $
 *
 * $Log: imagefilterlens.h_v $
 * Revision 1.3  2015/12/22 23:45:27  bcolombe
 * Fix inclusion guard name
 *
 * Revision 1.2  2015/08/20 21:13:51  mju
 * Remove cropFactor parm from ctor, add method to set instead.
 *
 * Revision 1.1  2015/08/07 21:22:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLENS_H
#define	INC_SPATMOD_IMAGEFILTERLENS_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct lfLens;
struct lfModifier;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base class for lens correction and adjustment filters.
class FILTER_LENS : public FILTER {
	public:

		// Destructor.
		virtual ~FILTER_LENS () = 0;

		//! Set crop factor if different from lens.
		void SetCropFactor (
			float CropFactor
			) { m_CropFactor = CropFactor; }

	protected:

		//! Construct.
		FILTER_LENS (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const lfLens *lfLens,					//!< Lens data
			float FocalLengthMM,						//!< Focal length in millimeters
			float Aperture,							//!< Aperture (F-number)
			float FocusDistanceM						//!< Approximate focus distance in meters
			);

		const lfModifier* GetModifier () const
			{ return (m_lfModifier); }

		int GetModifyFlagsUsed () const
			{ return (m_ModifyFlagsUsed); }

		ERRVALUE InitModifier (int ModifyFlags, float scale = 1, int TgtGeometry = 0);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const lfLens *m_lfLens;
		lfModifier *m_lfModifier;
		float m_CropFactor;
		float m_FocalLengthMM;
		float m_Aperture;
		float m_FocusDistanceM;
		int m_ModifyFlagsUsed;

		// Unimplemented
		FILTER_LENS (const FILTER_LENS &);
		FILTER_LENS& operator= (const FILTER_LENS &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLENS_H

