/**
 * \file spatmod/imagefilterlensvignetting.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LENS_VIGNETTING definition.
 *
 * \if NODOC
 * $Id: imagefilterlensvignetting.h_v 1.2 2015/08/20 21:14:32 mju Exp $
 *
 * $Log: imagefilterlensvignetting.h_v $
 * Revision 1.2  2015/08/20 21:14:32  mju
 * Remove crop factor parm from ctor.
 *
 * Revision 1.1  2015/08/07 21:22:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLENSVIGNETTING_H
#define	INC_SPATMOD_IMAGEFILTERLENSVIGNETTING_H

#ifndef	INC_SPATMOD_IMAGEFILTERLENS_H
#include <spatmod/imagefilterlens.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform lens vignetting correction.
class FILTER_LENS_VIGNETTING : public FILTER_LENS {
	public:

		//! Construct.
		FILTER_LENS_VIGNETTING (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const lfLens *lfLens,					//!< Lens data
			float FocalLengthMM,						//!< Focal length in millimeters
			float Aperture,							//!< Aperture (F-number)
			float FocusDistanceM						//!< Approximate focus distance in meters
			);

		// Destructor.
		virtual ~FILTER_LENS_VIGNETTING ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT8> m_WorkBuf;
		int m_CompRole;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);

		// Unimplemented
		FILTER_LENS_VIGNETTING (const FILTER_LENS_VIGNETTING &);
		FILTER_LENS_VIGNETTING& operator= (const FILTER_LENS_VIGNETTING &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLENSVIGNETTING_H

