/**
 * \file imagefilterreflectance.h <spatmod/imagefilterreflectance.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_REFLECTANCE definitions.
 *
 * \if NODOC
 * $Id: imagefilterreflectance.h_v 1.4 2015/12/18 21:12:26 bcolombe Exp $
 *
 * $Log: imagefilterreflectance.h_v $
 * Revision 1.4  2015/12/18 21:12:26  bcolombe
 * Remove dark object, just apply to radiance input
 *
 * Revision 1.3  2015/10/19 16:04:52  bcolombe
 * Change to flags
 *
 * Revision 1.2  2015/08/31 19:34:52  bcolombe
 * Add darkobject and cost
 *
 * Revision 1.1  2015/08/17 17:06:53  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERREFLECTANCE_H
#define	INC_SPATMOD_IMAGEFILTERREFLECTANCE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef INC_MI32_DATETIME_H
#include <mi32/datetime.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;
class SAMPLEITERATOR;

//=====================================================================================================================

//! Filter to compute top-of atmosphere reflectance (apparent reflectance)
//! Requires previous stage of spectral radiance
class FILTER_REFLECTANCE : public FILTER {
	public:
		enum FLAGS {
			FLAG_None =					0,
			FLAG_ApplyCOST =			0x01
			};

		//! Constructor.
		FILTER_REFLECTANCE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			double SolarIrradiance,
			const DATETIME& DateOfAcquisition,
			double SolarElevationAngle,
			FLAGS flags = FLAG_None
			);

		// Destructor.
		virtual ~FILTER_REFLECTANCE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DATETIME m_DateOfAcquisition;
		double m_SolarIrradiance;
		double m_SolarElevationAngle;
		FLAGS m_Flags;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_REFLECTANCE (const FILTER_REFLECTANCE&);
		FILTER_REFLECTANCE& operator= (const FILTER_REFLECTANCE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILTER_REFLECTANCE::FLAGS)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERREFLECTANCE_H

