/**
 * \file spatmod/imagefiltertopographiccorrection.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_TOPOGRAPHIC_CORRECTION definitions.
 *
 * \if NODOC
 * $Id: imagefiltertopographiccorrection.h_v 1.2 2016/04/25 16:31:47 mju Exp $
 *
 * $Log: imagefiltertopographiccorrection.h_v $
 * Revision 1.2  2016/04/25 16:31:47  mju
 * Add surfaceFraction member.
 *
 * Revision 1.1  2015/11/20 23:21:48  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERTOPOGRAPHICCORRECTION_H
#define	INC_SPATMOD_IMAGEFILTERTOPOGRAPHICCORRECTION_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to compute topographic correction
class FILTER_TOPOGRAPHIC_CORRECTION : public FILTER {
	public:

		//! Constructor.
		FILTER_TOPOGRAPHIC_CORRECTION (
			STAGE *pImageStage,
			bool TakeOwnershipImage,
			STAGE *pDEMStage,
			bool TakeOwnershipDEM,
			double SunElevationAngle,
			double SunDirectionAngle,
			double SkylightFraction
			);

		// Destructor.
		virtual ~FILTER_TOPOGRAPHIC_CORRECTION ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_SunElevationAngle;
		double m_SunDirectionAngle;
		double m_SurfaceFraction;
		double m_SkylightFraction;

		typedef FILTER BASECLASS;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_TOPOGRAPHIC_CORRECTION (const FILTER_TOPOGRAPHIC_CORRECTION&);
		FILTER_TOPOGRAPHIC_CORRECTION& operator= (const FILTER_TOPOGRAPHIC_CORRECTION&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERTOPOGRAPHICCORRECTION_H

