/**
 * \file <spatmod/imagefilterwindow.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_WINDOW definition.
 *
 * \if NODOC
 * $Id: imagefilterwindow.h_v 1.1 2015/09/14 20:09:08 mju Exp $
 *
 * $Log: imagefilterwindow.h_v $
 * Revision 1.1  2015/09/14 20:09:08  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERWINDOW_H
#define	INC_SPATMOD_IMAGEFILTERWINDOW_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_WINDOWFUNCTION_H
#include <mi32/windowfunction.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to "window" image.
class FILTER_WINDOW : public FILTER {
	public:

		//! Construct with specified window type.
		//! The size of the window will match the size of the image in each dimension.
		FILTER_WINDOW (
			STAGE *pInputImage,					//!< Input image stage
			bool TakeOwnership,					//!< Filter takes ownership of source stage and will delete when needed
			WINDOWFUNCTION::TYPE type,			//!< Function type
			WINDOWFUNCTION::SUBTYPE subtype	//!< Function subtype
			);

		//! Construct with horizontal/vertical arrays.
		FILTER_WINDOW (
			STAGE *pInputImage,			//!< Input image stage
			bool TakeOwnership,			//!< Filter takes ownership of source stage and will delete when needed
			const double *WinH,			//!< Horizontal array, must have one entry for each input image column
			const double *WinV			//!< Vertical array, must have one entry for each input image row
			);

		// Destructor.
		virtual ~FILTER_WINDOW ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<double> m_WinH;
		DOUBLE_ARRAY<double> m_WinV;

		// STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);

		// Unimplemented
		FILTER_WINDOW (const FILTER_WINDOW&);
		FILTER_WINDOW& operator= (const FILTER_WINDOW&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERWINDOW_H

