/**
 * \file spatmod/imagepassthrustatunivariate.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::PASSTHRU definition.
 *
 * \if NODOC
 * $Id: imagepassthrustatunivariate.h_v 1.1 2017/03/15 18:22:39 mju Exp $
 *
 * $Log: imagepassthrustatunivariate.h_v $
 * Revision 1.1  2017/03/15 18:22:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEPASSTHRUSTATUNIVARIATE_H
#define	INC_SPATMOD_IMAGEPASSTHRUSTATUNIVARIATE_H

#ifndef	INC_SPATMOD_IMAGEPASSTHRU_H
#include <spatmod/imagepassthru.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class STAT_UNIVARIATE;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pass-through stage to compute univariate statistics.
class PASSTHRU_STATUNIVARIATE : public PASSTHRU {
	public:

		//! Construct with specified input stage and ownership.
		PASSTHRU_STATUNIVARIATE (
			STAGE *pInputStage,						//!< Pointer to input stage
			bool bTakeOwnership						//!< Whether to take ownership of input stage and will delete when needed
			);

		// Destructor.
		virtual ~PASSTHRU_STATUNIVARIATE ();

		//! Set bin info.
		//! This must be called before Initialize for any samples having more than 16 bits.
		void SetBinInfo (
			UINT16 SampleIdx,							//!< Sample index
			UINT32 NumBins,							//!< Number of bins, if 0 will determine from bin range and width
			double MinBin,								//!< Minimum bin value
			double MaxBin,								//!< Maximum bin value
			double BinWidth = 0						//!< Bin width, if 0 will determine from number of bins and range
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef PASSTHRU BASECLASS;

		struct BININFO {
			double m_MinBin;
			double m_MaxBin;
			double m_BinWidth;
			UINT32 m_NumBins;

			BININFO ();
			bool IsValid () const;
			void Set (UINT32 NumBins, double MinBin, double MaxBin, double BinWidth)
				{ m_NumBins = NumBins; m_MinBin = MinBin; m_MaxBin = MaxBin; m_BinWidth = BinWidth; }
			};

		BININFO *m_BinInfo;
		STAT_UNIVARIATE *m_Stats;
		STATSVALIDITY m_StatsValidity;

		const char* v_GetTypeName () const;
		ERRVALUE v_Initialize ();
		ERRVALUE v_OnPullBegin (const SECTION_ITERATOR &iterator);
		ERRVALUE v_OnPullEnd ();
		ERRVALUE v_OnPullSubSection (const SECTION &SubSection, const BUFFER &buffer);

		// Unimplemented
		PASSTHRU_STATUNIVARIATE (const PASSTHRU_STATUNIVARIATE &);
		PASSTHRU_STATUNIVARIATE& operator= (const PASSTHRU_STATUNIVARIATE &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEPASSTHRUSTATUNIVARIATE_H

