/**
 * \file spatmod/imagesourcexpixmap.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_XPIXMAP definition.
 *
 * \if NODOC
 * $Id: imagesourcexpixmap.h_v 1.1 2013/07/03 14:10:42 mju Exp $
 *
 * $Log: imagesourcexpixmap.h_v $
 * Revision 1.1  2013/07/03 14:10:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEXPIXMAP_H
#define	INC_SPATMOD_IMAGESOURCEXPIXMAP_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	_XLIB_H_
	#include <X11/Xlib.h>
#endif


namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pipeline source for reading X Pixmap.
class SOURCE_XPIXMAP : public SOURCE {
	public:

		//! Construct with X Display/Pixmap.
		SOURCE_XPIXMAP (
			Display *pxDisplay,
			Pixmap xPixmap
			);

		//! Destructor.
		virtual ~SOURCE_XPIXMAP ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		Display *m_display;
		Pixmap m_pixmap;
		XImage *m_ximage;
		UINT8 *m_data;
		INT32 m_step;
		INT32 m_offset[3];

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_XPIXMAP (const SOURCE_XPIXMAP&);
		SOURCE_XPIXMAP& operator= (const SOURCE_XPIXMAP&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEXPIXMAP_H
