/**
 * \file spatmod/imagetargetcalcmultilinear.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_CALC_MULTILINEAR definition.
 *
 * \if NODOC
 * $Id: imagetargetcalcmultilinear.h_v 1.2 2015/01/05 15:08:40 mju Exp $
 *
 * $Log: imagetargetcalcmultilinear.h_v $
 * Revision 1.2  2015/01/05 15:08:40  mju
 * Change type of statCells.
 *
 * Revision 1.1  2014/12/31 16:57:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETNULL_H
#define	INC_SPATMOD_IMAGETARGETNULL_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif

namespace SPATMOD {
namespace IMAGE {

#ifndef GENERATING_DOXYGEN_OUTPUT
class SAMPLEITERATOR_CONST;
#endif

//=====================================================================================================================

//! Target to calculate multilinear regression coefficients and offsets.
class TARGET_CALC_MULTILINEAR : public TARGET {
	public:

		//! Construct target with specified input stage
		explicit TARGET_CALC_MULTILINEAR (
			STAGE *pInputStage							//!< Input stage
			);

		//! Destructor.
		virtual ~TARGET_CALC_MULTILINEAR ();

		//! Get coefficients.
		const MIMATRIX& GetCoefficients () const
			{ return (m_Coefficients); }

		//! Get offsets.
		const MIMATRIX& GetOffsets () const
			{ return (m_Offsets); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MIMATRIX m_Coefficients;
		MIMATRIX m_Offsets;
		DOUBLE_ARRAY<double> m_sum;
		DOUBLE_ARRAY<double> m_sum2;
		SIMPLE_ARRAY<const double *> m_pValues;
		DOUBLE_ARRAY<double> m_values;
		SAMPLEITERATOR_CONST *m_itValues;
		UINT64 m_StatCells;
		bool m_bAllDouble;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION &SubSection, const BUFFER &buffer);

		// Unimplemented
		TARGET_CALC_MULTILINEAR (const TARGET_CALC_MULTILINEAR &);
		TARGET_CALC_MULTILINEAR& operator= (const TARGET_CALC_MULTILINEAR &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETNULL_H

