/**
 * \file spatmod/imagetargetopencvmat.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_OPENCVMAT definition.
 *
 * \if NODOC
 * $Id: imagetargetopencvmat.h_v 1.1 2015/10/06 17:33:10 mju Exp $
 *
 * $Log: imagetargetopencvmat.h_v $
 * Revision 1.1  2015/10/06 17:33:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETOPENCVMAT_H
#define	INC_SPATMOD_IMAGETARGETOPENCVMAT_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace cv {
	class Mat;
	}
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! OpenCV "cv::Mat" target.
class TARGET_OPENCVMAT : public TARGET {
	public:

		//! Construct target with specified input stage
		explicit TARGET_OPENCVMAT (
			STAGE *pInputStage							//!< Stage in pipeline to obtain image data from
			);

		//! Destructor.
		virtual ~TARGET_OPENCVMAT ();

		//! Get reference to image pixels.
		const cv::Mat& GetImage () const
			{ return (*m_pImage); }

		//! Get reference to mask.
		const cv::Mat& GetMask () const
			{ return (*m_pMask); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		cv::Mat *m_pImage;
		cv::Mat *m_pMask;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// Unimplemented
		TARGET_OPENCVMAT (const TARGET_OPENCVMAT&);
		TARGET_OPENCVMAT& operator= (const TARGET_OPENCVMAT&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETOPENCVMAT_H

